@@ -19,7 +19,7 @@
  * by Tk_GetReliefFromObj.
  */
 
-static CONST char *reliefStrings[] = {
+static const char *const reliefStrings[] = {
     "flat", "groove", "raised", "ridge", "solid", "sunken", NULL
 };
 
@@ -30,6 +30,7 @@ static CONST char *reliefStrings[] = {
 static void		BorderInit(TkDisplay *dispPtr);
 static void		DupBorderObjProc(Tcl_Obj *srcObjPtr,
 			    Tcl_Obj *dupObjPtr);
+static void		FreeBorderObj(Tcl_Obj *objPtr);
 static void		FreeBorderObjProc(Tcl_Obj *objPtr);
 static int		Intersect(XPoint *a1Ptr, XPoint *a2Ptr,
 			    XPoint *b1Ptr, XPoint *b2Ptr, XPoint *iPtr);
@@ -45,7 +46,7 @@ static void		ShiftLine(XPoint *p1Ptr, XPoint *p2Ptr,
  * is set.
  */
 
-Tcl_ObjType tkBorderObjType = {
+const Tcl_ObjType tkBorderObjType = {
     "border",			/* name */
     FreeBorderObjProc,		/* freeIntRepProc */
     DupBorderObjProc,		/* dupIntRepProc */
@@ -71,8 +72,8 @@ Tcl_ObjType tkBorderObjType = {
  * Side effects:
  *	The border is added to an internal database with a reference count.
  *	For each call to this function, there should eventually be a call to
- *	FreeBorderObjProc so that the database is cleaned up when borders
- *	aren't in use anymore.
+ *	FreeBorderObj so that the database is cleaned up when borders aren't
+ *	in use anymore.
  *
  *----------------------------------------------------------------------
  */
@@ -89,7 +90,7 @@ Tk_Alloc3DBorderFromObj(
     if (objPtr->typePtr != &tkBorderObjType) {
 	InitBorderObj(objPtr);
     }
-    borderPtr = (TkBorder *) objPtr->internalRep.twoPtrValue.ptr1;
+    borderPtr = objPtr->internalRep.twoPtrValue.ptr1;
 
     /*
      * If the object currently points to a TkBorder, see if it's the one we
@@ -103,7 +104,7 @@ Tk_Alloc3DBorderFromObj(
 	     * longer in use. Clear the reference.
 	     */
 
-	    FreeBorderObjProc(objPtr);
+	    FreeBorderObj(objPtr);
 	    borderPtr = NULL;
 	} else if ((Tk_Screen(tkwin) == borderPtr->screen)
 		&& (Tk_Colormap(tkwin) == borderPtr->colormap)) {
@@ -116,9 +117,7 @@ Tk_Alloc3DBorderFromObj(
      * The object didn't point to the border that we wanted. Search the list
      * of borders with the same name to see if one of the others is the right
      * one.
-     */
-
-    /*
+     *
      * If the cached value is NULL, either the object type was not a color
      * going in, or the object is a color type but had previously been freed.
      *
@@ -128,16 +127,16 @@ Tk_Alloc3DBorderFromObj(
      */
 
     if (borderPtr != NULL) {
-	TkBorder *firstBorderPtr =
-		(TkBorder *) Tcl_GetHashValue(borderPtr->hashPtr);
-	FreeBorderObjProc(objPtr);
+	TkBorder *firstBorderPtr = Tcl_GetHashValue(borderPtr->hashPtr);
+
+	FreeBorderObj(objPtr);
 	for (borderPtr = firstBorderPtr ; borderPtr != NULL;
 		borderPtr = borderPtr->nextPtr) {
 	    if ((Tk_Screen(tkwin) == borderPtr->screen)
-		&& (Tk_Colormap(tkwin) == borderPtr->colormap)) {
+		    && (Tk_Colormap(tkwin) == borderPtr->colormap)) {
 		borderPtr->resourceRefCount++;
 		borderPtr->objRefCount++;
-		objPtr->internalRep.twoPtrValue.ptr1 = (void *) borderPtr;
+		objPtr->internalRep.twoPtrValue.ptr1 = borderPtr;
 		return (Tk_3DBorder) borderPtr;
 	    }
 	}
@@ -149,7 +148,7 @@ Tk_Alloc3DBorderFromObj(
 
     borderPtr = (TkBorder *) Tk_Get3DBorder(interp, tkwin,
 	    Tcl_GetString(objPtr));
-    objPtr->internalRep.twoPtrValue.ptr1 = (void *) borderPtr;
+    objPtr->internalRep.twoPtrValue.ptr1 = borderPtr;
     if (borderPtr != NULL) {
 	borderPtr->objRefCount++;
     }
@@ -201,7 +200,7 @@ Tk_Get3DBorder(
 
     hashPtr = Tcl_CreateHashEntry(&dispPtr->borderTable, colorName, &isNew);
     if (!isNew) {
-	existingBorderPtr = (TkBorder *) Tcl_GetHashValue(hashPtr);
+	existingBorderPtr = Tcl_GetHashValue(hashPtr);
 	for (borderPtr = existingBorderPtr; borderPtr != NULL;
 		borderPtr = borderPtr->nextPtr) {
 	    if ((Tk_Screen(tkwin) == borderPtr->screen)
@@ -318,7 +317,7 @@ Tk_Draw3DRectangle(
  *--------------------------------------------------------------
  */
 
-CONST char *
+const char *
 Tk_NameOf3DBorder(
     Tk_3DBorder border)		/* Token for border. */
 {
@@ -426,7 +425,7 @@ Tk_Free3DBorder(
 	return;
     }
 
-    prevPtr = (TkBorder *) Tcl_GetHashValue(borderPtr->hashPtr);
+    prevPtr = Tcl_GetHashValue(borderPtr->hashPtr);
     TkpFreeBorder(borderPtr);
     if (borderPtr->bgColorPtr != NULL) {
 	Tk_FreeColor(borderPtr->bgColorPtr);
@@ -462,7 +461,7 @@ Tk_Free3DBorder(
 	prevPtr->nextPtr = borderPtr->nextPtr;
     }
     if (borderPtr->objRefCount == 0) {
-	ckfree((char *) borderPtr);
+	ckfree(borderPtr);
     }
 }
 
@@ -494,13 +493,13 @@ Tk_Free3DBorderFromObj(
     Tcl_Obj *objPtr)		/* The Tcl_Obj * to be freed. */
 {
     Tk_Free3DBorder(Tk_Get3DBorderFromObj(tkwin, objPtr));
-    FreeBorderObjProc(objPtr);
+    FreeBorderObj(objPtr);
 }
 
 /*
  *---------------------------------------------------------------------------
  *
- * FreeBorderObjProc --
+ * FreeBorderObjProc, FreeBorderObj --
  *
  *	This proc is called to release an object reference to a border. Called
  *	when the object's internal rep is released or when the cached
@@ -520,13 +519,21 @@ static void
 FreeBorderObjProc(
     Tcl_Obj *objPtr)		/* The object we are releasing. */
 {
-    TkBorder *borderPtr = (TkBorder *) objPtr->internalRep.twoPtrValue.ptr1;
+    FreeBorderObj(objPtr);
+    objPtr->typePtr = NULL;
+}
+
+static void
+FreeBorderObj(
+    Tcl_Obj *objPtr)		/* The object we are releasing. */
+{
+    TkBorder *borderPtr = objPtr->internalRep.twoPtrValue.ptr1;
 
     if (borderPtr != NULL) {
 	borderPtr->objRefCount--;
 	if ((borderPtr->objRefCount == 0)
 		&& (borderPtr->resourceRefCount == 0)) {
-	    ckfree((char *) borderPtr);
+	    ckfree(borderPtr);
 	}
 	objPtr->internalRep.twoPtrValue.ptr1 = NULL;
     }
@@ -555,10 +562,10 @@ DupBorderObjProc(
     Tcl_Obj *srcObjPtr,		/* The object we are copying from. */
     Tcl_Obj *dupObjPtr)		/* The object we are copying to. */
 {
-    TkBorder *borderPtr = (TkBorder *) srcObjPtr->internalRep.twoPtrValue.ptr1;
+    TkBorder *borderPtr = srcObjPtr->internalRep.twoPtrValue.ptr1;
 
     dupObjPtr->typePtr = srcObjPtr->typePtr;
-    dupObjPtr->internalRep.twoPtrValue.ptr1 = (void *) borderPtr;
+    dupObjPtr->internalRep.twoPtrValue.ptr1 = borderPtr;
 
     if (borderPtr != NULL) {
 	borderPtr->objRefCount++;
@@ -643,7 +650,7 @@ Tk_GetReliefFromObj(
 int
 Tk_GetRelief(
     Tcl_Interp *interp,		/* For error messages. */
-    CONST char *name,		/* Name of a relief type. */
+    const char *name,		/* Name of a relief type. */
     int *reliefPtr)		/* Where to store converted relief. */
 {
     char c;
@@ -655,22 +662,21 @@ Tk_GetRelief(
 	*reliefPtr = TK_RELIEF_FLAT;
     } else if ((c == 'g') && (strncmp(name, "groove", length) == 0)
 	    && (length >= 2)) {
-        *reliefPtr = TK_RELIEF_GROOVE;
+	*reliefPtr = TK_RELIEF_GROOVE;
     } else if ((c == 'r') && (strncmp(name, "raised", length) == 0)
 	    && (length >= 2)) {
 	*reliefPtr = TK_RELIEF_RAISED;
     } else if ((c == 'r') && (strncmp(name, "ridge", length) == 0)) {
-        *reliefPtr = TK_RELIEF_RIDGE;
+	*reliefPtr = TK_RELIEF_RIDGE;
     } else if ((c == 's') && (strncmp(name, "solid", length) == 0)) {
 	*reliefPtr = TK_RELIEF_SOLID;
     } else if ((c == 's') && (strncmp(name, "sunken", length) == 0)) {
 	*reliefPtr = TK_RELIEF_SUNKEN;
     } else {
-	char buf[200];
-
-	sprintf(buf, "bad relief type \"%.50s\": must be %s",
-		name, "flat, groove, raised, ridge, solid, or sunken");
-	Tcl_SetResult(interp, buf, TCL_VOLATILE);
+	Tcl_SetObjResult(interp,
+		Tcl_ObjPrintf("bad relief \"%.50s\": must be %s",
+		name, "flat, groove, raised, ridge, solid, or sunken"));
+	Tcl_SetErrorCode(interp, "TK", "VALUE", "RELIEF", NULL);
 	return TCL_ERROR;
     }
     return TCL_OK;
@@ -692,7 +698,7 @@ Tk_GetRelief(
  *--------------------------------------------------------------
  */
 
-CONST char *
+const char *
 Tk_NameOfRelief(
     int relief)		/* One of TK_RELIEF_FLAT, TK_RELIEF_RAISED, or
 			 * TK_RELIEF_SUNKEN. */
@@ -768,9 +774,8 @@ Tk_Draw3DPolygon(
      */
 
     if ((leftRelief == TK_RELIEF_GROOVE) || (leftRelief == TK_RELIEF_RIDGE)) {
-	int halfWidth;
+	int halfWidth = borderWidth/2;
 
-	halfWidth = borderWidth/2;
 	Tk_Draw3DPolygon(tkwin, drawable, border, pointPtr, numPoints,
 		halfWidth, (leftRelief == TK_RELIEF_GROOVE) ? TK_RELIEF_RAISED
 		: TK_RELIEF_SUNKEN);
@@ -980,8 +985,8 @@ Tk_Fill3DRectangle(
     if ((width > doubleBorder) && (height > doubleBorder)) {
 	XFillRectangle(Tk_Display(tkwin), drawable, borderPtr->bgGC,
 		x + borderWidth, y + borderWidth,
-		(unsigned int) (width - doubleBorder),
-		(unsigned int) (height - doubleBorder));
+		(unsigned) (width - doubleBorder),
+		(unsigned) (height - doubleBorder));
     }
     if (borderWidth) {
 	Tk_Draw3DRectangle(tkwin, drawable, border, x, y, width,
@@ -1083,19 +1088,18 @@ ShiftLine(
     XPoint *p3Ptr)		/* Store coords of point on new line here. */
 {
     int dx, dy, dxNeg, dyNeg;
-
-    /*
-     * The table below is used for a quick approximation in computing the new
-     * point. An index into the table is 128 times the slope of the original
-     * line (the slope must always be between 0 and 1). The value of the table
-     * entry is 128 times the amount to displace the new line in y for each
-     * unit of perpendicular distance. In other words, the table maps from the
-     * tangent of an angle to the inverse of its cosine. If the slope of the
-     * original line is greater than 1, then the displacement is done in x
-     * rather than in y.
-     */
-
-    static int shiftTable[129];
+    static int shiftTable[129];	/* Used for a quick approximation in computing
+				 * the new point. An index into the table is
+				 * 128 times the slope of the original line
+				 * (the slope must always be between 0 and 1).
+				 * The value of the table entry is 128 times
+				 * the amount to displace the new line in y
+				 * for each unit of perpendicular distance. In
+				 * other words, the table maps from the
+				 * tangent of an angle to the inverse of its
+				 * cosine. If the slope of the original line
+				 * is greater than 1, then the displacement is
+				 * done in x rather than in y. */
 
     /*
      * Initialize the table if this is the first time it is used.
@@ -1249,7 +1253,7 @@ Tk_Get3DBorderFromObj(
      * cached in the internal representation of the Tcl_Obj. Check it out...
      */
 
-    borderPtr = (TkBorder *) objPtr->internalRep.twoPtrValue.ptr1;
+    borderPtr = objPtr->internalRep.twoPtrValue.ptr1;
     if ((borderPtr != NULL)
 	    && (borderPtr->resourceRefCount > 0)
 	    && (Tk_Screen(tkwin) == borderPtr->screen)
@@ -1277,12 +1281,12 @@ Tk_Get3DBorderFromObj(
     if (hashPtr == NULL) {
 	goto error;
     }
-    for (borderPtr = (TkBorder *) Tcl_GetHashValue(hashPtr);
-	    (borderPtr != NULL); borderPtr = borderPtr->nextPtr) {
+    for (borderPtr = Tcl_GetHashValue(hashPtr); borderPtr != NULL;
+	    borderPtr = borderPtr->nextPtr) {
 	if ((Tk_Screen(tkwin) == borderPtr->screen)
 		&& (Tk_Colormap(tkwin) == borderPtr->colormap)) {
-	    FreeBorderObjProc(objPtr);
-	    objPtr->internalRep.twoPtrValue.ptr1 = (void *) borderPtr;
+	    FreeBorderObj(objPtr);
+	    objPtr->internalRep.twoPtrValue.ptr1 = borderPtr;
 	    borderPtr->objRefCount++;
 	    return (Tk_3DBorder) borderPtr;
 	}
@@ -1329,7 +1333,7 @@ InitBorderObj(
     Tcl_GetString(objPtr);
     typePtr = objPtr->typePtr;
     if ((typePtr != NULL) && (typePtr->freeIntRepProc != NULL)) {
-	(*typePtr->freeIntRepProc)(objPtr);
+	typePtr->freeIntRepProc(objPtr);
     }
     objPtr->typePtr = &tkBorderObjType;
     objPtr->internalRep.twoPtrValue.ptr1 = NULL;
@@ -1358,22 +1362,23 @@ Tcl_Obj *
 TkDebugBorder(
     Tk_Window tkwin,		/* The window in which the border will be used
 				 * (not currently used). */
-    char *name)			/* Name of the desired color. */
+    const char *name)		/* Name of the desired color. */
 {
-    TkBorder *borderPtr;
     Tcl_HashEntry *hashPtr;
-    Tcl_Obj *resultPtr, *objPtr;
+    Tcl_Obj *resultPtr;
     TkDisplay *dispPtr = ((TkWindow *) tkwin)->dispPtr;
 
     resultPtr = Tcl_NewObj();
     hashPtr = Tcl_FindHashEntry(&dispPtr->borderTable, name);
     if (hashPtr != NULL) {
-	borderPtr = (TkBorder *) Tcl_GetHashValue(hashPtr);
+	TkBorder *borderPtr = Tcl_GetHashValue(hashPtr);
+
 	if (borderPtr == NULL) {
 	    Tcl_Panic("TkDebugBorder found empty hash table entry");
 	}
 	for ( ; (borderPtr != NULL); borderPtr = borderPtr->nextPtr) {
-	    objPtr = Tcl_NewObj();
+	    Tcl_Obj *objPtr = Tcl_NewObj();
+
 	    Tcl_ListObjAppendElement(NULL, objPtr,
 		    Tcl_NewIntObj(borderPtr->resourceRefCount));
 	    Tcl_ListObjAppendElement(NULL, objPtr,
