@@ -147,10 +147,6 @@ typedef struct PanedWindow {
     GC gc;			/* Graphics context for copying from
 				 * off-screen pixmap onto screen. */
     int proxyx, proxyy;		/* Proxy x,y coordinates. */
-    Tk_3DBorder proxyBackground;/* Background color used to draw proxy. If NULL, use background. */
-    Tcl_Obj *proxyBorderWidthPtr; /* Tcl_Obj rep for proxyBorderWidth */
-    int proxyBorderWidth;	/* Borderwidth used to draw proxy. */
-    int proxyRelief;		/* Relief used to draw proxy, if TK_RELIEF_NULL then use relief. */
     Slave **slaves;		/* Pointer to array of Slaves. */
     int numSlaves;		/* Number of slaves. */
     int sizeofSlaves;		/* Number of elements in the slaves array. */
@@ -207,8 +203,6 @@ static void		PanedWindowReqProc(ClientData clientData,
 static void		ArrangePanes(ClientData clientData);
 static void		Unlink(Slave *slavePtr);
 static Slave *		GetPane(PanedWindow *pwPtr, Tk_Window tkwin);
-static void		GetFirstLastVisiblePane(PanedWindow *pwPtr,
-			    int *firstPtr, int *lastPtr);
 static void		SlaveStructureProc(ClientData clientData,
 			    XEvent *eventPtr);
 static int		PanedWindowSashCommand(PanedWindow *pwPtr,
@@ -276,11 +270,11 @@ static const Tk_ObjCustomOption stickyOption = {
 static const Tk_OptionSpec optionSpecs[] = {
     {TK_OPTION_BORDER, "-background", "background", "Background",
 	 DEF_PANEDWINDOW_BG_COLOR, -1, Tk_Offset(PanedWindow, background), 0,
-	 (ClientData) DEF_PANEDWINDOW_BG_MONO},
+	 DEF_PANEDWINDOW_BG_MONO, 0},
     {TK_OPTION_SYNONYM, "-bd", NULL, NULL,
-	 NULL, 0, -1, 0, (ClientData) "-borderwidth"},
+	 NULL, 0, -1, 0, "-borderwidth", 0},
     {TK_OPTION_SYNONYM, "-bg", NULL, NULL,
-	 NULL, 0, -1, 0, (ClientData) "-background"},
+	 NULL, 0, -1, 0, "-background", 0},
     {TK_OPTION_PIXELS, "-borderwidth", "borderWidth", "BorderWidth",
 	 DEF_PANEDWINDOW_BORDERWIDTH, -1, Tk_Offset(PanedWindow, borderWidth),
 	 0, 0, GEOMETRY},
@@ -301,16 +295,7 @@ static const Tk_OptionSpec optionSpecs[] = {
 	 Tk_Offset(PanedWindow, resizeOpaque), 0, 0, 0},
     {TK_OPTION_STRING_TABLE, "-orient", "orient", "Orient",
 	 DEF_PANEDWINDOW_ORIENT, -1, Tk_Offset(PanedWindow, orient),
-	 0, (ClientData) orientStrings, GEOMETRY},
-    {TK_OPTION_BORDER, "-proxybackground", "proxyBackground", "ProxyBackground",
-	 0, -1, Tk_Offset(PanedWindow, proxyBackground), TK_OPTION_NULL_OK,
-	 (ClientData) DEF_PANEDWINDOW_BG_MONO},
-    {TK_OPTION_PIXELS, "-proxyborderwidth", "proxyBorderWidth", "ProxyBorderWidth",
-	 DEF_PANEDWINDOW_PROXYBORDER, Tk_Offset(PanedWindow, proxyBorderWidthPtr),
-	 Tk_Offset(PanedWindow, proxyBorderWidth), 0, 0, GEOMETRY},
-    {TK_OPTION_RELIEF, "-proxyrelief", "proxyRelief", "Relief",
-	 0, -1, Tk_Offset(PanedWindow, proxyRelief),
-	 TK_OPTION_NULL_OK, 0, 0},
+	 0, orientStrings, GEOMETRY},
     {TK_OPTION_RELIEF, "-relief", "relief", "Relief",
 	 DEF_PANEDWINDOW_RELIEF, -1, Tk_Offset(PanedWindow, relief), 0, 0, 0},
     {TK_OPTION_CURSOR, "-sashcursor", "sashCursor", "Cursor",
@@ -331,7 +316,7 @@ static const Tk_OptionSpec optionSpecs[] = {
     {TK_OPTION_PIXELS, "-width", "width", "Width",
 	 DEF_PANEDWINDOW_WIDTH, Tk_Offset(PanedWindow, widthPtr),
 	 Tk_Offset(PanedWindow, width), TK_OPTION_NULL_OK, 0, GEOMETRY},
-    {TK_OPTION_END}
+    {TK_OPTION_END, NULL, NULL, NULL, NULL, 0, 0, 0, 0, 0}
 };
 
 static const Tk_OptionSpec slaveOptionSpecs[] = {
@@ -354,14 +339,14 @@ static const Tk_OptionSpec slaveOptionSpecs[] = {
 	 DEF_PANEDWINDOW_PANE_PADY, -1, Tk_Offset(Slave, pady), 0, 0, 0},
     {TK_OPTION_CUSTOM, "-sticky", NULL, NULL,
 	 DEF_PANEDWINDOW_PANE_STICKY, -1, Tk_Offset(Slave, sticky), 0,
-	 (ClientData) &stickyOption, 0},
+	 &stickyOption, 0},
     {TK_OPTION_STRING_TABLE, "-stretch", "stretch", "Stretch",
 	DEF_PANEDWINDOW_PANE_STRETCH, -1, Tk_Offset(Slave, stretch), 0,
 	(ClientData) stretchStrings, 0},
     {TK_OPTION_PIXELS, "-width", NULL, NULL,
 	 DEF_PANEDWINDOW_PANE_WIDTH, Tk_Offset(Slave, widthPtr),
 	 Tk_Offset(Slave, width), TK_OPTION_NULL_OK, 0, 0},
-    {TK_OPTION_END}
+    {TK_OPTION_END, NULL, NULL, NULL, NULL, 0, 0, 0, 0, 0}
 };
 
 /*
@@ -394,7 +379,7 @@ Tk_PanedWindowObjCmd(
     XSetWindowAttributes atts;
 
     if (objc < 2) {
-	Tcl_WrongNumArgs(interp, 1, objv, "pathName ?options?");
+	Tcl_WrongNumArgs(interp, 1, objv, "pathName ?-option value ...?");
 	return TCL_ERROR;
     }
 
@@ -414,14 +399,14 @@ Tk_PanedWindowObjCmd(
 	 * easy access to it in the future.
 	 */
 
-	pwOpts = (OptionTables *) ckalloc(sizeof(OptionTables));
+	pwOpts = ckalloc(sizeof(OptionTables));
 
 	/*
 	 * Set up an exit handler to free the optionTables struct.
 	 */
 
 	Tcl_SetAssocData(interp, "PanedWindowOptionTables",
-		DestroyOptionTables, (ClientData) pwOpts);
+		DestroyOptionTables, pwOpts);
 
 	/*
 	 * Create the paned window option tables.
@@ -437,14 +422,14 @@ Tk_PanedWindowObjCmd(
      * Allocate and initialize the widget record.
      */
 
-    pwPtr = (PanedWindow *) ckalloc(sizeof(PanedWindow));
+    pwPtr = ckalloc(sizeof(PanedWindow));
     memset((void *)pwPtr, 0, (sizeof(PanedWindow)));
     pwPtr->tkwin = tkwin;
     pwPtr->display = Tk_Display(tkwin);
     pwPtr->interp = interp;
     pwPtr->widgetCmd = Tcl_CreateObjCommand(interp,
-	    Tk_PathName(pwPtr->tkwin), PanedWindowWidgetObjCmd,
-	    (ClientData) pwPtr, PanedWindowCmdDeletedProc);
+	    Tk_PathName(pwPtr->tkwin), PanedWindowWidgetObjCmd, pwPtr,
+	    PanedWindowCmdDeletedProc);
     pwPtr->optionTable = pwOpts->pwOptions;
     pwPtr->slaveOpts = pwOpts->slaveOpts;
     pwPtr->relief = TK_RELIEF_RAISED;
@@ -457,7 +442,7 @@ Tk_PanedWindowObjCmd(
      * otherwise Tk might free it while we still need it.
      */
 
-    Tcl_Preserve((ClientData) pwPtr->tkwin);
+    Tcl_Preserve(pwPtr->tkwin);
 
     if (Tk_InitOptions(interp, (char *) pwPtr, pwOpts->pwOptions,
 	    tkwin) != TCL_OK) {
@@ -466,7 +451,7 @@ Tk_PanedWindowObjCmd(
     }
 
     Tk_CreateEventHandler(pwPtr->tkwin, ExposureMask|StructureNotifyMask,
-	    PanedWindowEventProc, (ClientData) pwPtr);
+	    PanedWindowEventProc, pwPtr);
 
     /*
      * Find the toplevel ancestor of the panedwindow, and make a proxy win as
@@ -496,7 +481,7 @@ Tk_PanedWindowObjCmd(
     Tk_SetWindowVisual(pwPtr->proxywin,
 	    Tk_Visual(tkwin), Tk_Depth(tkwin), Tk_Colormap(tkwin));
     Tk_CreateEventHandler(pwPtr->proxywin, ExposureMask, ProxyWindowEventProc,
-	    (ClientData) pwPtr);
+	    pwPtr);
     atts.save_under = True;
     Tk_ChangeWindowAttributes(pwPtr->proxywin, CWSaveUnder, &atts);
 
@@ -506,7 +491,7 @@ Tk_PanedWindowObjCmd(
 	return TCL_ERROR;
     }
 
-    Tcl_SetStringObj(Tcl_GetObjResult(interp), Tk_PathName(pwPtr->tkwin), -1);
+    Tcl_SetObjResult(interp, TkNewWindowObj(pwPtr->tkwin));
     return TCL_OK;
 }
 
@@ -535,9 +520,9 @@ PanedWindowWidgetObjCmd(
     int objc,			/* Number of arguments. */
     Tcl_Obj * const objv[])	/* Argument objects. */
 {
-    PanedWindow *pwPtr = (PanedWindow *) clientData;
+    PanedWindow *pwPtr = clientData;
     int result = TCL_OK;
-    static const char *optionStrings[] = {
+    static const char *const optionStrings[] = {
 	"add", "cget", "configure", "forget", "identify", "panecget",
 	"paneconfigure", "panes", "proxy", "sash", NULL
     };
@@ -560,7 +545,7 @@ PanedWindowWidgetObjCmd(
 	return TCL_ERROR;
     }
 
-    Tcl_Preserve((ClientData) pwPtr);
+    Tcl_Preserve(pwPtr);
 
     switch ((enum options) index) {
     case PW_ADD:
@@ -618,16 +603,17 @@ PanedWindowWidgetObjCmd(
 	for (count = 0, i = 2; i < objc; i++) {
 	    Tk_Window slave = Tk_NameToWindow(interp, Tcl_GetString(objv[i]),
 		    pwPtr->tkwin);
+
 	    if (slave == NULL) {
 		continue;
 	    }
 	    slavePtr = GetPane(pwPtr, slave);
 	    if ((slavePtr != NULL) && (slavePtr->masterPtr != NULL)) {
 		count++;
-		Tk_ManageGeometry(slave, NULL, (ClientData)NULL);
+		Tk_ManageGeometry(slave, NULL, NULL);
 		Tk_UnmaintainGeometry(slavePtr->tkwin, pwPtr->tkwin);
 		Tk_DeleteEventHandler(slavePtr->tkwin, StructureNotifyMask,
-			SlaveStructureProc, (ClientData) slavePtr);
+			SlaveStructureProc, slavePtr);
 		Tk_UnmapWindow(slavePtr->tkwin);
 		Unlink(slavePtr);
 	    }
@@ -672,10 +658,13 @@ PanedWindowWidgetObjCmd(
 			objv[3], tkwin);
 	    }
 	}
-	if (i == pwPtr->numSlaves) {
-	    Tcl_SetResult(interp, "not managed by this window", TCL_STATIC);
-	}
 	if (resultObj == NULL) {
+	    if (i == pwPtr->numSlaves) {
+		Tcl_SetObjResult(interp, Tcl_NewStringObj(
+			"not managed by this window", -1));
+		Tcl_SetErrorCode(interp, "TK", "PANEDWINDOW", "UNMANAGED",
+			NULL);
+	    }
 	    result = TCL_ERROR;
 	} else {
 	    Tcl_SetObjResult(interp, resultObj);
@@ -693,15 +682,6 @@ PanedWindowWidgetObjCmd(
 	if (objc <= 4) {
 	    tkwin = Tk_NameToWindow(interp, Tcl_GetString(objv[2]),
 		    pwPtr->tkwin);
-            if (tkwin == NULL) {
-                /*
-                 * Just a plain old bad window; Tk_NameToWindow filled in an
-                 * error message for us.
-                 */
-
-                result = TCL_ERROR;
-                break;
-            }
 	    for (i = 0; i < pwPtr->numSlaves; i++) {
 		if (pwPtr->slaves[i]->tkwin == tkwin) {
 		    resultObj = Tk_GetOptionInfo(interp,
@@ -723,15 +703,11 @@ PanedWindowWidgetObjCmd(
 
     case PW_PANES:
 	resultObj = Tcl_NewObj();
-
-	Tcl_IncrRefCount(resultObj);
-
 	for (i = 0; i < pwPtr->numSlaves; i++) {
-	    Tcl_ListObjAppendElement(interp, resultObj,
-		    Tcl_NewStringObj(Tk_PathName(pwPtr->slaves[i]->tkwin),-1));
+	    Tcl_ListObjAppendElement(NULL, resultObj,
+		    TkNewWindowObj(pwPtr->slaves[i]->tkwin));
 	}
 	Tcl_SetObjResult(interp, resultObj);
-	Tcl_DecrRefCount(resultObj);
 	break;
 
     case PW_PROXY:
@@ -742,7 +718,7 @@ PanedWindowWidgetObjCmd(
 	result = PanedWindowSashCommand(pwPtr, interp, objc, objv);
 	break;
     }
-    Tcl_Release((ClientData) pwPtr);
+    Tcl_Release(pwPtr);
     return result;
 }
 
@@ -775,7 +751,7 @@ ConfigureSlaves(
     Tk_Window tkwin = NULL, ancestor, parent;
     Slave *slavePtr, **inserts, **newSlaves;
     Slave options;
-    char *arg;
+    const char *arg;
 
     /*
      * Find the non-window name arguments; these are the configure options for
@@ -801,18 +777,19 @@ ConfigureSlaves(
 		 * A panedwindow cannot manage itself.
 		 */
 
-		Tcl_ResetResult(interp);
-		Tcl_AppendResult(interp, "can't add ", arg, " to itself",
-			NULL);
+		Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+			"can't add %s to itself", arg));
+		Tcl_SetErrorCode(interp, "TK", "GEOMETRY", "SELF", NULL);
 		return TCL_ERROR;
 	    } else if (Tk_IsTopLevel(tkwin)) {
 		/*
 		 * A panedwindow cannot manage a toplevel.
 		 */
 
-		Tcl_ResetResult(interp);
-		Tcl_AppendResult(interp, "can't add toplevel ", arg, " to ",
-			Tk_PathName(pwPtr->tkwin), NULL);
+		Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+			"can't add toplevel %s to %s", arg,
+			Tk_PathName(pwPtr->tkwin)));
+		Tcl_SetErrorCode(interp, "TK", "GEOMETRY", "TOPLEVEL", NULL);
 		return TCL_ERROR;
 	    } else {
 		/*
@@ -826,9 +803,11 @@ ConfigureSlaves(
 			break;
 		    }
 		    if (Tk_IsTopLevel(ancestor)) {
-			Tcl_ResetResult(interp);
-			Tcl_AppendResult(interp, "can't add ", arg, " to ",
-				Tk_PathName(pwPtr->tkwin), NULL);
+			Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+				"can't add %s to %s", arg,
+				Tk_PathName(pwPtr->tkwin)));
+			Tcl_SetErrorCode(interp, "TK", "GEOMETRY",
+				"HIERARCHY", NULL);
 			return TCL_ERROR;
 		    }
 		}
@@ -885,9 +864,10 @@ ConfigureSlaves(
      */
 
     if (haveLoc && index == -1) {
-	Tcl_ResetResult(interp);
-	Tcl_AppendResult(interp, "window \"", Tk_PathName(tkwin),
-		"\" is not managed by ", Tk_PathName(pwPtr->tkwin), NULL);
+	Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+		"window \"%s\" is not managed by %s",
+		Tk_PathName(tkwin), Tk_PathName(pwPtr->tkwin)));
+	Tcl_SetErrorCode(interp, "TK", "PANEDWINDOW", "UNMANAGED", NULL);
 	Tk_FreeConfigOptions((char *) &options, pwPtr->slaveOpts,
 		pwPtr->tkwin);
 	return TCL_ERROR;
@@ -899,7 +879,7 @@ ConfigureSlaves(
      * structures may already have existed, some may be new.
      */
 
-    inserts = (Slave **)ckalloc(sizeof(Slave *) * (firstOptionArg - 2));
+    inserts = ckalloc(sizeof(Slave *) * (firstOptionArg - 2));
     insertIndex = 0;
 
     /*
@@ -966,7 +946,7 @@ ConfigureSlaves(
 	 * out with their "natural" dimensions.
 	 */
 
-	slavePtr = (Slave *) ckalloc(sizeof(Slave));
+	slavePtr = ckalloc(sizeof(Slave));
 	memset(slavePtr, 0, sizeof(Slave));
 	Tk_InitOptions(interp, (char *)slavePtr, pwPtr->slaveOpts,
 		pwPtr->tkwin);
@@ -995,9 +975,8 @@ ConfigureSlaves(
 	 */
 
 	Tk_CreateEventHandler(slavePtr->tkwin, StructureNotifyMask,
-		SlaveStructureProc, (ClientData) slavePtr);
-	Tk_ManageGeometry(slavePtr->tkwin, &panedWindowMgrType,
-		(ClientData) slavePtr);
+		SlaveStructureProc, slavePtr);
+	Tk_ManageGeometry(slavePtr->tkwin, &panedWindowMgrType, slavePtr);
 	inserts[insertIndex++] = slavePtr;
 	numNewSlaves++;
     }
@@ -1006,8 +985,8 @@ ConfigureSlaves(
      * Allocate the new slaves array, then copy the slaves into it, in order.
      */
 
-    i = sizeof(Slave *) * (pwPtr->numSlaves+numNewSlaves);
-    newSlaves = (Slave **)ckalloc((unsigned) i);
+    i = sizeof(Slave *) * (pwPtr->numSlaves + numNewSlaves);
+    newSlaves = ckalloc(i);
     memset(newSlaves, 0, (size_t) i);
     if (index == -1) {
 	/*
@@ -1050,8 +1029,8 @@ ConfigureSlaves(
      * Make the new slaves array the paned window's slave array, and clean up.
      */
 
-    ckfree((void *)pwPtr->slaves);
-    ckfree((void *)inserts);
+    ckfree(pwPtr->slaves);
+    ckfree(inserts);
     pwPtr->slaves = newSlaves;
 
     /*
@@ -1090,7 +1069,7 @@ PanedWindowSashCommand(
     int objc,			/* Number of arguments. */
     Tcl_Obj *const objv[])	/* Argument objects. */
 {
-    static const char *sashOptionStrings[] = {
+    static const char *const sashOptionStrings[] = {
 	"coord", "dragto", "mark", "place", NULL
     };
     enum sashOptions {
@@ -1110,7 +1089,6 @@ PanedWindowSashCommand(
 	return TCL_ERROR;
     }
 
-    Tcl_ResetResult(interp);
     switch ((enum sashOptions) index) {
     case SASH_COORD:
 	if (objc != 4) {
@@ -1123,15 +1101,16 @@ PanedWindowSashCommand(
 	}
 
 	if (!ValidSashIndex(pwPtr, sash)) {
-	    Tcl_ResetResult(interp);
-	    Tcl_SetResult(interp, "invalid sash index", TCL_STATIC);
+	    Tcl_SetObjResult(interp, Tcl_NewStringObj(
+		    "invalid sash index", -1));
+	    Tcl_SetErrorCode(interp, "TK", "VALUE", "SASH_INDEX", NULL);
 	    return TCL_ERROR;
 	}
 	slavePtr = pwPtr->slaves[sash];
 
 	coords[0] = Tcl_NewIntObj(slavePtr->sashx);
 	coords[1] = Tcl_NewIntObj(slavePtr->sashy);
-	Tcl_SetListObj(Tcl_GetObjResult(interp), 2, coords);
+	Tcl_SetObjResult(interp, Tcl_NewListObj(2, coords));
 	break;
 
     case SASH_MARK:
@@ -1145,8 +1124,9 @@ PanedWindowSashCommand(
 	}
 
 	if (!ValidSashIndex(pwPtr, sash)) {
-	    Tcl_ResetResult(interp);
-	    Tcl_SetResult(interp, "invalid sash index", TCL_STATIC);
+	    Tcl_SetObjResult(interp, Tcl_NewStringObj(
+		    "invalid sash index", -1));
+	    Tcl_SetErrorCode(interp, "TK", "VALUE", "SASH_INDEX", NULL);
 	    return TCL_ERROR;
 	}
 
@@ -1164,7 +1144,7 @@ PanedWindowSashCommand(
 	} else {
 	    coords[0] = Tcl_NewIntObj(pwPtr->slaves[sash]->markx);
 	    coords[1] = Tcl_NewIntObj(pwPtr->slaves[sash]->marky);
-	    Tcl_SetListObj(Tcl_GetObjResult(interp), 2, coords);
+	    Tcl_SetObjResult(interp, Tcl_NewListObj(2, coords));
 	}
 	break;
 
@@ -1180,8 +1160,9 @@ PanedWindowSashCommand(
 	}
 
 	if (!ValidSashIndex(pwPtr, sash)) {
-	    Tcl_ResetResult(interp);
-	    Tcl_SetResult(interp, "invalid sash index", TCL_STATIC);
+	    Tcl_SetObjResult(interp, Tcl_NewStringObj(
+		    "invalid sash index", -1));
+	    Tcl_SetErrorCode(interp, "TK", "VALUE", "SASH_INDEX", NULL);
 	    return TCL_ERROR;
 	}
 
@@ -1252,7 +1233,7 @@ ConfigurePanedWindow(
 
     Tk_FreeSavedOptions(&savedOptions);
 
-    PanedWindowWorldChanged((ClientData) pwPtr);
+    PanedWindowWorldChanged(pwPtr);
 
     /*
      * If an option that affects geometry has changed, make a re-layout
@@ -1290,7 +1271,7 @@ PanedWindowWorldChanged(
 {
     XGCValues gcValues;
     GC newGC;
-    PanedWindow *pwPtr = (PanedWindow *) instanceData;
+    PanedWindow *pwPtr = instanceData;
 
     /*
      * Allocated a graphics context for drawing the paned window widget
@@ -1319,7 +1300,7 @@ PanedWindowWorldChanged(
      */
 
     if (Tk_IsMapped(pwPtr->tkwin) && !(pwPtr->flags & REDRAW_PENDING)) {
-	Tcl_DoWhenIdle(DisplayPanedWindow, (ClientData) pwPtr);
+	Tcl_DoWhenIdle(DisplayPanedWindow, pwPtr);
 	pwPtr->flags |= REDRAW_PENDING;
     }
 }
@@ -1347,30 +1328,21 @@ PanedWindowEventProc(
     ClientData clientData,	/* Information about window. */
     XEvent *eventPtr)		/* Information about event. */
 {
-    PanedWindow *pwPtr = (PanedWindow *) clientData;
-    int i;
+    PanedWindow *pwPtr = clientData;
 
     if (eventPtr->type == Expose) {
 	if (pwPtr->tkwin != NULL && !(pwPtr->flags & REDRAW_PENDING)) {
-	    Tcl_DoWhenIdle(DisplayPanedWindow, (ClientData) pwPtr);
+	    Tcl_DoWhenIdle(DisplayPanedWindow, pwPtr);
 	    pwPtr->flags |= REDRAW_PENDING;
 	}
     } else if (eventPtr->type == ConfigureNotify) {
 	pwPtr->flags |= REQUESTED_RELAYOUT;
 	if (pwPtr->tkwin != NULL && !(pwPtr->flags & REDRAW_PENDING)) {
-	    Tcl_DoWhenIdle(DisplayPanedWindow, (ClientData) pwPtr);
+	    Tcl_DoWhenIdle(DisplayPanedWindow, pwPtr);
 	    pwPtr->flags |= REDRAW_PENDING;
 	}
     } else if (eventPtr->type == DestroyNotify) {
 	DestroyPanedWindow(pwPtr);
-    } else if (eventPtr->type == UnmapNotify) {
-        for (i = 0; i < pwPtr->numSlaves; i++) {
-            Tk_UnmapWindow(pwPtr->slaves[i]->tkwin);
-        }
-    } else if (eventPtr->type == MapNotify) {
-        for (i = 0; i < pwPtr->numSlaves; i++) {
-            Tk_MapWindow(pwPtr->slaves[i]->tkwin);
-        }
     }
 }
 
@@ -1396,7 +1368,7 @@ static void
 PanedWindowCmdDeletedProc(
     ClientData clientData)	/* Pointer to widget record for widget. */
 {
-    PanedWindow *pwPtr = (PanedWindow *) clientData;
+    PanedWindow *pwPtr = clientData;
 
     /*
      * This function could be invoked either because the window was destroyed
@@ -1433,13 +1405,12 @@ static void
 DisplayPanedWindow(
     ClientData clientData)	/* Information about window. */
 {
-    PanedWindow *pwPtr = (PanedWindow *) clientData;
+    PanedWindow *pwPtr = clientData;
     Slave *slavePtr;
     Pixmap pixmap;
     Tk_Window tkwin = pwPtr->tkwin;
     int i, sashWidth, sashHeight;
     const int horizontal = (pwPtr->orient == ORIENT_HORIZONTAL);
-    int first, last;
 
     pwPtr->flags &= ~REDRAW_PENDING;
     if ((pwPtr->tkwin == NULL) || !Tk_IsMapped(tkwin)) {
@@ -1486,10 +1457,9 @@ DisplayPanedWindow(
      * Draw the sashes.
      */
 
-    GetFirstLastVisiblePane(pwPtr, &first, &last);
     for (i = 0; i < pwPtr->numSlaves - 1; i++) {
 	slavePtr = pwPtr->slaves[i];
-	if (slavePtr->hide || i == last) {
+	if (slavePtr->hide) {
 	    continue;
 	}
 	if (sashWidth > 0 && sashHeight > 0) {
@@ -1553,10 +1523,10 @@ DestroyPanedWindow(
      */
 
     if (pwPtr->flags & REDRAW_PENDING) {
-	Tcl_CancelIdleCall(DisplayPanedWindow, (ClientData) pwPtr);
+	Tcl_CancelIdleCall(DisplayPanedWindow, pwPtr);
     }
     if (pwPtr->flags & RESIZE_PENDING) {
-	Tcl_CancelIdleCall(ArrangePanes, (ClientData) pwPtr);
+	Tcl_CancelIdleCall(ArrangePanes, pwPtr);
     }
 
     /*
@@ -1568,15 +1538,15 @@ DestroyPanedWindow(
 
     for (i = 0; i < pwPtr->numSlaves; i++) {
 	Tk_DeleteEventHandler(pwPtr->slaves[i]->tkwin, StructureNotifyMask,
-		SlaveStructureProc, (ClientData) pwPtr->slaves[i]);
+		SlaveStructureProc, pwPtr->slaves[i]);
 	Tk_ManageGeometry(pwPtr->slaves[i]->tkwin, NULL, NULL);
-	Tk_FreeConfigOptions((char *)pwPtr->slaves[i], pwPtr->slaveOpts,
+	Tk_FreeConfigOptions((char *) pwPtr->slaves[i], pwPtr->slaveOpts,
 		pwPtr->tkwin);
-	ckfree((void *)pwPtr->slaves[i]);
+	ckfree(pwPtr->slaves[i]);
 	pwPtr->slaves[i] = NULL;
     }
     if (pwPtr->slaves) {
-	ckfree((char *) pwPtr->slaves);
+	ckfree(pwPtr->slaves);
     }
 
     /*
@@ -1590,10 +1560,10 @@ DestroyPanedWindow(
      */
 
     Tk_FreeConfigOptions((char *) pwPtr, pwPtr->optionTable, pwPtr->tkwin);
-    Tcl_Release((ClientData) pwPtr->tkwin);
+    Tcl_Release(pwPtr->tkwin);
     pwPtr->tkwin = NULL;
 
-    Tcl_EventuallyFree((ClientData) pwPtr, TCL_DYNAMIC);
+    Tcl_EventuallyFree(pwPtr, TCL_DYNAMIC);
 }
 
 /*
@@ -1621,12 +1591,13 @@ PanedWindowReqProc(
     Tk_Window tkwin)		/* Other Tk-related information about the
 				 * window. */
 {
-    Slave *slavePtr = (Slave *) clientData;
-    PanedWindow *pwPtr = (PanedWindow *) (slavePtr->masterPtr);
+    Slave *slavePtr = clientData;
+    PanedWindow *pwPtr = (PanedWindow *) slavePtr->masterPtr;
+
     if (Tk_IsMapped(pwPtr->tkwin)) {
 	if (!(pwPtr->flags & RESIZE_PENDING)) {
 	    pwPtr->flags |= RESIZE_PENDING;
-	    Tcl_DoWhenIdle(ArrangePanes, (ClientData) pwPtr);
+	    Tcl_DoWhenIdle(ArrangePanes, pwPtr);
 	}
     } else {
 	int doubleBw = 2 * Tk_Changes(slavePtr->tkwin)->border_width;
@@ -1665,18 +1636,18 @@ PanedWindowLostSlaveProc(
 				 * stolen away. */
     Tk_Window tkwin)		/* Tk's handle for the slave window. */
 {
-    register Slave *slavePtr = (Slave *) clientData;
-    PanedWindow *pwPtr = (PanedWindow *) (slavePtr->masterPtr);
+    register Slave *slavePtr = clientData;
+    PanedWindow *pwPtr = (PanedWindow *) slavePtr->masterPtr;
 
     if (pwPtr->tkwin != Tk_Parent(slavePtr->tkwin)) {
 	Tk_UnmaintainGeometry(slavePtr->tkwin, pwPtr->tkwin);
     }
     Unlink(slavePtr);
     Tk_DeleteEventHandler(slavePtr->tkwin, StructureNotifyMask,
-	    SlaveStructureProc, (ClientData) slavePtr);
+	    SlaveStructureProc, slavePtr);
     Tk_UnmapWindow(slavePtr->tkwin);
     slavePtr->tkwin = NULL;
-    ckfree((void *)slavePtr);
+    ckfree(slavePtr);
     ComputeGeometry(pwPtr);
 }
 
@@ -1704,7 +1675,7 @@ ArrangePanes(
     ClientData clientData)	/* Structure describing parent whose slaves
 				 * are to be re-layed out. */
 {
-    register PanedWindow *pwPtr = (PanedWindow *) clientData;
+    register PanedWindow *pwPtr = clientData;
     register Slave *slavePtr;
     int i, slaveWidth, slaveHeight, slaveX, slaveY;
     int paneWidth, paneHeight, paneSize, paneMinSize;
@@ -1731,13 +1702,20 @@ ArrangePanes(
 	return;
     }
 
-    Tcl_Preserve((ClientData) pwPtr);
+    Tcl_Preserve(pwPtr);
 
     /*
-     * Find index of first and last visible panes.
+     * Find index of last visible pane.
      */
 
-    GetFirstLastVisiblePane(pwPtr, &first, &last);
+    for (i = 0, last = 0, first = -1; i < pwPtr->numSlaves; i++) {
+	if (pwPtr->slaves[i]->hide == 0) {
+	    if (first < 0) {
+		first = i;
+	    }
+	    last = i;
+	}
+    }
 
     /*
      * First pass; compute sizes
@@ -1967,7 +1945,7 @@ ArrangePanes(
 	}
 	sashCount--;
     }
-    Tcl_Release((ClientData) pwPtr);
+    Tcl_Release(pwPtr);
 }
 
 /*
@@ -2028,7 +2006,7 @@ Unlink(
     masterPtr->flags |= REQUESTED_RELAYOUT;
     if (!(masterPtr->flags & REDRAW_PENDING)) {
 	masterPtr->flags |= REDRAW_PENDING;
-	Tcl_DoWhenIdle(DisplayPanedWindow, (ClientData) masterPtr);
+	Tcl_DoWhenIdle(DisplayPanedWindow, masterPtr);
     }
 
     /*
@@ -2074,41 +2052,6 @@ GetPane(
     return NULL;
 }
 
-/*
- *----------------------------------------------------------------------
- *
- * GetFirstLastVisiblePane --
- *
- *	Given panedwindow, find the index of the first and last visible panes
- *	of that paned window.
- *
- * Results:
- *	Index of the first and last visible panes.
- *
- * Side effects:
- *	None.
- *
- *----------------------------------------------------------------------
- */
-
-static void
-GetFirstLastVisiblePane(
-    PanedWindow *pwPtr,		/* Pointer to the paned window info. */
-    int *firstPtr, 		/* Returned index for first. */
-    int *lastPtr)  		/* Returned index for last. */
-{
-    int i;
-
-    for (i = 0, *lastPtr = 0, *firstPtr = -1; i < pwPtr->numSlaves; i++) {
-	if (pwPtr->slaves[i]->hide == 0) {
-	    if (*firstPtr < 0) {
-		*firstPtr = i;
-	    }
-	    *lastPtr = i;
-	}
-    }
-}
-
 /*
  *--------------------------------------------------------------
  *
@@ -2134,13 +2077,13 @@ SlaveStructureProc(
     ClientData clientData,	/* Pointer to record describing window item. */
     XEvent *eventPtr)		/* Describes what just happened. */
 {
-    Slave *slavePtr = (Slave *) clientData;
+    Slave *slavePtr = clientData;
     PanedWindow *pwPtr = slavePtr->masterPtr;
 
     if (eventPtr->type == DestroyNotify) {
 	Unlink(slavePtr);
 	slavePtr->tkwin = NULL;
-	ckfree((void *)slavePtr);
+	ckfree(slavePtr);
 	ComputeGeometry(pwPtr);
     }
 }
@@ -2319,7 +2262,7 @@ ComputeGeometry(
     Tk_GeometryRequest(pwPtr->tkwin, reqWidth, reqHeight);
     if (Tk_IsMapped(pwPtr->tkwin) && !(pwPtr->flags & REDRAW_PENDING)) {
 	pwPtr->flags |= REDRAW_PENDING;
-	Tcl_DoWhenIdle(DisplayPanedWindow, (ClientData) pwPtr);
+	Tcl_DoWhenIdle(DisplayPanedWindow, pwPtr);
     }
 }
 
@@ -2346,8 +2289,7 @@ DestroyOptionTables(
     ClientData clientData,	/* Pointer to the OptionTables struct */
     Tcl_Interp *interp)		/* Pointer to the calling interp */
 {
-    ckfree((char *)clientData);
-    return;
+    ckfree(clientData);
 }
 
 /*
@@ -2376,22 +2318,22 @@ GetSticky(
 				 * sticky value. */
 {
     int sticky = *(int *)(recordPtr + internalOffset);
-    static char buffer[5];
-    int count = 0;
+    char buffer[5];
+    char *p = &buffer[0];
 
     if (sticky & STICK_NORTH) {
-	buffer[count++] = 'n';
+	*p++ = 'n';
     }
     if (sticky & STICK_EAST) {
-	buffer[count++] = 'e';
+	*p++ = 'e';
     }
     if (sticky & STICK_SOUTH) {
-	buffer[count++] = 's';
+	*p++ = 's';
     }
     if (sticky & STICK_WEST) {
-	buffer[count++] = 'w';
+	*p++ = 'w';
     }
-    buffer[count] = '\0';
+    *p = '\0';
 
     return Tcl_NewStringObj(buffer, -1);
 }
@@ -2430,7 +2372,8 @@ SetSticky(
     int flags)			/* Flags for the option, set Tk_SetOptions. */
 {
     int sticky = 0;
-    char c, *string, *internalPtr;
+    char c, *internalPtr;
+    const char *string;
 
     internalPtr = ComputeSlotAddress(recordPtr, internalOffset);
 
@@ -2460,10 +2403,11 @@ SetSticky(
 	    case ' ': case ',': case '\t': case '\r': case '\n':
 		break;
 	    default:
-		Tcl_ResetResult(interp);
-		Tcl_AppendResult(interp, "bad stickyness value \"",
-			Tcl_GetString(*value), "\": must be a string ",
-			"containing zero or more of n, e, s, and w", NULL);
+		Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+			"bad stickyness value \"%s\": must be a string"
+			" containing zero or more of n, e, s, and w",
+			Tcl_GetString(*value)));
+		Tcl_SetErrorCode(interp, "TK", "VALUE", "STICKY", NULL);
 		return TCL_ERROR;
 	    }
 	}
@@ -2609,7 +2553,7 @@ MoveSash(
 	    slavePtr->paneWidth = slavePtr->width = slavePtr->sashx
 		    - sashOffset - slavePtr->x - (2 * slavePtr->padx);
 	} else {
-	    slavePtr->paneHeight = slavePtr->height = slavePtr->sashy
+	    slavePtr->paneWidth = slavePtr->height = slavePtr->sashy
 		    - sashOffset - slavePtr->y - (2 * slavePtr->pady);
 	}
     }
@@ -2716,7 +2660,7 @@ MoveSash(
  *	None.
  *
  * Side effects:
- *	When the window gets deleted, internal structures get cleaned up. Whena
+ *	When the window gets deleted, internal structures get cleaned up. When
  *	it gets exposed, it is redisplayed.
  *
  *--------------------------------------------------------------
@@ -2727,11 +2671,11 @@ ProxyWindowEventProc(
     ClientData clientData,	/* Information about window. */
     XEvent *eventPtr)		/* Information about event. */
 {
-    PanedWindow *pwPtr = (PanedWindow *) clientData;
+    PanedWindow *pwPtr = clientData;
 
     if (eventPtr->type == Expose) {
 	if (pwPtr->proxywin != NULL &&!(pwPtr->flags & PROXY_REDRAW_PENDING)) {
-	    Tcl_DoWhenIdle(DisplayProxyWindow, (ClientData) pwPtr);
+	    Tcl_DoWhenIdle(DisplayProxyWindow, pwPtr);
 	    pwPtr->flags |= PROXY_REDRAW_PENDING;
 	}
     }
@@ -2759,7 +2703,7 @@ static void
 DisplayProxyWindow(
     ClientData clientData)	/* Information about window. */
 {
-    PanedWindow *pwPtr = (PanedWindow *) clientData;
+    PanedWindow *pwPtr = clientData;
     Pixmap pixmap;
     Tk_Window tkwin = pwPtr->proxywin;
     pwPtr->flags &= ~PROXY_REDRAW_PENDING;
@@ -2782,10 +2726,8 @@ DisplayProxyWindow(
      * Redraw the widget's background and border.
      */
 
-    Tk_Fill3DRectangle(tkwin, pixmap,
-	    pwPtr->proxyBackground ? pwPtr->proxyBackground : pwPtr->background,
-	    0, 0, Tk_Width(tkwin), Tk_Height(tkwin), pwPtr->proxyBorderWidth,
-	    (pwPtr->proxyRelief != TK_RELIEF_NULL) ? pwPtr->proxyRelief : pwPtr->sashRelief);
+    Tk_Fill3DRectangle(tkwin, pixmap, pwPtr->background, 0, 0,
+	    Tk_Width(tkwin), Tk_Height(tkwin), 2, pwPtr->sashRelief);
 
 #ifndef TK_NO_DOUBLE_BUFFERING
     /*
@@ -2822,14 +2764,13 @@ PanedWindowProxyCommand(
     int objc,			/* Number of arguments. */
     Tcl_Obj *const objv[])	/* Argument objects. */
 {
-    static const char *optionStrings[] = {
+    static const char *const optionStrings[] = {
 	"coord", "forget", "place", NULL
     };
     enum options {
 	PROXY_COORD, PROXY_FORGET, PROXY_PLACE
     };
     int index, x, y, sashWidth, sashHeight;
-    int internalBW, pwWidth, pwHeight;
     Tcl_Obj *coords[2];
 
     if (objc < 3) {
@@ -2851,7 +2792,7 @@ PanedWindowProxyCommand(
 
 	coords[0] = Tcl_NewIntObj(pwPtr->proxyx);
 	coords[1] = Tcl_NewIntObj(pwPtr->proxyy);
-	Tcl_SetListObj(Tcl_GetObjResult(interp), 2, coords);
+	Tcl_SetObjResult(interp, Tcl_NewListObj(2, coords));
 	break;
 
     case PROXY_FORGET:
@@ -2879,16 +2820,11 @@ PanedWindowProxyCommand(
 	    return TCL_ERROR;
 	}
 
-        internalBW = Tk_InternalBorderWidth(pwPtr->tkwin);
 	if (pwPtr->orient == ORIENT_HORIZONTAL) {
 	    if (x < 0) {
 		x = 0;
 	    }
-            pwWidth = Tk_Width(pwPtr->tkwin) - (2 * internalBW);
-            if (x > pwWidth) {
-                x = pwWidth;
-            }
-            y = Tk_InternalBorderWidth(pwPtr->tkwin);
+	    y = Tk_InternalBorderWidth(pwPtr->tkwin);
 	    sashWidth = pwPtr->sashWidth;
 	    sashHeight = Tk_Height(pwPtr->tkwin) -
 		    (2 * Tk_InternalBorderWidth(pwPtr->tkwin));
@@ -2896,22 +2832,18 @@ PanedWindowProxyCommand(
 	    if (y < 0) {
 		y = 0;
 	    }
-            pwHeight = Tk_Height(pwPtr->tkwin) - (2 * internalBW);
-            if (y > pwHeight) {
-                y = pwHeight;
-            }
 	    x = Tk_InternalBorderWidth(pwPtr->tkwin);
 	    sashHeight = pwPtr->sashWidth;
 	    sashWidth = Tk_Width(pwPtr->tkwin) -
 		    (2 * Tk_InternalBorderWidth(pwPtr->tkwin));
 	}
 
-        if (sashWidth < 1) {
-            sashWidth = 1;
-        }
-        if (sashHeight < 1) {
-            sashHeight = 1;
-        }
+	if (sashWidth < 1) {
+	    sashWidth = 1;
+	}
+	if (sashHeight < 1) {
+	    sashHeight = 1;
+	}
 
 	/*
 	 * Stash the proxy coordinates for future "proxy coord" calls.
@@ -3032,11 +2964,8 @@ PanedWindowIdentifyCoords(
     Tcl_Interp *interp,		/* Interpreter in which to store result. */
     int x, int y)		/* Coordinates of the point to identify. */
 {
-    Tcl_Obj *list;
     int i, sashHeight, sashWidth, thisx, thisy;
     int found, isHandle, lpad, rpad, tpad, bpad;
-    int first, last;
-    list = Tcl_NewObj();
 
     if (pwPtr->orient == ORIENT_HORIZONTAL) {
 	if (Tk_IsMapped(pwPtr->tkwin)) {
@@ -3076,11 +3005,10 @@ PanedWindowIdentifyCoords(
 	lpad = rpad = 0;
     }
 
-    GetFirstLastVisiblePane(pwPtr, &first, &last);
     isHandle = 0;
     found = -1;
     for (i = 0; i < pwPtr->numSlaves - 1; i++) {
-	if (pwPtr->slaves[i]->hide || i == last) {
+	if (pwPtr->slaves[i]->hide) {
 	    continue;
 	}
 	thisx = pwPtr->slaves[i]->sashx;
@@ -3112,16 +3040,17 @@ PanedWindowIdentifyCoords(
     }
 
     /*
-     * Set results.
+     * Set results. Note that the empty string is the default (this function
+     * is called inside the implementation of a command).
      */
 
     if (found != -1) {
-	Tcl_ListObjAppendElement(interp, list, Tcl_NewIntObj(found));
-	Tcl_ListObjAppendElement(interp, list, Tcl_NewStringObj(
-		(isHandle ? "handle" : "sash"), -1));
-    }
+	Tcl_Obj *list[2];
 
-    Tcl_SetObjResult(interp, list);
+	list[0] = Tcl_NewIntObj(found);
+	list[1] = Tcl_NewStringObj((isHandle ? "handle" : "sash"), -1);
+	Tcl_SetObjResult(interp, Tcl_NewListObj(2, list));
+    }
     return TCL_OK;
 }
 
