@@ -18,11 +18,6 @@
 #include "tkInt.h"
 #endif
 
-#ifdef BUILD_tk
-# undef TCL_STORAGE_CLASS
-# define TCL_STORAGE_CLASS DLLEXPORT
-#endif
-
 /*
  * Legal values for the "orient" field of TkScale records.
  */
@@ -219,6 +214,12 @@ typedef struct TkScale {
 
 #define SPACING 2
 
+/*
+ * How many characters of space to provide when formatting the scale's value:
+ */
+
+#define PRINT_CHARS 150
+
 /*
  * Declaration of procedures used in the implementation of the scale widget.
  */
@@ -234,7 +235,4 @@ MODULE_SCOPE void	TkScaleSetValue(TkScale *scalePtr, double value,
 MODULE_SCOPE double	TkScalePixelToValue(TkScale *scalePtr, int x, int y);
 MODULE_SCOPE int	TkScaleValueToPixel(TkScale *scalePtr, double value);
 
-# undef TCL_STORAGE_CLASS
-# define TCL_STORAGE_CLASS DLLIMPORT
-
 #endif /* _TKSCALE */
