@@ -30,69 +30,40 @@
 #include "tkPlatDecls.h"
 #include "tkIntXlibDecls.h"
 
-#define TkUnusedStubEntry NULL
+static const TkIntStubs tkIntStubs;
+
+/*
+ * Remove macro that might interfere with the definition below.
+ */
+
+#undef Tk_MainEx
 
 #ifdef __WIN32__
 
-static int
-doNothing(void)
+int
+TkpCmapStressed(Tk_Window tkwin, Colormap colormap)
 {
     /* dummy implementation, no need to do anything */
     return 0;
 }
+void
+TkpSync(Display *display)
+{
+    /* dummy implementation, no need to do anything */
+}
 
-#define TkCreateXEventSource TkPlatCreateXEventSource
-static void
+void
 TkCreateXEventSource(void)
 {
-	TkWinXInit(Tk_GetHINSTANCE());
+    TkWinXInit(Tk_GetHINSTANCE());
 }
 
-#undef XFree
-#define XFree TkPlatXFree
-static int
-XFree(void *data)
-{
-	if (data != NULL) {
-		ckfree((char *) data);
-	}
-	return 0;
-}
-
-#undef XVisualIDFromVisual
-#define XVisualIDFromVisual TkPlatXVisualIDFromVisual
-static VisualID
-XVisualIDFromVisual(Visual *visual)
-{
-    return visual->visualid;
-}
-
-/*
- * Remove macros that will interfere with the definitions below.
- */
-#   undef TkpCmapStressed
-#   undef TkpSync
-#   undef XFlush
-#   undef XGrabServer
-#   undef XUngrabServer
-#   undef XNoOp
-#   undef XSynchronize
-#   undef XSync
-
-#   define TkpCmapStressed (int (*) (Tk_Window, Colormap)) doNothing
-#   define TkpSync (void (*) (Display *)) doNothing
 #   define TkUnixContainerId 0
 #   define TkUnixDoOneXEvent 0
 #   define TkUnixSetMenubar 0
-#   define TkWmCleanup (void (*) (TkDisplay *)) doNothing
-#   define TkSendCleanup (void (*) (TkDisplay *)) doNothing
+#   define TkWmCleanup (void (*)(TkDisplay *)) TkpSync
+#   define TkSendCleanup (void (*)(TkDisplay *)) TkpSync
 #   define TkpTestsendCmd 0
-#   define XFlush (int (*) (Display *)) doNothing
-#   define XGrabServer (int (*) (Display *)) doNothing
-#   define XUngrabServer (int (*) (Display *)) doNothing
-#   define XNoOp (int (*) (Display *)) doNothing
-#   define XSynchronize (XAfterFunction (*) (Display *, Bool)) doNothing
-#   define XSync (int (*) (Display *, Bool)) doNothing
 
 #else /* !__WIN32__ */
 
@@ -104,8 +75,6 @@ XVisualIDFromVisual(Visual *visual)
 
 #   ifdef __CYGWIN__
 
-	TkIntStubs tkIntStubs;
-
 /*
  * Trick, so we don't have to include <windows.h> here, which in any
  * case lacks this function anyway.
@@ -136,7 +105,7 @@ TkpPrintWindowId(
 				 * the hex representation of a pointer. */
     Window window)		/* Window to be printed into buffer. */
 {
-	sprintf(buf, "%#08lx", (unsigned long) (window));
+    sprintf(buf, "%#08lx", (unsigned long) (window));
 }
 
 int
@@ -235,8 +204,6 @@ void TkSubtractRegion (TkRegion a, TkRegion b, TkRegion c)
 #	define TkWinGetPlatformTheme 0
 #	define TkWinChildProc 0
 
-#	define TkBindDeadWindow 0 /* On purpose not in Cygwin's stub table */
-
 #   elif !defined(MAC_OSX_TK) /* UNIX */
 
 #	undef TkClipBox
@@ -259,6 +226,9 @@ void TkSubtractRegion (TkRegion a, TkRegion b, TkRegion c)
 #   endif
 #endif /* !__WIN32__ */
 
+#define reserved274 (void (*)(void)) Tk_DefineBitmap
+#define reserved275 (void (*)(void)) Tk_GetBitmapFromData
+
 /*
  * WARNING: The contents of this file is automatically generated by the
  * tools/genStubs.tcl script. Any modifications to the function declarations
@@ -267,21 +237,21 @@ void TkSubtractRegion (TkRegion a, TkRegion b, TkRegion c)
 
 /* !BEGIN!: Do not edit below this line. */
 
-TkIntStubs tkIntStubs = {
+static const TkIntStubs tkIntStubs = {
     TCL_STUB_MAGIC,
-    NULL,
+    0,
     TkAllocWindow, /* 0 */
     TkBezierPoints, /* 1 */
     TkBezierScreenPoints, /* 2 */
-    TkBindDeadWindow, /* 3 */
+    0, /* 3 */
     TkBindEventProc, /* 4 */
     TkBindFree, /* 5 */
     TkBindInit, /* 6 */
     TkChangeEventWindow, /* 7 */
     TkClipInit, /* 8 */
     TkComputeAnchor, /* 9 */
-    TkCopyAndGlobalEval, /* 10 */
-    TkCreateBindingProcedure, /* 11 */
+    0, /* 10 */
+    0, /* 11 */
     TkCreateCursorFromData, /* 12 */
     TkCreateFrame, /* 13 */
     TkCreateMainWindow, /* 14 */
@@ -346,7 +316,7 @@ TkIntStubs tkIntStubs = {
     TkpRedirectKeyEvent, /* 73 */
     TkpSetMainMenubar, /* 74 */
     TkpUseWindow, /* 75 */
-    TkpWindowWasRecentlyDeleted, /* 76 */
+    0, /* 76 */
     TkQueueEventForAllChildren, /* 77 */
     TkReadBitmapFile, /* 78 */
     TkScrollWindow, /* 79 */
@@ -354,7 +324,7 @@ TkIntStubs tkIntStubs = {
     TkSelEventProc, /* 81 */
     TkSelInit, /* 82 */
     TkSelPropProc, /* 83 */
-    NULL, /* 84 */
+    0, /* 84 */
     TkSetWindowMenuBar, /* 85 */
     TkStringToKeysym, /* 86 */
     TkThickPolyLineToArea, /* 87 */
@@ -390,48 +360,48 @@ TkIntStubs tkIntStubs = {
     TkRectInRegion, /* 117 */
     TkSetRegion, /* 118 */
     TkUnionRectWithRegion, /* 119 */
-    NULL, /* 120 */
+    0, /* 120 */
 #if !(defined(__WIN32__) || defined(MAC_OSX_TK)) /* X11 */
-    NULL, /* 121 */
+    0, /* 121 */
 #endif /* X11 */
 #if defined(__WIN32__) /* WIN */
-    NULL, /* 121 */
+    0, /* 121 */
 #endif /* WIN */
 #ifdef MAC_OSX_TK /* AQUA */
-    NULL, /* 121 */ /* Dummy entry for stubs table backwards compatibility */
+    0, /* 121 */ /* Dummy entry for stubs table backwards compatibility */
     TkpCreateNativeBitmap, /* 121 */
 #endif /* AQUA */
 #if !(defined(__WIN32__) || defined(MAC_OSX_TK)) /* X11 */
-    NULL, /* 122 */
+    0, /* 122 */
 #endif /* X11 */
 #if defined(__WIN32__) /* WIN */
-    NULL, /* 122 */
+    0, /* 122 */
 #endif /* WIN */
 #ifdef MAC_OSX_TK /* AQUA */
-    NULL, /* 122 */ /* Dummy entry for stubs table backwards compatibility */
+    0, /* 122 */ /* Dummy entry for stubs table backwards compatibility */
     TkpDefineNativeBitmaps, /* 122 */
 #endif /* AQUA */
-    NULL, /* 123 */
+    0, /* 123 */
 #if !(defined(__WIN32__) || defined(MAC_OSX_TK)) /* X11 */
-    NULL, /* 124 */
+    0, /* 124 */
 #endif /* X11 */
 #if defined(__WIN32__) /* WIN */
-    NULL, /* 124 */
+    0, /* 124 */
 #endif /* WIN */
 #ifdef MAC_OSX_TK /* AQUA */
-    NULL, /* 124 */ /* Dummy entry for stubs table backwards compatibility */
+    0, /* 124 */ /* Dummy entry for stubs table backwards compatibility */
     TkpGetNativeAppBitmap, /* 124 */
 #endif /* AQUA */
-    NULL, /* 125 */
-    NULL, /* 126 */
-    NULL, /* 127 */
-    NULL, /* 128 */
-    NULL, /* 129 */
-    NULL, /* 130 */
-    NULL, /* 131 */
-    NULL, /* 132 */
-    NULL, /* 133 */
-    NULL, /* 134 */
+    0, /* 125 */
+    0, /* 126 */
+    0, /* 127 */
+    0, /* 128 */
+    0, /* 129 */
+    0, /* 130 */
+    0, /* 131 */
+    0, /* 132 */
+    0, /* 133 */
+    0, /* 134 */
     TkpDrawHighlightBorder, /* 135 */
     TkSetFocusWin, /* 136 */
     TkpSetKeycodeAndState, /* 137 */
@@ -452,20 +422,20 @@ TkIntStubs tkIntStubs = {
     TkpDrawFrame, /* 152 */
     TkCreateThreadExitHandler, /* 153 */
     TkDeleteThreadExitHandler, /* 154 */
-    NULL, /* 155 */
+    0, /* 155 */
     TkpTestembedCmd, /* 156 */
     TkpTesttextCmd, /* 157 */
-    NULL, /* 158 */
-    NULL, /* 159 */
-    NULL, /* 160 */
-    NULL, /* 161 */
-    NULL, /* 162 */
-    NULL, /* 163 */
-    NULL, /* 164 */
-    NULL, /* 165 */
-    NULL, /* 166 */
-    NULL, /* 167 */
-    NULL, /* 168 */
+    TkSelGetSelection, /* 158 */
+    TkTextGetIndex, /* 159 */
+    TkTextIndexBackBytes, /* 160 */
+    TkTextIndexForwBytes, /* 161 */
+    TkTextMakeByteIndex, /* 162 */
+    TkTextPrintIndex, /* 163 */
+    TkTextSetMark, /* 164 */
+    TkTextXviewCmd, /* 165 */
+    TkTextChanged, /* 166 */
+    TkBTreeNumLines, /* 167 */
+    TkTextInsertDisplayProc, /* 168 */
     TkStateParseProc, /* 169 */
     TkStatePrintProc, /* 170 */
     TkCanvasDashParseProc, /* 171 */
@@ -478,18 +448,18 @@ TkIntStubs tkIntStubs = {
     TkOrientPrintProc, /* 178 */
     TkSmoothParseProc, /* 179 */
     TkSmoothPrintProc, /* 180 */
-    NULL, /* 181 */
-    NULL, /* 182 */
-    NULL, /* 183 */
-    TkUnusedStubEntry, /* 184 */
+    TkDrawAngledTextLayout, /* 181 */
+    TkUnderlineAngledTextLayout, /* 182 */
+    TkIntersectAngledTextLayout, /* 183 */
+    TkDrawAngledChars, /* 184 */
 };
 
-TkIntPlatStubs tkIntPlatStubs = {
+static const TkIntPlatStubs tkIntPlatStubs = {
     TCL_STUB_MAGIC,
-    NULL,
+    0,
 #if defined(__WIN32__) || defined(__CYGWIN__) /* WIN */
     TkAlignImageData, /* 0 */
-    NULL, /* 1 */
+    0, /* 1 */
     TkGenerateActivateEvents, /* 2 */
     TkpGetMS, /* 3 */
     TkPointerDeadWindow, /* 4 */
@@ -537,8 +507,8 @@ TkIntPlatStubs tkIntPlatStubs = {
 #endif /* WIN */
 #ifdef MAC_OSX_TK /* AQUA */
     TkGenerateActivateEvents, /* 0 */
-    NULL, /* 1 */
-    NULL, /* 2 */
+    0, /* 1 */
+    0, /* 2 */
     TkPointerDeadWindow, /* 3 */
     TkpSetCapture, /* 4 */
     TkpSetCursor, /* 5 */
@@ -549,14 +519,14 @@ TkIntPlatStubs tkIntPlatStubs = {
     TkMacOSXDispatchMenuEvent, /* 10 */
     TkMacOSXInstallCursor, /* 11 */
     TkMacOSXHandleTearoffMenu, /* 12 */
-    NULL, /* 13 */
+    0, /* 13 */
     TkMacOSXDoHLEvent, /* 14 */
-    NULL, /* 15 */
+    0, /* 15 */
     TkMacOSXGetXWindow, /* 16 */
     TkMacOSXGrowToplevel, /* 17 */
     TkMacOSXHandleMenuSelect, /* 18 */
-    NULL, /* 19 */
-    NULL, /* 20 */
+    0, /* 19 */
+    0, /* 20 */
     TkMacOSXInvalidateWindow, /* 21 */
     TkMacOSXIsCharacterMissing, /* 22 */
     TkMacOSXMakeRealWindowExist, /* 23 */
@@ -584,19 +554,18 @@ TkIntPlatStubs tkIntPlatStubs = {
     TkMacOSXPreprocessMenu, /* 45 */
     TkpIsWindowFloating, /* 46 */
     TkMacOSXGetCapture, /* 47 */
-    NULL, /* 48 */
+    0, /* 48 */
     TkGetTransientMaster, /* 49 */
     TkGenerateButtonEvent, /* 50 */
     TkGenWMDestroyEvent, /* 51 */
-    TkMacOSXSetDrawingEnabled, /* 52 */
+    0, /* 52 */
     TkpGetMS, /* 53 */
     TkMacOSXDrawable, /* 54 */
-    TkpScanWindowId, /* 55 */
 #endif /* AQUA */
 #if !(defined(__WIN32__) || defined(__CYGWIN__) || defined(MAC_OSX_TK)) /* X11 */
     TkCreateXEventSource, /* 0 */
-    TkFreeWindowId, /* 1 */
-    TkInitXId, /* 2 */
+    0, /* 1 */
+    0, /* 2 */
     TkpCmapStressed, /* 3 */
     TkpSync, /* 4 */
     TkUnixContainerId, /* 5 */
@@ -605,15 +574,15 @@ TkIntPlatStubs tkIntPlatStubs = {
     TkpScanWindowId, /* 8 */
     TkWmCleanup, /* 9 */
     TkSendCleanup, /* 10 */
-    TkFreeXId, /* 11 */
+    0, /* 11 */
     TkpWmSetState, /* 12 */
     TkpTestsendCmd, /* 13 */
 #endif /* X11 */
 };
 
-TkIntXlibStubs tkIntXlibStubs = {
+static const TkIntXlibStubs tkIntXlibStubs = {
     TCL_STUB_MAGIC,
-    NULL,
+    0,
 #if defined(__WIN32__) || defined(__CYGWIN__) /* WIN */
     XSetDashes, /* 0 */
     XGetModifierMapping, /* 1 */
@@ -696,7 +665,7 @@ TkIntXlibStubs tkIntXlibStubs = {
     XFilterEvent, /* 78 */
     XmbLookupString, /* 79 */
     TkPutImage, /* 80 */
-    NULL, /* 81 */
+    0, /* 81 */
     XParseColor, /* 82 */
     XCreateGC, /* 83 */
     XFreeGC, /* 84 */
@@ -827,9 +796,9 @@ TkIntXlibStubs tkIntXlibStubs = {
 #endif /* AQUA */
 };
 
-TkPlatStubs tkPlatStubs = {
+static const TkPlatStubs tkPlatStubs = {
     TCL_STUB_MAGIC,
-    NULL,
+    0,
 #if defined(__WIN32__) || defined(__CYGWIN__) /* WIN */
     Tk_AttachHWND, /* 0 */
     Tk_GetHINSTANCE, /* 1 */
@@ -853,14 +822,14 @@ TkPlatStubs tkPlatStubs = {
 #endif /* AQUA */
 };
 
-static TkStubHooks tkStubHooks = {
+static const TkStubHooks tkStubHooks = {
     &tkPlatStubs,
     &tkIntStubs,
     &tkIntPlatStubs,
     &tkIntXlibStubs
 };
 
-TkStubs tkStubs = {
+const TkStubs tkStubs = {
     TCL_STUB_MAGIC,
     &tkStubHooks,
     Tk_MainLoop, /* 0 */
@@ -1081,8 +1050,8 @@ TkStubs tkStubs = {
     Tk_InitConsoleChannels, /* 215 */
     Tk_CreateConsoleWindow, /* 216 */
     Tk_CreateSmoothMethod, /* 217 */
-    NULL, /* 218 */
-    NULL, /* 219 */
+    0, /* 218 */
+    0, /* 219 */
     Tk_GetDash, /* 220 */
     Tk_CreateOutline, /* 221 */
     Tk_DeleteOutline, /* 222 */
@@ -1137,11 +1106,8 @@ TkStubs tkStubs = {
     Tk_Interp, /* 271 */
     Tk_CreateOldImageType, /* 272 */
     Tk_CreateOldPhotoImageFormat, /* 273 */
-    NULL, /* 274 */
-    TkUnusedStubEntry, /* 275 */
+    reserved274, /* 274 */
+    reserved275, /* 275 */
 };
 
 /* !END!: Do not edit above this line. */
-
-#undef UNIX_TK
-#undef MAC_OSX_TK
