@@ -33,16 +33,13 @@
  * Forward declarations for functions defined later in this file:
  */
 
-static CONST char *	ForwBack(TkText *textPtr, CONST char *string,
+static const char *	ForwBack(TkText *textPtr, const char *string,
 			    TkTextIndex *indexPtr);
-static CONST char *	StartEnd(TkText *textPtr, CONST char *string,
+static const char *	StartEnd(TkText *textPtr, const char *string,
 			    TkTextIndex *indexPtr);
 static int		GetIndex(Tcl_Interp *interp, TkSharedText *sharedPtr,
-			    TkText *textPtr, CONST char *string,
+			    TkText *textPtr, const char *string,
 			    TkTextIndex *indexPtr, int *canCachePtr);
-static int              IndexCountBytesOrdered(CONST TkText *textPtr,
-                            CONST TkTextIndex *indexPtr1,
-                            CONST TkTextIndex *indexPtr2);
 
 /*
  * The "textindex" Tcl_Obj definition:
@@ -64,39 +61,41 @@ static void		UpdateStringOfTextIndex(Tcl_Obj *objPtr);
 #define GET_INDEXEPOCH(objPtr) \
 	(PTR2INT((objPtr)->internalRep.twoPtrValue.ptr2))
 #define SET_TEXTINDEX(objPtr, indexPtr) \
-	((objPtr)->internalRep.twoPtrValue.ptr1 = (VOID *) (indexPtr))
+	((objPtr)->internalRep.twoPtrValue.ptr1 = (void *) (indexPtr))
 #define SET_INDEXEPOCH(objPtr, epoch) \
 	((objPtr)->internalRep.twoPtrValue.ptr2 = INT2PTR(epoch))
-
+
 /*
  * Define the 'textindex' object type, which Tk uses to represent indices in
  * text widgets internally.
  */
 
-Tcl_ObjType tkTextIndexType = {
+const Tcl_ObjType tkTextIndexType = {
     "textindex",		/* name */
     FreeTextIndexInternalRep,	/* freeIntRepProc */
     DupTextIndexInternalRep,	/* dupIntRepProc */
     NULL,			/* updateStringProc */
     SetTextIndexFromAny		/* setFromAnyProc */
 };
-
+
 static void
 FreeTextIndexInternalRep(
     Tcl_Obj *indexObjPtr)	/* TextIndex object with internal rep to
 				 * free. */
 {
     TkTextIndex *indexPtr = GET_TEXTINDEX(indexObjPtr);
+
     if (indexPtr->textPtr != NULL) {
 	if (--indexPtr->textPtr->refCount == 0) {
 	    /*
 	     * The text widget has been deleted and we need to free it now.
 	     */
 
-	    ckfree((char *) (indexPtr->textPtr));
+	    ckfree(indexPtr->textPtr);
 	}
     }
-    ckfree((char *) indexPtr);
+    ckfree(indexPtr);
+    indexObjPtr->typePtr = NULL;
 }
 
 static void
@@ -107,7 +106,7 @@ DupTextIndexInternalRep(
     int epoch;
     TkTextIndex *dupIndexPtr, *indexPtr;
 
-    dupIndexPtr = (TkTextIndex *) ckalloc(sizeof(TkTextIndex));
+    dupIndexPtr = ckalloc(sizeof(TkTextIndex));
     indexPtr = GET_TEXTINDEX(srcPtr);
     epoch = GET_INDEXEPOCH(srcPtr);
 
@@ -122,7 +121,7 @@ DupTextIndexInternalRep(
     SET_INDEXEPOCH(copyPtr, epoch);
     copyPtr->typePtr = &tkTextIndexType;
 }
-
+
 /*
  * This will not be called except by TkTextNewIndexObj below. This is because
  * if a TkTextIndex is no longer valid, it is not possible to regenerate the
@@ -135,12 +134,11 @@ UpdateStringOfTextIndex(
 {
     char buffer[TK_POS_CHARS];
     register int len;
-
-    CONST TkTextIndex *indexPtr = GET_TEXTINDEX(objPtr);
+    const TkTextIndex *indexPtr = GET_TEXTINDEX(objPtr);
 
     len = TkTextPrintIndex(indexPtr->textPtr, indexPtr, buffer);
 
-    objPtr->bytes = ckalloc((unsigned) len + 1);
+    objPtr->bytes = ckalloc(len + 1);
     strcpy(objPtr->bytes, buffer);
     objPtr->length = len;
 }
@@ -150,9 +148,10 @@ SetTextIndexFromAny(
     Tcl_Interp *interp,		/* Used for error reporting if not NULL. */
     Tcl_Obj *objPtr)		/* The object to convert. */
 {
-    Tcl_AppendToObj(Tcl_GetObjResult(interp),
-	    "can't convert value to textindex except via TkTextGetIndexFromObj API",
-	    -1);
+    Tcl_SetObjResult(interp, Tcl_NewStringObj(
+	    "can't convert value to textindex except via"
+	    " TkTextGetIndexFromObj API", -1));
+    Tcl_SetErrorCode(interp, "TK", "API_ABUSE", NULL);
     return TCL_ERROR;
 }
 
@@ -186,9 +185,9 @@ MakeObjIndex(
     TkText *textPtr,		/* Information about text widget. */
     Tcl_Obj *objPtr,		/* Object containing description of
 				 * position. */
-    CONST TkTextIndex *origPtr)	/* Pointer to index. */
+    const TkTextIndex *origPtr)	/* Pointer to index. */
 {
-    TkTextIndex *indexPtr = (TkTextIndex *) ckalloc(sizeof(TkTextIndex));
+    TkTextIndex *indexPtr = ckalloc(sizeof(TkTextIndex));
 
     indexPtr->tree = origPtr->tree;
     indexPtr->linePtr = origPtr->linePtr;
@@ -205,8 +204,8 @@ MakeObjIndex(
     }
     return indexPtr;
 }
-
-CONST TkTextIndex *
+
+const TkTextIndex *
 TkTextGetIndexFromObj(
     Tcl_Interp *interp,		/* Use this for error reporting. */
     TkText *textPtr,		/* Information about text widget. */
@@ -245,8 +244,8 @@ TkTextGetIndexFromObj(
 	if (objPtr->bytes == NULL) {
 	    objPtr->typePtr->updateStringProc(objPtr);
 	}
-	if ((objPtr->typePtr->freeIntRepProc) != NULL) {
-	    (*objPtr->typePtr->freeIntRepProc)(objPtr);
+	if (objPtr->typePtr->freeIntRepProc != NULL) {
+	    objPtr->typePtr->freeIntRepProc(objPtr);
 	}
     }
 
@@ -274,7 +273,7 @@ TkTextGetIndexFromObj(
 Tcl_Obj *
 TkTextNewIndexObj(
     TkText *textPtr,		/* Text widget for this index */
-    CONST TkTextIndex *indexPtr)/* Pointer to index. */
+    const TkTextIndex *indexPtr)/* Pointer to index. */
 {
     Tcl_Obj *retVal;
 
@@ -388,9 +387,9 @@ TkTextMakePixelIndex(
 
 TkTextIndex *
 TkTextMakeByteIndex(
-    TkTextBTree tree,		/* Tree that lineIndex and byteIndex refer
+    TkTextBTree tree,	/* Tree that lineIndex and byteIndex refer
 				 * to. */
-    CONST TkText *textPtr,
+    const TkText *textPtr,
     int lineIndex,		/* Index of desired line (0 means first line
 				 * of text). */
     int byteIndex,		/* Byte index of desired character. */
@@ -398,7 +397,7 @@ TkTextMakeByteIndex(
 {
     TkTextSegment *segPtr;
     int index;
-    CONST char *p, *start;
+    const char *p, *start;
     Tcl_UniChar ch;
 
     indexPtr->tree = tree;
@@ -442,7 +441,7 @@ TkTextMakeByteIndex(
 	    if ((byteIndex > index) && (segPtr->typePtr == &tkTextCharType)) {
 		/*
 		 * Prevent UTF-8 character from being split up by ensuring
-		 * that byteIndex falls on a character boundary. If index
+		 * that byteIndex falls on a character boundary. If the index
 		 * falls in the middle of a UTF-8 character, it will be
 		 * adjusted to the end of that UTF-8 character.
 		 */
@@ -576,7 +575,7 @@ TkTextMakeCharIndex(
 
 TkTextSegment *
 TkTextIndexToSeg(
-    CONST TkTextIndex *indexPtr,/* Text index. */
+    const TkTextIndex *indexPtr,/* Text index. */
     int *offsetPtr)		/* Where to store offset within segment, or
 				 * NULL if offset isn't wanted. */
 {
@@ -614,10 +613,10 @@ TkTextIndexToSeg(
 
 int
 TkTextSegToOffset(
-    CONST TkTextSegment *segPtr,/* Segment whose offset is desired. */
-    CONST TkTextLine *linePtr)	/* Line containing segPtr. */
+    const TkTextSegment *segPtr,/* Segment whose offset is desired. */
+    const TkTextLine *linePtr)	/* Line containing segPtr. */
 {
-    CONST TkTextSegment *segPtr2;
+    const TkTextSegment *segPtr2;
     int offset = 0;
 
     for (segPtr2 = linePtr->segPtr; segPtr2 != segPtr;
@@ -708,7 +707,7 @@ int
 TkTextGetIndex(
     Tcl_Interp *interp,		/* Use this for error reporting. */
     TkText *textPtr,		/* Information about text widget. */
-    CONST char *string,		/* Textual description of position. */
+    const char *string,		/* Textual description of position. */
     TkTextIndex *indexPtr)	/* Index structure to fill in. */
 {
     return GetIndex(interp, NULL, textPtr, string, indexPtr, NULL);
@@ -742,7 +741,7 @@ GetIndex(
     Tcl_Interp *interp,		/* Use this for error reporting. */
     TkSharedText *sharedPtr,
     TkText *textPtr,		/* Information about text widget. */
-    CONST char *string,		/* Textual description of position. */
+    const char *string,		/* Textual description of position. */
     TkTextIndex *indexPtr,	/* Index structure to fill in. */
     int *canCachePtr)		/* Pointer to integer to store whether we can
 				 * cache the index (or NULL). */
@@ -751,7 +750,7 @@ GetIndex(
     TkTextIndex first, last;
     int wantLast, result;
     char c;
-    CONST char *cp;
+    const char *cp;
     Tcl_DString copy;
     int canCache = 0;
 
@@ -774,11 +773,11 @@ GetIndex(
     }
 
     if (TkTextWindowIndex(textPtr, string, indexPtr) != 0) {
-	goto done;
+	return TCL_OK;
     }
 
     if (TkTextImageIndex(textPtr, string, indexPtr) != 0) {
-	goto done;
+	return TCL_OK;
     }
 
     /*
@@ -803,7 +802,7 @@ GetIndex(
 	TkTextSearch search;
 	TkTextTag *tagPtr;
 	Tcl_HashEntry *hPtr = NULL;
-	CONST char *tagName;
+	const char *tagName;
 
 	if ((p[1] == 'f') && (strncmp(p+1, "first", 5) == 0)) {
 	    wantLast = 0;
@@ -828,7 +827,7 @@ GetIndex(
 	    hPtr = Tcl_FindHashEntry(&sharedPtr->tagTable, tagName);
 	    *p = '.';
 	    if (hPtr != NULL) {
-		tagPtr = (TkTextTag *) Tcl_GetHashValue(hPtr);
+		tagPtr = Tcl_GetHashValue(hPtr);
 	    }
 	}
 
@@ -843,13 +842,14 @@ GetIndex(
 	if (!TkBTreeCharTagged(&first, tagPtr) && !TkBTreeNextTag(&search)) {
 	    if (tagPtr == textPtr->selTagPtr) {
 		tagName = "sel";
-	    } else {
+	    } else if (hPtr != NULL) {
 		tagName = Tcl_GetHashKey(&sharedPtr->tagTable, hPtr);
 	    }
-	    Tcl_ResetResult(interp);
-	    Tcl_AppendResult(interp,
-		    "text doesn't contain any characters tagged with \"",
-		    tagName, "\"", NULL);
+	    Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+		    "text doesn't contain any characters tagged with \"%s\"",
+		    tagName));
+	    Tcl_SetErrorCode(interp, "TK", "LOOKUP", "TEXT_INDEX", tagName,
+		    NULL);
 	    Tcl_DStringFree(&copy);
 	    return TCL_ERROR;
 	}
@@ -1012,8 +1012,8 @@ GetIndex(
 
   error:
     Tcl_DStringFree(&copy);
-    Tcl_ResetResult(interp);
-    Tcl_AppendResult(interp, "bad text index \"", string, "\"", NULL);
+    Tcl_SetObjResult(interp, Tcl_ObjPrintf("bad text index \"%s\"", string));
+    Tcl_SetErrorCode(interp, "TK", "TEXT", "BAD_INDEX", NULL);
     return TCL_ERROR;
 }
 
@@ -1037,8 +1037,8 @@ GetIndex(
 
 int
 TkTextPrintIndex(
-    CONST TkText *textPtr,
-    CONST TkTextIndex *indexPtr,/* Pointer to index. */
+    const TkText *textPtr,
+    const TkTextIndex *indexPtr,/* Pointer to index. */
     char *string)		/* Place to store the position. Must have at
 				 * least TK_POS_CHARS characters. */
 {
@@ -1101,8 +1101,8 @@ TkTextPrintIndex(
 
 int
 TkTextIndexCmp(
-    CONST TkTextIndex*index1Ptr,/* First index. */
-    CONST TkTextIndex*index2Ptr)/* Second index. */
+    const TkTextIndex*index1Ptr,/* First index. */
+    const TkTextIndex*index2Ptr)/* Second index. */
 {
     int line1, line2;
 
@@ -1154,15 +1154,15 @@ TkTextIndexCmp(
  *---------------------------------------------------------------------------
  */
 
-static CONST char *
+static const char *
 ForwBack(
     TkText *textPtr,		/* Information about text widget. */
-    CONST char *string,		/* String to parse for additional info about
+    const char *string,		/* String to parse for additional info about
 				 * modifier (count and units). Points to "+"
 				 * or "-" that starts modifier. */
     TkTextIndex *indexPtr)	/* Index to update as specified in string. */
 {
-    register CONST char *p, *units;
+    register const char *p, *units;
     char *end;
     int count, lineIndex, modifier;
     size_t length;
@@ -1402,8 +1402,8 @@ ForwBack(
 
 int
 TkTextIndexForwBytes(
-    CONST TkText *textPtr,
-    CONST TkTextIndex *srcPtr,	/* Source index. */
+    const TkText *textPtr,
+    const TkTextIndex *srcPtr,	/* Source index. */
     int byteCount,		/* How many bytes forward to move. May be
 				 * negative. */
     TkTextIndex *dstPtr)	/* Destination index: gets modified. */
@@ -1474,8 +1474,8 @@ TkTextIndexForwBytes(
 
 void
 TkTextIndexForwChars(
-    CONST TkText *textPtr,	/* Overall information about text widget. */
-    CONST TkTextIndex *srcPtr,	/* Source index. */
+    const TkText *textPtr,	/* Overall information about text widget. */
+    const TkTextIndex *srcPtr,	/* Source index. */
     int charCount,		/* How many characters forward to move. May
 				 * be negative. */
     TkTextIndex *dstPtr,	/* Destination index: gets modified. */
@@ -1495,8 +1495,7 @@ TkTextIndexForwChars(
 	return;
     }
     if (checkElided) {
-	infoPtr = (TkTextElideInfo *)
-		ckalloc((unsigned) sizeof(TkTextElideInfo));
+	infoPtr = ckalloc(sizeof(TkTextElideInfo));
 	elide = TkTextIsElided(textPtr, srcPtr, infoPtr);
     }
 
@@ -1624,94 +1623,10 @@ TkTextIndexForwChars(
   forwardCharDone:
     if (infoPtr != NULL) {
 	TkTextFreeElideInfo(infoPtr);
-	ckfree((char *) infoPtr);
+	ckfree(infoPtr);
     }
 }
 
-/*
- *---------------------------------------------------------------------------
- *
- * TkTextIndexCountBytes --
- *
- *	Given a pair of indices in a text widget, this function counts how
- *	many bytes are between the two indices. The two indices do not need
- *	to be ordered.
- *
- * Results:
- *	The number of bytes in the given range.
- *
- * Side effects:
- *	None.
- *
- *---------------------------------------------------------------------------
- */
-
-int
-TkTextIndexCountBytes(
-    CONST TkText *textPtr,
-    CONST TkTextIndex *indexPtr1, /* Index describing one location. */
-    CONST TkTextIndex *indexPtr2) /* Index describing second location. */
-{
-    int compare = TkTextIndexCmp(indexPtr1, indexPtr2);
-
-    if (compare == 0) {
-	return 0;
-    } else if (compare > 0) {
-	return IndexCountBytesOrdered(textPtr, indexPtr2, indexPtr1);
-    } else {
-	return IndexCountBytesOrdered(textPtr, indexPtr1, indexPtr2);
-    }
-}
-
-static int
-IndexCountBytesOrdered(
-    CONST TkText *textPtr,
-    CONST TkTextIndex *indexPtr1,
-				/* Index describing location of character from
-				 * which to count. */
-    CONST TkTextIndex *indexPtr2)
-				/* Index describing location of last character
-				 * at which to stop the count. */
-{
-    int byteCount, offset;
-    TkTextSegment *segPtr, *segPtr1;
-    TkTextLine *linePtr;
-
-    if (indexPtr1->linePtr == indexPtr2->linePtr) {
-        return indexPtr2->byteIndex - indexPtr1->byteIndex;
-    }
-
-    /*
-     * indexPtr2 is on a line strictly after the line containing indexPtr1.
-     * Add up:
-     *   bytes between indexPtr1 and end of its line
-     *   bytes in lines strictly between indexPtr1 and indexPtr2
-     *   bytes between start of the indexPtr2 line and indexPtr2
-     */
-
-    segPtr1 = TkTextIndexToSeg(indexPtr1, &offset);
-    byteCount = -offset;
-    for (segPtr = segPtr1; segPtr != NULL; segPtr = segPtr->nextPtr) {
-        byteCount += segPtr->size;
-    }
-
-    linePtr = TkBTreeNextLine(textPtr, indexPtr1->linePtr);
-    while (linePtr != indexPtr2->linePtr) {
-	for (segPtr = linePtr->segPtr; segPtr != NULL;
-                segPtr = segPtr->nextPtr) {
-            byteCount += segPtr->size;
-        }
-        linePtr = TkBTreeNextLine(textPtr, linePtr);
-        if (linePtr == NULL) {
-            Tcl_Panic("TextIndexCountBytesOrdered ran out of lines");
-        }
-    }
-
-    byteCount += indexPtr2->byteIndex;
-
-    return byteCount;
-}
-
 /*
  *---------------------------------------------------------------------------
  *
@@ -1738,11 +1653,11 @@ IndexCountBytesOrdered(
 
 int
 TkTextIndexCount(
-    CONST TkText *textPtr,	/* Overall information about text widget. */
-    CONST TkTextIndex *indexPtr1,
+    const TkText *textPtr,	/* Overall information about text widget. */
+    const TkTextIndex *indexPtr1,
 				/* Index describing location of character from
 				 * which to count. */
-    CONST TkTextIndex *indexPtr2,
+    const TkTextIndex *indexPtr2,
 				/* Index describing location of last character
 				 * at which to stop the count. */
     TkTextCountType type)	/* The kind of indices to count. */
@@ -1764,8 +1679,7 @@ TkTextIndexCount(
     seg2Ptr = TkTextIndexToSeg(indexPtr2, &maxBytes);
 
     if (checkElided) {
-	infoPtr = (TkTextElideInfo *)
-		ckalloc((unsigned) sizeof(TkTextElideInfo));
+	infoPtr = ckalloc(sizeof(TkTextElideInfo));
 	elide = TkTextIsElided(textPtr, indexPtr1, infoPtr);
     }
 
@@ -1905,11 +1819,11 @@ TkTextIndexCount(
   countDone:
     if (infoPtr != NULL) {
 	TkTextFreeElideInfo(infoPtr);
-	ckfree((char *) infoPtr);
+	ckfree(infoPtr);
     }
     return count;
 }
-
+
 /*
  *---------------------------------------------------------------------------
  *
@@ -1934,8 +1848,8 @@ TkTextIndexCount(
 
 int
 TkTextIndexBackBytes(
-    CONST TkText *textPtr,
-    CONST TkTextIndex *srcPtr,	/* Source index. */
+    const TkText *textPtr,
+    const TkTextIndex *srcPtr,	/* Source index. */
     int byteCount,		/* How many bytes backward to move. May be
 				 * negative. */
     TkTextIndex *dstPtr)	/* Destination index: gets modified. */
@@ -2004,8 +1918,8 @@ TkTextIndexBackBytes(
 
 void
 TkTextIndexBackChars(
-    CONST TkText *textPtr,	/* Overall information about text widget. */
-    CONST TkTextIndex *srcPtr,	/* Source index. */
+    const TkText *textPtr,	/* Overall information about text widget. */
+    const TkTextIndex *srcPtr,	/* Source index. */
     int charCount,		/* How many characters backward to move. May
 				 * be negative. */
     TkTextIndex *dstPtr,	/* Destination index: gets modified. */
@@ -2014,7 +1928,7 @@ TkTextIndexBackChars(
     TkTextSegment *segPtr, *oldPtr;
     TkTextElideInfo *infoPtr = NULL;
     int lineIndex, segSize;
-    CONST char *p, *start, *end;
+    const char *p, *start, *end;
     int elide = 0;
     int checkElided = (type & COUNT_DISPLAY);
 
@@ -2023,7 +1937,7 @@ TkTextIndexBackChars(
 	return;
     }
     if (checkElided) {
-	infoPtr = (TkTextElideInfo *) ckalloc(sizeof(TkTextElideInfo));
+	infoPtr = ckalloc(sizeof(TkTextElideInfo));
 	elide = TkTextIsElided(textPtr, srcPtr, infoPtr);
     }
 
@@ -2189,7 +2103,7 @@ TkTextIndexBackChars(
   backwardCharDone:
     if (infoPtr != NULL) {
 	TkTextFreeElideInfo(infoPtr);
-	ckfree((char *) infoPtr);
+	ckfree(infoPtr);
     }
 }
 
@@ -2213,15 +2127,15 @@ TkTextIndexBackChars(
  *----------------------------------------------------------------------
  */
 
-static CONST char *
+static const char *
 StartEnd(
     TkText *textPtr,		/* Information about text widget. */
-    CONST char *string,		/* String to parse for additional info about
+    const char *string,		/* String to parse for additional info about
 				 * modifier (count and units). Points to first
-				 * character of modifier word. */
+				 * character of modifer word. */
     TkTextIndex *indexPtr)	/* Index to modify based on string. */
 {
-    CONST char *p;
+    const char *p;
     size_t length;
     register TkTextSegment *segPtr;
     int modifier;
@@ -2340,7 +2254,7 @@ StartEnd(
 	int offset;
 
 	if (modifier == TKINDEX_DISPLAY) {
-	    TkTextIndexForwChars(textPtr, indexPtr, 0, indexPtr,
+	    TkTextIndexForwChars(NULL, indexPtr, 0, indexPtr,
 		    COUNT_DISPLAY_INDICES);
 	}
 
@@ -2369,20 +2283,11 @@ StartEnd(
 		}
 		firstChar = 0;
 	    }
-            if (offset == 0) {
-                if (modifier == TKINDEX_DISPLAY) {
-                    TkTextIndexBackChars(textPtr, indexPtr, 1, indexPtr,
-                        COUNT_DISPLAY_INDICES);
-                } else {
-                    TkTextIndexBackChars(NULL, indexPtr, 1, indexPtr,
-                        COUNT_INDICES);
-                }
-            } else {
-                indexPtr->byteIndex -= chSize;
-            }
-            offset -= chSize;
+	    offset -= chSize;
+	    indexPtr->byteIndex -= chSize;
 	    if (offset < 0) {
-		if (indexPtr->byteIndex == 0) {
+		if (indexPtr->byteIndex < 0) {
+		    indexPtr->byteIndex = 0;
 		    goto done;
 		}
 		segPtr = TkTextIndexToSeg(indexPtr, &offset);
