@@ -34,7 +34,7 @@
  * TtkScrollbarUpdateRequired, which will invoke step (5) (@@@ Fix this)
  */
 
-#include <tkInt.h>
+#include <tk.h>
 #include "ttkTheme.h"
 #include "ttkWidget.h"
 
@@ -55,7 +55,7 @@ struct ScrollHandleRec
  */
 ScrollHandle TtkCreateScrollHandle(WidgetCore *corePtr, Scrollable *scrollPtr)
 {
-    ScrollHandle h = (ScrollHandle)ckalloc(sizeof(*h));
+    ScrollHandle h = ckalloc(sizeof(*h));
 
     h->flags = 0;
     h->corePtr = corePtr;
@@ -78,7 +78,6 @@ static int UpdateScrollbar(Tcl_Interp *interp, ScrollHandle h)
     char arg1[TCL_DOUBLE_SPACE + 2];
     char arg2[TCL_DOUBLE_SPACE + 2];
     int code;
-    Tcl_DString buf;
 
     h->flags &= ~SCROLL_UPDATE_REQUIRED;
 
@@ -89,14 +88,9 @@ static int UpdateScrollbar(Tcl_Interp *interp, ScrollHandle h)
     arg1[0] = arg2[0] = ' ';
     Tcl_PrintDouble(interp, (double)s->first / s->total, arg1+1);
     Tcl_PrintDouble(interp, (double)s->last / s->total, arg2+1);
-    Tcl_DStringInit(&buf);
-    Tcl_DStringAppend(&buf, s->scrollCmd, -1);
-    Tcl_DStringAppend(&buf, arg1, -1);
-    Tcl_DStringAppend(&buf, arg2, -1);
 
     Tcl_Preserve(corePtr);
-    code = Tcl_EvalEx(interp, Tcl_DStringValue(&buf), -1, TCL_EVAL_GLOBAL);
-    Tcl_DStringFree(&buf);
+    code = Tcl_VarEval(interp, s->scrollCmd, arg1, arg2, NULL);
     if (WidgetDestroyed(corePtr)) {
 	Tcl_Release(corePtr);
 	return TCL_ERROR;
@@ -253,6 +247,6 @@ void TtkFreeScrollHandle(ScrollHandle h)
     if (h->flags & SCROLL_UPDATE_PENDING) {
 	Tcl_CancelIdleCall(UpdateScrollbarBG, (ClientData)h);
     }
-    ckfree((ClientData)h);
+    ckfree(h);
 }
 
