@@ -3,8 +3,6 @@
 # This file defines the default bindings for Tk listbox widgets
 # and provides procedures that help in implementing those bindings.
 #
-# RCS: @(#) $Id: listbox.tcl,v 1.6 1999/08/09 16:52:06 hobbs Exp $
-#
 # Copyright (c) 1994 The Regents of the University of California.
 # Copyright (c) 1994-1995 Sun Microsystems, Inc.
 # Copyright (c) 1998 by Scriptics Corporation.
@@ -13,7 +11,7 @@
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 
 #--------------------------------------------------------------------------
-# tkPriv elements used in this file:
+# tk::Priv elements used in this file:
 #
 # afterId -		Token returned by "after" for autoscanning.
 # listboxPrev -		The last element to be selected or deselected
@@ -35,7 +33,7 @@
 
 bind Listbox <1> {
     if {[winfo exists %W]} {
-	tkListboxBeginSelect %W [%W index @%x,%y]
+	tk::ListboxBeginSelect %W [%W index @%x,%y] 1
     }
 }
 
@@ -48,51 +46,51 @@ bind Listbox <Double-1> {
 }
 
 bind Listbox <B1-Motion> {
-    set tkPriv(x) %x
-    set tkPriv(y) %y
-    tkListboxMotion %W [%W index @%x,%y]
+    set tk::Priv(x) %x
+    set tk::Priv(y) %y
+    tk::ListboxMotion %W [%W index @%x,%y]
 }
 bind Listbox <ButtonRelease-1> {
-    tkCancelRepeat
+    tk::CancelRepeat
     %W activate @%x,%y
 }
 bind Listbox <Shift-1> {
-    tkListboxBeginExtend %W [%W index @%x,%y]
+    tk::ListboxBeginExtend %W [%W index @%x,%y]
 }
 bind Listbox <Control-1> {
-    tkListboxBeginToggle %W [%W index @%x,%y]
+    tk::ListboxBeginToggle %W [%W index @%x,%y]
 }
 bind Listbox <B1-Leave> {
-    set tkPriv(x) %x
-    set tkPriv(y) %y
-    tkListboxAutoScan %W
+    set tk::Priv(x) %x
+    set tk::Priv(y) %y
+    tk::ListboxAutoScan %W
 }
 bind Listbox <B1-Enter> {
-    tkCancelRepeat
+    tk::CancelRepeat
 }
 
-bind Listbox <Up> {
-    tkListboxUpDown %W -1
+bind Listbox <<PrevLine>> {
+    tk::ListboxUpDown %W -1
 }
-bind Listbox <Shift-Up> {
-    tkListboxExtendUpDown %W -1
+bind Listbox <<SelectPrevLine>> {
+    tk::ListboxExtendUpDown %W -1
 }
-bind Listbox <Down> {
-    tkListboxUpDown %W 1
+bind Listbox <<NextLine>> {
+    tk::ListboxUpDown %W 1
 }
-bind Listbox <Shift-Down> {
-    tkListboxExtendUpDown %W 1
+bind Listbox <<SelectNextLine>> {
+    tk::ListboxExtendUpDown %W 1
 }
-bind Listbox <Left> {
+bind Listbox <<PrevChar>> {
     %W xview scroll -1 units
 }
-bind Listbox <Control-Left> {
+bind Listbox <<PrevWord>> {
     %W xview scroll -1 pages
 }
-bind Listbox <Right> {
+bind Listbox <<NextChar>> {
     %W xview scroll 1 units
 }
-bind Listbox <Control-Right> {
+bind Listbox <<NextWord>> {
     %W xview scroll 1 pages
 }
 bind Listbox <Prior> {
@@ -109,10 +107,10 @@ bind Listbox <Control-Prior> {
 bind Listbox <Control-Next> {
     %W xview scroll 1 pages
 }
-bind Listbox <Home> {
+bind Listbox <<LineStart>> {
     %W xview moveto 0
 }
-bind Listbox <End> {
+bind Listbox <<LineEnd>> {
     %W xview moveto 1
 }
 bind Listbox <Control-Home> {
@@ -122,8 +120,8 @@ bind Listbox <Control-Home> {
     %W selection set 0
     event generate %W <<ListboxSelect>>
 }
-bind Listbox <Shift-Control-Home> {
-    tkListboxDataExtend %W 0
+bind Listbox <Control-Shift-Home> {
+    tk::ListboxDataExtend %W 0
 }
 bind Listbox <Control-End> {
     %W activate end
@@ -132,37 +130,37 @@ bind Listbox <Control-End> {
     %W selection set end
     event generate %W <<ListboxSelect>>
 }
-bind Listbox <Shift-Control-End> {
-    tkListboxDataExtend %W [%W index end]
+bind Listbox <Control-Shift-End> {
+    tk::ListboxDataExtend %W [%W index end]
 }
 bind Listbox <<Copy>> {
-    if {![string compare [selection own -displayof %W] "%W"]} {
+    if {[selection own -displayof %W] eq "%W"} {
 	clipboard clear -displayof %W
 	clipboard append -displayof %W [selection get -displayof %W]
     }
 }
 bind Listbox <space> {
-    tkListboxBeginSelect %W [%W index active]
+    tk::ListboxBeginSelect %W [%W index active]
 }
 bind Listbox <Select> {
-    tkListboxBeginSelect %W [%W index active]
+    tk::ListboxBeginSelect %W [%W index active]
 }
 bind Listbox <Control-Shift-space> {
-    tkListboxBeginExtend %W [%W index active]
+    tk::ListboxBeginExtend %W [%W index active]
 }
 bind Listbox <Shift-Select> {
-    tkListboxBeginExtend %W [%W index active]
+    tk::ListboxBeginExtend %W [%W index active]
 }
 bind Listbox <Escape> {
-    tkListboxCancel %W
+    tk::ListboxCancel %W
 }
-bind Listbox <Control-slash> {
-    tkListboxSelectAll %W
+bind Listbox <<SelectAll>> {
+    tk::ListboxSelectAll %W
 }
-bind Listbox <Control-backslash> {
-    if {[string compare [%W cget -selectmode] "browse"]} {
+bind Listbox <<SelectNone>> {
+    if {[%W cget -selectmode] ne "browse"} {
 	%W selection clear 0 end
-      event generate %W <<ListboxSelect>>
+	event generate %W <<ListboxSelect>>
     }
 }
 
@@ -175,15 +173,60 @@ bind Listbox <B2-Motion> {
     %W scan dragto %x %y
 }
 
-# The MouseWheel will typically only fire on Windows.  However,
-# someone could use the "event generate" command to produce one
-# on other platforms.
+# The MouseWheel will typically only fire on Windows and Mac OS X.
+# However, someone could use the "event generate" command to produce
+# one on other platforms.
 
-bind Listbox <MouseWheel> {
-    %W yview scroll [expr {- (%D / 120) * 4}] units
+if {[tk windowingsystem] eq "aqua"} {
+    bind Listbox <MouseWheel> {
+        %W yview scroll [expr {- (%D)}] units
+    }
+    bind Listbox <Option-MouseWheel> {
+        %W yview scroll [expr {-10 * (%D)}] units
+    }
+    bind Listbox <Shift-MouseWheel> {
+        %W xview scroll [expr {- (%D)}] units
+    }
+    bind Listbox <Shift-Option-MouseWheel> {
+        %W xview scroll [expr {-10 * (%D)}] units
+    }
+} else {
+    bind Listbox <MouseWheel> {
+        %W yview scroll [expr {- (%D / 120) * 4}] units
+    }
+    bind Listbox <Shift-MouseWheel> {
+        %W xview scroll [expr {- (%D / 120) * 4}] units
+    }
 }
 
-# tkListboxBeginSelect --
+if {"x11" eq [tk windowingsystem]} {
+    # Support for mousewheels on Linux/Unix commonly comes through mapping
+    # the wheel to the extended buttons.  If you have a mousewheel, find
+    # Linux configuration info at:
+    #	http://www.inria.fr/koala/colas/mouse-wheel-scroll/
+    bind Listbox <4> {
+	if {!$tk_strictMotif} {
+	    %W yview scroll -5 units
+	}
+    }
+    bind Listbox <Shift-4> {
+	if {!$tk_strictMotif} {
+	    %W xview scroll -5 units
+	}
+    }
+    bind Listbox <5> {
+	if {!$tk_strictMotif} {
+	    %W yview scroll 5 units
+	}
+    }
+    bind Listbox <Shift-5> {
+	if {!$tk_strictMotif} {
+	    %W xview scroll 5 units
+	}
+    }
+}
+
+# ::tk::ListboxBeginSelect --
 #
 # This procedure is typically invoked on button-1 presses.  It begins
 # the process of making a selection in the listbox.  Its exact behavior
@@ -195,9 +238,9 @@ bind Listbox <MouseWheel> {
 # el -		The element for the selection operation (typically the
 #		one under the pointer).  Must be in numerical form.
 
-proc tkListboxBeginSelect {w el} {
-    global tkPriv
-    if {![string compare [$w cget -selectmode] "multiple"]} {
+proc ::tk::ListboxBeginSelect {w el {focus 1}} {
+    variable ::tk::Priv
+    if {[$w cget -selectmode] eq "multiple"} {
 	if {[$w selection includes $el]} {
 	    $w selection clear $el
 	} else {
@@ -207,13 +250,17 @@ proc tkListboxBeginSelect {w el} {
 	$w selection clear 0 end
 	$w selection set $el
 	$w selection anchor $el
-	set tkPriv(listboxSelection) {}
-	set tkPriv(listboxPrev) $el
+	set Priv(listboxSelection) {}
+	set Priv(listboxPrev) $el
     }
     event generate $w <<ListboxSelect>>
+    # check existence as ListboxSelect may destroy us
+    if {$focus && [winfo exists $w] && [$w cget -state] eq "normal"} {
+	focus $w
+    }
 }
 
-# tkListboxMotion --
+# ::tk::ListboxMotion --
 #
 # This procedure is called to process mouse motion events while
 # button 1 is down.  It may move or extend the selection, depending
@@ -223,9 +270,9 @@ proc tkListboxBeginSelect {w el} {
 # w -		The listbox widget.
 # el -		The element under the pointer (must be a number).
 
-proc tkListboxMotion {w el} {
-    global tkPriv
-    if {$el == $tkPriv(listboxPrev)} {
+proc ::tk::ListboxMotion {w el} {
+    variable ::tk::Priv
+    if {$el == $Priv(listboxPrev)} {
 	return
     }
     set anchor [$w index anchor]
@@ -233,12 +280,12 @@ proc tkListboxMotion {w el} {
 	browse {
 	    $w selection clear 0 end
 	    $w selection set $el
-	    set tkPriv(listboxPrev) $el
+	    set Priv(listboxPrev) $el
 	    event generate $w <<ListboxSelect>>
 	}
 	extended {
-	    set i $tkPriv(listboxPrev)
-	    if {[string equal {} $i]} {
+	    set i $Priv(listboxPrev)
+	    if {$i eq ""} {
 		set i $el
 		$w selection set $el
 	    }
@@ -249,25 +296,28 @@ proc tkListboxMotion {w el} {
 		$w selection clear $i $el
 		$w selection clear anchor $el
 	    }
+	    if {![info exists Priv(listboxSelection)]} {
+		set Priv(listboxSelection) [$w curselection]
+	    }
 	    while {($i < $el) && ($i < $anchor)} {
-		if {[lsearch $tkPriv(listboxSelection) $i] >= 0} {
+		if {[lsearch $Priv(listboxSelection) $i] >= 0} {
 		    $w selection set $i
 		}
 		incr i
 	    }
 	    while {($i > $el) && ($i > $anchor)} {
-		if {[lsearch $tkPriv(listboxSelection) $i] >= 0} {
+		if {[lsearch $Priv(listboxSelection) $i] >= 0} {
 		    $w selection set $i
 		}
 		incr i -1
 	    }
-	    set tkPriv(listboxPrev) $el
+	    set Priv(listboxPrev) $el
 	    event generate $w <<ListboxSelect>>
 	}
     }
 }
 
-# tkListboxBeginExtend --
+# ::tk::ListboxBeginExtend --
 #
 # This procedure is typically invoked on shift-button-1 presses.  It
 # begins the process of extending a selection in the listbox.  Its
@@ -279,18 +329,18 @@ proc tkListboxMotion {w el} {
 # el -		The element for the selection operation (typically the
 #		one under the pointer).  Must be in numerical form.
 
-proc tkListboxBeginExtend {w el} {
-    if {![string compare [$w cget -selectmode] "extended"]} {
+proc ::tk::ListboxBeginExtend {w el} {
+    if {[$w cget -selectmode] eq "extended"} {
 	if {[$w selection includes anchor]} {
-	    tkListboxMotion $w $el
+	    ListboxMotion $w $el
 	} else {
 	    # No selection yet; simulate the begin-select operation.
-	    tkListboxBeginSelect $w $el
+	    ListboxBeginSelect $w $el
 	}
     }
 }
 
-# tkListboxBeginToggle --
+# ::tk::ListboxBeginToggle --
 #
 # This procedure is typically invoked on control-button-1 presses.  It
 # begins the process of toggling a selection in the listbox.  Its
@@ -302,22 +352,22 @@ proc tkListboxBeginExtend {w el} {
 # el -		The element for the selection operation (typically the
 #		one under the pointer).  Must be in numerical form.
 
-proc tkListboxBeginToggle {w el} {
-    global tkPriv
-    if {![string compare [$w cget -selectmode] "extended"]} {
-	set tkPriv(listboxSelection) [$w curselection]
-	set tkPriv(listboxPrev) $el
+proc ::tk::ListboxBeginToggle {w el} {
+    variable ::tk::Priv
+    if {[$w cget -selectmode] eq "extended"} {
+	set Priv(listboxSelection) [$w curselection]
+	set Priv(listboxPrev) $el
 	$w selection anchor $el
 	if {[$w selection includes $el]} {
 	    $w selection clear $el
 	} else {
 	    $w selection set $el
 	}
-      event generate $w <<ListboxSelect>>
+	event generate $w <<ListboxSelect>>
     }
 }
 
-# tkListboxAutoScan --
+# ::tk::ListboxAutoScan --
 # This procedure is invoked when the mouse leaves an entry window
 # with button 1 down.  It scrolls the window up, down, left, or
 # right, depending on where the mouse left the window, and reschedules
@@ -327,11 +377,11 @@ proc tkListboxBeginToggle {w el} {
 # Arguments:
 # w -		The entry window.
 
-proc tkListboxAutoScan {w} {
-    global tkPriv
+proc ::tk::ListboxAutoScan {w} {
+    variable ::tk::Priv
     if {![winfo exists $w]} return
-    set x $tkPriv(x)
-    set y $tkPriv(y)
+    set x $Priv(x)
+    set y $Priv(y)
     if {$y >= [winfo height $w]} {
 	$w yview scroll 1 units
     } elseif {$y < 0} {
@@ -343,11 +393,11 @@ proc tkListboxAutoScan {w} {
     } else {
 	return
     }
-    tkListboxMotion $w [$w index @$x,$y]
-    set tkPriv(afterId) [after 50 tkListboxAutoScan $w]
+    ListboxMotion $w [$w index @$x,$y]
+    set Priv(afterId) [after 50 [list tk::ListboxAutoScan $w]]
 }
 
-# tkListboxUpDown --
+# ::tk::ListboxUpDown --
 #
 # Moves the location cursor (active element) up or down by one element,
 # and changes the selection if we're in browse or extended selection
@@ -357,28 +407,28 @@ proc tkListboxAutoScan {w} {
 # w -		The listbox widget.
 # amount -	+1 to move down one item, -1 to move back one item.
 
-proc tkListboxUpDown {w amount} {
-    global tkPriv
+proc ::tk::ListboxUpDown {w amount} {
+    variable ::tk::Priv
     $w activate [expr {[$w index active] + $amount}]
     $w see active
     switch [$w cget -selectmode] {
 	browse {
 	    $w selection clear 0 end
 	    $w selection set active
-          event generate $w <<ListboxSelect>>
+	    event generate $w <<ListboxSelect>>
 	}
 	extended {
 	    $w selection clear 0 end
 	    $w selection set active
 	    $w selection anchor active
-	    set tkPriv(listboxPrev) [$w index active]
-	    set tkPriv(listboxSelection) {}
-          event generate $w <<ListboxSelect>>
+	    set Priv(listboxPrev) [$w index active]
+	    set Priv(listboxSelection) {}
+	    event generate $w <<ListboxSelect>>
 	}
     }
 }
 
-# tkListboxExtendUpDown --
+# ::tk::ListboxExtendUpDown --
 #
 # Does nothing unless we're in extended selection mode;  in this
 # case it moves the location cursor (active element) up or down by
@@ -388,16 +438,22 @@ proc tkListboxUpDown {w amount} {
 # w -		The listbox widget.
 # amount -	+1 to move down one item, -1 to move back one item.
 
-proc tkListboxExtendUpDown {w amount} {
-    if {[string compare [$w cget -selectmode] "extended"]} {
+proc ::tk::ListboxExtendUpDown {w amount} {
+    variable ::tk::Priv
+    if {[$w cget -selectmode] ne "extended"} {
 	return
     }
-    $w activate [expr {[$w index active] + $amount}]
+    set active [$w index active]
+    if {![info exists Priv(listboxSelection)]} {
+	$w selection set $active
+	set Priv(listboxSelection) [$w curselection]
+    }
+    $w activate [expr {$active + $amount}]
     $w see active
-    tkListboxMotion $w [$w index active]
+    ListboxMotion $w [$w index active]
 }
 
-# tkListboxDataExtend
+# ::tk::ListboxDataExtend
 #
 # This procedure is called for key-presses such as Shift-KEndData.
 # If the selection mode isn't multiple or extend then it does nothing.
@@ -408,21 +464,21 @@ proc tkListboxExtendUpDown {w amount} {
 # w -		The listbox widget.
 # el -		An integer element number.
 
-proc tkListboxDataExtend {w el} {
+proc ::tk::ListboxDataExtend {w el} {
     set mode [$w cget -selectmode]
-    if {![string compare $mode "extended"]} {
+    if {$mode eq "extended"} {
 	$w activate $el
 	$w see $el
         if {[$w selection includes anchor]} {
-	    tkListboxMotion $w $el
+	    ListboxMotion $w $el
 	}
-    } elseif {![string compare $mode "multiple"]} {
+    } elseif {$mode eq "multiple"} {
 	$w activate $el
 	$w see $el
     }
 }
 
-# tkListboxCancel
+# ::tk::ListboxCancel
 #
 # This procedure is invoked to cancel an extended selection in
 # progress.  If there is an extended selection in progress, it
@@ -432,13 +488,17 @@ proc tkListboxDataExtend {w el} {
 # Arguments:
 # w -		The listbox widget.
 
-proc tkListboxCancel w {
-    global tkPriv
-    if {[string compare [$w cget -selectmode] "extended"]} {
+proc ::tk::ListboxCancel w {
+    variable ::tk::Priv
+    if {[$w cget -selectmode] ne "extended"} {
 	return
     }
     set first [$w index anchor]
-    set last $tkPriv(listboxPrev)
+    set last $Priv(listboxPrev)
+    if {$last eq ""} {
+	# Not actually doing any selection right now
+	return
+    }
     if {$first > $last} {
 	set tmp $first
 	set first $last
@@ -446,7 +506,7 @@ proc tkListboxCancel w {
     }
     $w selection clear $first $last
     while {$first <= $last} {
-	if {[lsearch $tkPriv(listboxSelection) $first] >= 0} {
+	if {[lsearch $Priv(listboxSelection) $first] >= 0} {
 	    $w selection set $first
 	}
 	incr first
@@ -454,7 +514,7 @@ proc tkListboxCancel w {
     event generate $w <<ListboxSelect>>
 }
 
-# tkListboxSelectAll
+# ::tk::ListboxSelectAll
 #
 # This procedure is invoked to handle the "select all" operation.
 # For single and browse mode, it just selects the active element.
@@ -463,9 +523,9 @@ proc tkListboxCancel w {
 # Arguments:
 # w -		The listbox widget.
 
-proc tkListboxSelectAll w {
+proc ::tk::ListboxSelectAll w {
     set mode [$w cget -selectmode]
-    if {![string compare $mode "single"] || ![string compare $mode "browse"]} {
+    if {$mode eq "single" || $mode eq "browse"} {
 	$w selection clear 0 end
 	$w selection set active
     } else {
