@@ -14,19 +14,6 @@
 #include "tkMacOSXPrivate.h"
 #include "tkFileFilter.h"
 
-#if MAC_OS_X_VERSION_MIN_REQUIRED < 1090
-#define modalOK     NSOKButton
-#define modalCancel NSCancelButton
-#else
-#define modalOK     NSModalResponseOK
-#define modalCancel NSModalResponseCancel
-#endif
-#define modalOther  -1
-#define modalError  -2
-
-static int TkBackgroundEvalObjv(Tcl_Interp *interp, int objc,
-    Tcl_Obj *const *objv, int flags);
-
 static const char *const colorOptionStrings[] = {
     "-initialcolor", "-parent", "-title", NULL
 };
@@ -143,23 +130,6 @@ static const short alertNativeButtonIndexAndTypeToButtonIndex[][3] = {
     [TYPE_YESNO] =		{5, 6, 0},
     [TYPE_YESNOCANCEL] =	{5, 6, 4},
 };
-
-/*
- * Construct a file URL from directory and filename.  Either may
- * be nil.  If both are nil, returns nil.
- */
-#if MAC_OS_X_VERSION_MIN_REQUIRED > 1050
-static NSURL *getFileURL(NSString *directory, NSString *filename) {
-    NSURL *url = nil;
-    if (directory) {
-	url = [NSURL fileURLWithPath:directory];
-    }
-    if (filename) {
-	url = [NSURL URLWithString:filename relativeToURL:url];
-    }
-    return url;
-}
-#endif
 
 #pragma mark TKApplication(TKDialog)
 
@@ -179,12 +149,12 @@ static NSURL *getFileURL(NSString *directory, NSString *filename) {
 
 	if (callbackInfo->multiple) {
 	    resultObj = Tcl_NewListObj(0, NULL);
-	    for (NSURL *url in [(NSOpenPanel*)panel URLs]) {
+	    for (NSString *name in [(NSOpenPanel*)panel filenames]) {
 		Tcl_ListObjAppendElement(callbackInfo->interp, resultObj,
-			Tcl_NewStringObj([[url path] UTF8String], -1));
+			Tcl_NewStringObj([name UTF8String], -1));
 	    }
 	} else {
-	    resultObj = Tcl_NewStringObj([[[panel URL]path] UTF8String], -1);
+	    resultObj = Tcl_NewStringObj([[panel filename] UTF8String], -1);
 	}
 	if (callbackInfo->cmdObj) {
 	    Tcl_Obj **objv, **tmpv;
@@ -192,12 +162,12 @@ static NSURL *getFileURL(NSString *directory, NSString *filename) {
 		    callbackInfo->cmdObj, &objc, &objv);
 
 	    if (result == TCL_OK && objc) {
-		tmpv = (Tcl_Obj **) ckalloc(sizeof(Tcl_Obj *) * (objc + 2));
+		tmpv = ckalloc(sizeof(Tcl_Obj *) * (objc + 2));
 		memcpy(tmpv, objv, sizeof(Tcl_Obj *) * objc);
 		tmpv[objc] = resultObj;
 		TkBackgroundEvalObjv(callbackInfo->interp, objc + 1, tmpv,
 			TCL_EVAL_GLOBAL);
-		ckfree((char *)tmpv);
+		ckfree(tmpv);
 	    }
 	} else {
 	    Tcl_SetObjResult(callbackInfo->interp, resultObj);
@@ -210,7 +180,7 @@ static NSURL *getFileURL(NSString *directory, NSString *filename) {
     }
     if (callbackInfo->cmdObj) {
 	Tcl_DecrRefCount(callbackInfo->cmdObj);
-	ckfree((char *)callbackInfo);
+	ckfree(callbackInfo);
     }
 }
 
@@ -219,7 +189,7 @@ static NSURL *getFileURL(NSString *directory, NSString *filename) {
 {
     AlertCallbackInfo *callbackInfo = contextInfo;
 
-    if (returnCode >= NSAlertFirstButtonReturn) {
+    if (returnCode != NSAlertErrorReturn) {
 	Tcl_Obj *resultObj = Tcl_NewStringObj(alertButtonStrings[
 		alertNativeButtonIndexAndTypeToButtonIndex[callbackInfo->
 		typeIndex][returnCode - NSAlertFirstButtonReturn]], -1);
@@ -230,12 +200,12 @@ static NSURL *getFileURL(NSString *directory, NSString *filename) {
 		    callbackInfo->cmdObj, &objc, &objv);
 
 	    if (result == TCL_OK && objc) {
-		tmpv = (Tcl_Obj **) ckalloc(sizeof(Tcl_Obj *) * (objc + 2));
+		tmpv = ckalloc(sizeof(Tcl_Obj *) * (objc + 2));
 		memcpy(tmpv, objv, sizeof(Tcl_Obj *) * objc);
 		tmpv[objc] = resultObj;
 		TkBackgroundEvalObjv(callbackInfo->interp, objc + 1, tmpv,
 			TCL_EVAL_GLOBAL);
-		ckfree((char *)tmpv);
+		ckfree(tmpv);
 	    }
 	} else {
 	    Tcl_SetObjResult(callbackInfo->interp, resultObj);
@@ -246,7 +216,7 @@ static NSURL *getFileURL(NSString *directory, NSString *filename) {
     }
     if (callbackInfo->cmdObj) {
 	Tcl_DecrRefCount(callbackInfo->cmdObj);
-	ckfree((char *) callbackInfo);
+	ckfree(callbackInfo);
     }
 }
 @end
@@ -289,8 +259,8 @@ Tk_ChooseColorObjCmd(
 	int index;
 	const char *value;
 
-	if (Tcl_GetIndexFromObjStruct(interp, objv[i], colorOptionStrings,
-		sizeof(char *), "option", TCL_EXACT, &index) != TCL_OK) {
+	if (Tcl_GetIndexFromObj(interp, objv[i], colorOptionStrings, "option",
+		TCL_EXACT, &index) != TCL_OK) {
 	    goto end;
 	}
 	if (i + 1 == objc) {
@@ -339,7 +309,7 @@ Tk_ChooseColorObjCmd(
 	[colorPanel setColor:initialColor];
     }
     returnCode = [NSApp runModalForWindow:colorPanel];
-    if (returnCode == modalOK) {
+    if (returnCode == NSOKButton) {
 	color = [[colorPanel color] colorUsingColorSpace:
 		[NSColorSpace genericRGBColorSpace]];
 	numberOfComponents = [color numberOfComponents];
@@ -399,13 +369,12 @@ Tk_GetOpenFileObjCmd(
     NSWindow *parent;
     NSMutableArray *fileTypes = nil;
     NSOpenPanel *panel = [NSOpenPanel openPanel];
-    NSInteger modalReturnCode = modalError;
-    BOOL parentIsKey = NO;
+    NSInteger returnCode = NSAlertErrorReturn;
 
     TkInitFileFilters(&fl);
     for (i = 1; i < objc; i += 2) {
-	if (Tcl_GetIndexFromObjStruct(interp, objv[i], openOptionStrings,
-		sizeof(char *), "option", TCL_EXACT, &index) != TCL_OK) {
+	if (Tcl_GetIndexFromObj(interp, objv[i], openOptionStrings, "option",
+		TCL_EXACT, &index) != TCL_OK) {
 	    goto end;
 	}
 	if (i + 1 == objc) {
@@ -470,7 +439,6 @@ Tk_GetOpenFileObjCmd(
 	    break;
 	}
     }
-    [panel setAllowsMultipleSelection:multiple];
     if (fl.filters) {
 	fileTypes = [NSMutableArray array];
 	for (FileFilter *filterPtr = fl.filters; filterPtr;
@@ -503,9 +471,9 @@ Tk_GetOpenFileObjCmd(
 	    }
 	}
     }
-    [panel setAllowedFileTypes:fileTypes];
+    [panel setAllowsMultipleSelection:multiple];
     if (cmdObj) {
-	callbackInfo = (FilePanelCallbackInfo *)ckalloc(sizeof(FilePanelCallbackInfo));
+	callbackInfo = ckalloc(sizeof(FilePanelCallbackInfo));
 	if (Tcl_IsShared(cmdObj)) {
 	    cmdObj = Tcl_DuplicateObj(cmdObj);
 	}
@@ -516,48 +484,26 @@ Tk_GetOpenFileObjCmd(
     callbackInfo->multiple = multiple;
     parent = TkMacOSXDrawableWindow(((TkWindow *) tkwin)->window);
     if (haveParentOption && parent && ![parent attachedSheet]) {
-	    parentIsKey = [parent isKeyWindow];
-#if MAC_OS_X_VERSION_MIN_REQUIRED < 1060
-	[panel beginSheetForDirectory:directory
-	       file:filename
-	       types:fileTypes
-	       modalForWindow:parent
-	       modalDelegate:NSApp
-	       didEndSelector:
-		   @selector(tkFilePanelDidEnd:returnCode:contextInfo:)
-	       contextInfo:callbackInfo];
-#else
-	[panel setAllowedFileTypes:fileTypes];
-	[panel setDirectoryURL:getFileURL(directory, filename)];
-	[panel beginSheetModalForWindow:parent
-	       completionHandler:^(NSInteger returnCode)
-	       { [NSApp tkFilePanelDidEnd:panel
-		       returnCode:returnCode
-		       contextInfo:callbackInfo ]; } ];
-#endif
-	modalReturnCode = cmdObj ? modalOther : [NSApp runModalForWindow:panel];
+	[panel beginSheetForDirectory:directory file:filename types:fileTypes
+		modalForWindow:parent modalDelegate:NSApp didEndSelector:
+		@selector(tkFilePanelDidEnd:returnCode:contextInfo:)
+		contextInfo:callbackInfo];
+	returnCode = cmdObj ? NSAlertOtherReturn :
+		[NSApp runModalForWindow:panel];
     } else {
-#if MAC_OS_X_VERSION_MIN_REQUIRED < 1060
-	modalReturnCode = [panel runModalForDirectory:directory
-				 file:filename];
-#else
-	[panel setDirectoryURL:getFileURL(directory, filename)];
-	modalReturnCode = [panel runModal];
-#endif
-	[NSApp tkFilePanelDidEnd:panel returnCode:modalReturnCode
+	returnCode = [panel runModalForDirectory:directory file:filename
+		types:fileTypes];
+	[NSApp tkFilePanelDidEnd:panel returnCode:returnCode
 		contextInfo:callbackInfo];
     }
-    result = (modalReturnCode != modalError) ? TCL_OK : TCL_ERROR;
-    if (parentIsKey) {
-	[parent makeKeyWindow];
-    }
+    result = (returnCode != NSAlertErrorReturn) ? TCL_OK : TCL_ERROR;
     if (typeVariablePtr && result == TCL_OK) {
 	/*
 	 * The -typevariable option is not really supported.
 	 */
 
-	Tcl_SetVar2(interp, Tcl_GetString(typeVariablePtr), NULL,
-		"", TCL_GLOBAL_ONLY);
+	Tcl_SetVar(interp, Tcl_GetString(typeVariablePtr), "",
+		TCL_GLOBAL_ONLY);
     }
 
   end:
@@ -602,13 +548,12 @@ Tk_GetSaveFileObjCmd(
     NSWindow *parent;
     NSMutableArray *fileTypes = nil;
     NSSavePanel *panel = [NSSavePanel savePanel];
-    NSInteger modalReturnCode = modalError;
-    BOOL parentIsKey = NO;
+    NSInteger returnCode = NSAlertErrorReturn;
 
     TkInitFileFilters(&fl);
     for (i = 1; i < objc; i += 2) {
-	if (Tcl_GetIndexFromObjStruct(interp, objv[i], saveOptionStrings,
-		sizeof(char *), "option", TCL_EXACT, &index) != TCL_OK) {
+	if (Tcl_GetIndexFromObj(interp, objv[i], saveOptionStrings, "option",
+		TCL_EXACT, &index) != TCL_OK) {
 	    goto end;
 	}
 	if (i + 1 == objc) {
@@ -709,7 +654,7 @@ Tk_GetSaveFileObjCmd(
     [panel setCanSelectHiddenExtension:YES];
     [panel setExtensionHidden:NO];
     if (cmdObj) {
-	callbackInfo = (FilePanelCallbackInfo *)ckalloc(sizeof(FilePanelCallbackInfo));
+	callbackInfo = ckalloc(sizeof(FilePanelCallbackInfo));
 	if (Tcl_IsShared(cmdObj)) {
 	    cmdObj = Tcl_DuplicateObj(cmdObj);
 	}
@@ -720,38 +665,19 @@ Tk_GetSaveFileObjCmd(
     callbackInfo->multiple = 0;
     parent = TkMacOSXDrawableWindow(((TkWindow *) tkwin)->window);
     if (haveParentOption && parent && ![parent attachedSheet]) {
-       parentIsKey = [parent isKeyWindow];
-#if MAC_OS_X_VERSION_MIN_REQUIRED < 1060
-	[panel beginSheetForDirectory:directory
-	       file:filename
-	       modalForWindow:parent
-	       modalDelegate:NSApp
-	       didEndSelector:
-		   @selector(tkFilePanelDidEnd:returnCode:contextInfo:)
-	       contextInfo:callbackInfo];
-#else
-	[panel setDirectoryURL:getFileURL(directory, filename)];
-	[panel beginSheetModalForWindow:parent
-	       completionHandler:^(NSInteger returnCode)
-	       { [NSApp tkFilePanelDidEnd:panel
-		       returnCode:returnCode
-		       contextInfo:callbackInfo ]; } ];
-#endif
-	modalReturnCode = cmdObj ? modalOther : [NSApp runModalForWindow:panel];
+	[panel beginSheetForDirectory:directory file:filename
+		modalForWindow:parent modalDelegate:NSApp didEndSelector:
+		@selector(tkFilePanelDidEnd:returnCode:contextInfo:)
+		contextInfo:callbackInfo];
+	returnCode = cmdObj ? NSAlertOtherReturn :
+		[NSApp runModalForWindow:panel];
     } else {
-#if MAC_OS_X_VERSION_MIN_REQUIRED < 1060
-	modalReturnCode = [panel runModalForDirectory:directory file:filename];
-#else
-	[panel setDirectoryURL:getFileURL(directory, filename)];
-	modalReturnCode = [panel runModal];
-#endif
-	[NSApp tkFilePanelDidEnd:panel returnCode:modalReturnCode
+	returnCode = [panel runModalForDirectory:directory file:filename];
+	[NSApp tkFilePanelDidEnd:panel returnCode:returnCode
 		contextInfo:callbackInfo];
     }
-    result = (modalReturnCode != modalError) ? TCL_OK : TCL_ERROR;
-    if (parentIsKey) {
-	[parent makeKeyWindow];
-    }
+    result = (returnCode != NSAlertErrorReturn) ? TCL_OK : TCL_ERROR;
+
   end:
     TkFreeFileFilters(&fl);
     return result;
@@ -793,12 +719,11 @@ Tk_ChooseDirectoryObjCmd(
     NSString *message, *title;
     NSWindow *parent;
     NSOpenPanel *panel = [NSOpenPanel openPanel];
-    NSInteger modalReturnCode = modalError;
-    BOOL parentIsKey = NO;
+    NSInteger returnCode = NSAlertErrorReturn;
 
     for (i = 1; i < objc; i += 2) {
-	if (Tcl_GetIndexFromObjStruct(interp, objv[i], chooseOptionStrings,
-		sizeof(char *), "option", TCL_EXACT, &index) != TCL_OK) {
+	if (Tcl_GetIndexFromObj(interp, objv[i], chooseOptionStrings, "option",
+		TCL_EXACT, &index) != TCL_OK) {
 	    goto end;
 	}
 	if (i + 1 == objc) {
@@ -851,7 +776,7 @@ Tk_ChooseDirectoryObjCmd(
     [panel setCanChooseDirectories:YES];
     [panel setCanCreateDirectories:!mustexist];
     if (cmdObj) {
-	callbackInfo = (FilePanelCallbackInfo *)ckalloc(sizeof(FilePanelCallbackInfo));
+	callbackInfo = ckalloc(sizeof(FilePanelCallbackInfo));
 	if (Tcl_IsShared(cmdObj)) {
 	    cmdObj = Tcl_DuplicateObj(cmdObj);
 	}
@@ -862,37 +787,19 @@ Tk_ChooseDirectoryObjCmd(
     callbackInfo->multiple = 0;
     parent = TkMacOSXDrawableWindow(((TkWindow *) tkwin)->window);
     if (haveParentOption && parent && ![parent attachedSheet]) {
-      parentIsKey = [parent isKeyWindow];
-#if MAC_OS_X_VERSION_MIN_REQUIRED < 1060
-	[panel beginSheetForDirectory:directory
-		file:filename
-		modalForWindow:parent
-		modalDelegate:NSApp
-		didEndSelector: @selector(tkFilePanelDidEnd:returnCode:contextInfo:)
+	[panel beginSheetForDirectory:directory file:filename
+		modalForWindow:parent modalDelegate:NSApp didEndSelector:
+		@selector(tkFilePanelDidEnd:returnCode:contextInfo:)
 		contextInfo:callbackInfo];
-#else
-	[panel setDirectoryURL:getFileURL(directory, filename)];
-	[panel beginSheetModalForWindow:parent
-	       completionHandler:^(NSInteger returnCode)
-	       { [NSApp tkFilePanelDidEnd:panel
-		       returnCode:returnCode
-		       contextInfo:callbackInfo ]; } ];
-#endif
-	modalReturnCode = cmdObj ? modalOther : [NSApp runModalForWindow:panel];
+	returnCode = cmdObj ? NSAlertOtherReturn :
+		[NSApp runModalForWindow:panel];
     } else {
-#if MAC_OS_X_VERSION_MIN_REQUIRED < 1060
-	modalReturnCode = [panel runModalForDirectory:directory file:nil];
-#else
-	[panel setDirectoryURL:getFileURL(directory, filename)];
-	modalReturnCode = [panel runModal];
-#endif
-	[NSApp tkFilePanelDidEnd:panel returnCode:modalReturnCode
+	returnCode = [panel runModalForDirectory:directory file:filename];
+	[NSApp tkFilePanelDidEnd:panel returnCode:returnCode
 		contextInfo:callbackInfo];
     }
-    result = (modalReturnCode != modalError) ? TCL_OK : TCL_ERROR;
-    if (parentIsKey) {
-	[parent makeKeyWindow];
-    }
+    result = (returnCode != NSAlertErrorReturn) ? TCL_OK : TCL_ERROR;
+
   end:
     return result;
 }
@@ -950,9 +857,6 @@ TkAboutDlg(void)
 	    [[[NSAttributedString alloc] initWithString:
 	    [NSString stringWithFormat:
 	    @"%1$C 1987-%2$@ Tcl Core Team." "\n\n"
-		"%1$C 1989-%2$@ Contributors." "\n\n"
-		"%1$C 2011-%2$@ Kevin Walzer/WordTech Communications LLC." "\n\n"
-		"%1$C 2014-%2$@ Marc Culler." "\n\n"
 		"%1$C 2002-%2$@ Daniel A. Steffen." "\n\n"
 		"%1$C 2001-2009 Apple Inc." "\n\n"
 		"%1$C 2001-2002 Jim Ingham & Ian Reid" "\n\n"
@@ -992,7 +896,7 @@ TkMacOSXStandardAboutPanelObjCmd(
 	Tcl_WrongNumArgs(interp, 1, objv, NULL);
 	return TCL_ERROR;
     }
-    [NSApp orderFrontStandardAboutPanelWithOptions:[NSDictionary dictionary]];
+    [NSApp orderFrontStandardAboutPanelWithOptions:nil];
     return TCL_OK;
 }
 
@@ -1030,14 +934,13 @@ Tk_MessageBoxObjCmd(
     NSWindow *parent;
     NSArray *buttons;
     NSAlert *alert = [NSAlert new];
-    NSInteger modalReturnCode = 1;
-    BOOL parentIsKey = NO;
+    NSInteger returnCode = NSAlertErrorReturn;
 
     iconIndex = ICON_INFO;
     typeIndex = TYPE_OK;
     for (i = 1; i < objc; i += 2) {
-	if (Tcl_GetIndexFromObjStruct(interp, objv[i], alertOptionStrings,
-		sizeof(char *), "option", TCL_EXACT, &index) != TCL_OK) {
+	if (Tcl_GetIndexFromObj(interp, objv[i], alertOptionStrings, "option",
+		TCL_EXACT, &index) != TCL_OK) {
 	    goto end;
 	}
 	if (i + 1 == objc) {
@@ -1064,8 +967,8 @@ Tk_MessageBoxObjCmd(
 	    break;
 
 	case ALERT_ICON:
-	    if (Tcl_GetIndexFromObjStruct(interp, objv[i + 1], alertIconStrings,
-		    sizeof(char *), "value", TCL_EXACT, &iconIndex) != TCL_OK) {
+	    if (Tcl_GetIndexFromObj(interp, objv[i + 1], alertIconStrings,
+		    "value", TCL_EXACT, &iconIndex) != TCL_OK) {
 		goto end;
 	    }
 	    break;
@@ -1094,8 +997,8 @@ Tk_MessageBoxObjCmd(
 	    break;
 
 	case ALERT_TYPE:
-	    if (Tcl_GetIndexFromObjStruct(interp, objv[i + 1], alertTypeStrings,
-		    sizeof(char *), "value", TCL_EXACT, &typeIndex) != TCL_OK) {
+	    if (Tcl_GetIndexFromObj(interp, objv[i + 1], alertTypeStrings,
+		    "value", TCL_EXACT, &typeIndex) != TCL_OK) {
 		goto end;
 	    }
 	    break;
@@ -1110,8 +1013,8 @@ Tk_MessageBoxObjCmd(
 	 * why we do this here.
 	 */
 
-	if (Tcl_GetIndexFromObjStruct(interp, objv[indexDefaultOption + 1],
-		alertButtonStrings, sizeof(char *), "value", TCL_EXACT, &index) != TCL_OK) {
+	if (Tcl_GetIndexFromObj(interp, objv[indexDefaultOption + 1],
+		alertButtonStrings, "value", TCL_EXACT, &index) != TCL_OK) {
 	    goto end;
 	}
 
@@ -1147,7 +1050,7 @@ Tk_MessageBoxObjCmd(
     [[buttons objectAtIndex: defaultNativeButtonIndex-1]
 	    setKeyEquivalent: @"\r"];
     if (cmdObj) {
-	callbackInfo = (AlertCallbackInfo *)ckalloc(sizeof(AlertCallbackInfo));
+	callbackInfo = ckalloc(sizeof(AlertCallbackInfo));
 	if (Tcl_IsShared(cmdObj)) {
 	    cmdObj = Tcl_DuplicateObj(cmdObj);
 	}
@@ -1158,97 +1061,595 @@ Tk_MessageBoxObjCmd(
     callbackInfo->typeIndex = typeIndex;
     parent = TkMacOSXDrawableWindow(((TkWindow *) tkwin)->window);
     if (haveParentOption && parent && ![parent attachedSheet]) {
-	parentIsKey = [parent isKeyWindow];
-#if MAC_OS_X_VERSION_MIN_REQUIRED > 1090
- 	[alert beginSheetModalForWindow:parent
-	       completionHandler:^(NSModalResponse returnCode)
-	       { [NSApp tkAlertDidEnd:alert
-			returnCode:returnCode
-			contextInfo:callbackInfo ]; } ];
-#else
-	[alert beginSheetModalForWindow:parent
-	       modalDelegate:NSApp
-	       didEndSelector:@selector(tkAlertDidEnd:returnCode:contextInfo:)
-	       contextInfo:callbackInfo];
-#endif
-	modalReturnCode = cmdObj ? 0 :
-	    [NSApp runModalForWindow:[alert window]];
+	[alert beginSheetModalForWindow:parent modalDelegate:NSApp
+		didEndSelector:@selector(tkAlertDidEnd:returnCode:contextInfo:)
+		contextInfo:callbackInfo];
+	returnCode = cmdObj ? NSAlertOtherReturn :
+		[NSApp runModalForWindow:[alert window]];
     } else {
-	modalReturnCode = [alert runModal];
-	[NSApp tkAlertDidEnd:alert returnCode:modalReturnCode
+	returnCode = [alert runModal];
+	[NSApp tkAlertDidEnd:alert returnCode:returnCode
 		contextInfo:callbackInfo];
     }
-    result = (modalReturnCode >= NSAlertFirstButtonReturn) ? TCL_OK : TCL_ERROR;
+    result = (returnCode != NSAlertErrorReturn) ? TCL_OK : TCL_ERROR;
   end:
     [alert release];
-    if (parentIsKey) {
-	[parent makeKeyWindow];
-    }
     return result;
 }
 
+/*
+ *----------------------------------------------------------------------
+ */
+#pragma mark [tk fontchooser] implementation (TIP 324)
+/*
+ *----------------------------------------------------------------------
+ */
+
+#include "tkMacOSXEvent.h"
+#include "tkMacOSXFont.h"
+
+typedef struct FontchooserData {
+    Tcl_Obj *titleObj;
+    Tcl_Obj *cmdObj;
+    Tk_Window parent;
+} FontchooserData;
+
+enum FontchooserEvent { FontchooserClosed, FontchooserSelection };
+
+static void		FontchooserEvent(int kind);
+static Tcl_Obj *	FontchooserCget(FontchooserData *fcdPtr,
+			    int optionIndex);
+static int		FontchooserConfigureCmd(ClientData clientData,
+			    Tcl_Interp *interp, int objc,
+			    Tcl_Obj *const objv[]);
+static int		FontchooserShowCmd(ClientData clientData,
+			    Tcl_Interp *interp, int objc,
+			    Tcl_Obj *const objv[]);
+static int		FontchooserHideCmd(ClientData clientData,
+			    Tcl_Interp *interp, int objc,
+			    Tcl_Obj *const objv[]);
+static void		FontchooserParentEventHandler(ClientData clientData,
+			    XEvent *eventPtr);
+static void		DeleteFontchooserData(ClientData clientData,
+			    Tcl_Interp *interp);
+
+MODULE_SCOPE const TkEnsemble tkFontchooserEnsemble[];
+const TkEnsemble tkFontchooserEnsemble[] = {
+    { "configure", FontchooserConfigureCmd, NULL },
+    { "show", FontchooserShowCmd, NULL },
+    { "hide", FontchooserHideCmd, NULL },
+    { NULL, NULL, NULL }
+};
+
+static Tcl_Interp *fontchooserInterp = NULL;
+static NSFont *fontPanelFont = nil;
+static NSMutableDictionary *fontPanelFontAttributes = nil;
+
+static const char *const fontchooserOptionStrings[] = {
+    "-parent", "-title", "-font", "-command",
+    "-visible", NULL
+};
+enum FontchooserOption {
+    FontchooserParent, FontchooserTitle, FontchooserFont, FontchooserCmd,
+    FontchooserVisible
+};
+
+@implementation TKApplication(TKFontPanel)
+
+- (void) changeFont: (id) sender
+{
+    NSFontManager *fm = [NSFontManager sharedFontManager];
+
+    if ([fm currentFontAction] == NSViaPanelFontAction) {
+	NSFont *font = [fm convertFont:fontPanelFont];
+
+	if (![fontPanelFont isEqual:font]) {
+	    [fontPanelFont release];
+	    fontPanelFont = [font retain];
+	    FontchooserEvent(FontchooserSelection);
+	}
+    }
+}
+
+- (void) changeAttributes: (id) sender
+{
+    NSDictionary *attributes = [sender convertAttributes:
+	    fontPanelFontAttributes];
+
+    if (![fontPanelFontAttributes isEqual:attributes]) {
+	[fontPanelFontAttributes setDictionary:attributes];
+	FontchooserEvent(FontchooserSelection);
+    }
+}
+
+- (NSUInteger) validModesForFontPanel: (NSFontPanel *) fontPanel
+{
+    return (NSFontPanelStandardModesMask & ~NSFontPanelAllEffectsModeMask) |
+	    NSFontPanelUnderlineEffectModeMask |
+	    NSFontPanelStrikethroughEffectModeMask;
+}
+
+- (void) windowDidOrderOffScreen: (NSNotification *) notification
+{
+#ifdef TK_MAC_DEBUG_NOTIFICATIONS
+    TKLog(@"-[%@(%p) %s] %@", [self class], self, _cmd, notification);
+#endif
+    if ([[notification object] isEqual:[[NSFontManager sharedFontManager]
+	    fontPanel:NO]]) {
+	FontchooserEvent(FontchooserClosed);
+    }
+}
+@end
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * FontchooserEvent --
+ *
+ *	This processes events generated by user interaction with the
+ *	font panel.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Additional events may be place on the Tk event queue.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static void
+FontchooserEvent(
+    int kind)
+{
+    FontchooserData *fcdPtr;
+    Tcl_Obj *fontObj;
+
+    if (!fontchooserInterp) {
+	return;
+    }
+    fcdPtr = Tcl_GetAssocData(fontchooserInterp, "::tk::fontchooser", NULL);
+    switch (kind) {
+    case FontchooserClosed:
+	if (fcdPtr->parent != None) {
+	    TkSendVirtualEvent(fcdPtr->parent, "TkFontchooserVisibility");
+	    fontchooserInterp = NULL;
+	}
+	break;
+    case FontchooserSelection:
+	fontObj = TkMacOSXFontDescriptionForNSFontAndNSFontAttributes(
+		fontPanelFont, fontPanelFontAttributes);
+	if (fontObj) {
+	    if (fcdPtr->cmdObj) {
+		int objc, result;
+		Tcl_Obj **objv, **tmpv;
+
+		result = Tcl_ListObjGetElements(fontchooserInterp,
+			fcdPtr->cmdObj, &objc, &objv);
+		if (result == TCL_OK) {
+		    tmpv = ckalloc(sizeof(Tcl_Obj *) * (objc + 2));
+		    memcpy(tmpv, objv, sizeof(Tcl_Obj *) * objc);
+		    tmpv[objc] = fontObj;
+		    TkBackgroundEvalObjv(fontchooserInterp, objc + 1, tmpv,
+			    TCL_EVAL_GLOBAL);
+		    ckfree(tmpv);
+		}
+	    }
+	    TkSendVirtualEvent(fcdPtr->parent, "TkFontchooserFontChanged");
+	}
+	break;
+    }
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * FontchooserCget --
+ *
+ *	Helper for the FontchooserConfigure command to return the
+ *	current value of any of the options (which may be NULL in
+ *	the structure)
+ *
+ * Results:
+ *	Tcl object of option value.
+ *
+ * Side effects:
+ *	None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static Tcl_Obj *
+FontchooserCget(
+    FontchooserData *fcdPtr,
+    int optionIndex)
+{
+    Tcl_Obj *resObj = NULL;
+
+    switch(optionIndex) {
+    case FontchooserParent:
+	if (fcdPtr->parent != None) {
+	    resObj = Tcl_NewStringObj(
+		    ((TkWindow *) fcdPtr->parent)->pathName, -1);
+	} else {
+	    resObj = Tcl_NewStringObj(".", 1);
+	}
+	break;
+    case FontchooserTitle:
+	if (fcdPtr->titleObj) {
+	    resObj = fcdPtr->titleObj;
+	} else {
+	    resObj = Tcl_NewObj();
+	}
+	break;
+    case FontchooserFont:
+	resObj = TkMacOSXFontDescriptionForNSFontAndNSFontAttributes(
+		fontPanelFont, fontPanelFontAttributes);
+	if (!resObj) {
+	    resObj = Tcl_NewObj();
+	}
+	break;
+    case FontchooserCmd:
+	if (fcdPtr->cmdObj) {
+	    resObj = fcdPtr->cmdObj;
+	} else {
+	    resObj = Tcl_NewObj();
+	}
+	break;
+    case FontchooserVisible:
+	resObj = Tcl_NewBooleanObj([[[NSFontManager sharedFontManager]
+		fontPanel:NO] isVisible]);
+	break;
+    default:
+	resObj = Tcl_NewObj();
+    }
+    return resObj;
+}
+
 /*
  * ----------------------------------------------------------------------
  *
- * TkBackgroundEvalObjv --
+ * FontchooserConfigureCmd --
  *
- *	Evaluate a command while ensuring that we do not affect the
- *	interpreters state. This is important when evaluating script
- *	during background tasks.
+ *	Implementation of the 'tk fontchooser configure' ensemble command.
+ *	See the user documentation for what it does.
  *
  * Results:
- *	A standard Tcl result code.
+ *	See the user documentation.
  *
- * Side Effects:
- *	The interpreters variables and code may be modified by the script
- *	but the result will not be modified.
+ * Side effects:
+ *	Per-interp data structure may be modified
  *
  * ----------------------------------------------------------------------
  */
 
-int
-TkBackgroundEvalObjv(
+static int
+FontchooserConfigureCmd(
+    ClientData clientData,	/* Main window */
     Tcl_Interp *interp,
     int objc,
-    Tcl_Obj *const *objv,
-    int flags)
+    Tcl_Obj *const objv[])
 {
-    Tcl_InterpState state;
-    int n, r = TCL_OK;
+    Tk_Window tkwin = (Tk_Window)clientData;
+    FontchooserData *fcdPtr = Tcl_GetAssocData(interp, "::tk::fontchooser",
+	    NULL);
+    int i, r = TCL_OK;
 
     /*
-     * Record the state of the interpreter.
+     * With no arguments we return all the options in a dict
      */
 
-    Tcl_Preserve(interp);
-    state = Tcl_SaveInterpState(interp, TCL_OK);
+    if (objc == 1) {
+	Tcl_Obj *keyObj, *valueObj;
+	Tcl_Obj *dictObj = Tcl_NewDictObj();
 
-    /*
-     * Evaluate the command and handle any error.
-     */
-
-    for (n = 0; n < objc; ++n) {
-	Tcl_IncrRefCount(objv[n]);
-    }
-    r = Tcl_EvalObjv(interp, objc, objv, flags);
-    for (n = 0; n < objc; ++n) {
-	Tcl_DecrRefCount(objv[n]);
-    }
-    if (r == TCL_ERROR) {
-	Tcl_AddErrorInfo(interp, "\n    (background event handler)");
-	Tcl_BackgroundError(interp);
+	for (i = 0; r == TCL_OK && fontchooserOptionStrings[i] != NULL; ++i) {
+	    keyObj = Tcl_NewStringObj(fontchooserOptionStrings[i], -1);
+	    valueObj = FontchooserCget(fcdPtr, i);
+	    r = Tcl_DictObjPut(interp, dictObj, keyObj, valueObj);
+	}
+	if (r == TCL_OK) {
+	    Tcl_SetObjResult(interp, dictObj);
+	}
+	return r;
     }
 
-    /*
-     * Restore the state of the interpreter.
-     */
+    for (i = 1; i < objc; i += 2) {
+	int optionIndex, len;
 
-    (void) Tcl_RestoreInterpState(interp, state);
-    Tcl_Release(interp);
+	if (Tcl_GetIndexFromObj(interp, objv[i], fontchooserOptionStrings,
+		"option", 0, &optionIndex) != TCL_OK) {
+	    return TCL_ERROR;
+	}
+	if (objc == 2) {
+	    /*
+	     * With one option and no arg, return the current value.
+	     */
 
-    return r;
+	    Tcl_SetObjResult(interp, FontchooserCget(fcdPtr, optionIndex));
+	    return TCL_OK;
+	}
+	if (i + 1 == objc) {
+	    Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+		    "value for \"%s\" missing", Tcl_GetString(objv[i])));
+	    Tcl_SetErrorCode(interp, "TK", "FONTDIALOG", "VALUE", NULL);
+	    return TCL_ERROR;
+	}
+	switch (optionIndex) {
+	case FontchooserVisible: {
+	    const char *msg = "cannot change read-only option "
+		    "\"-visible\": use the show or hide command";
+
+	    Tcl_SetObjResult(interp, Tcl_NewStringObj(msg, -1));
+	    Tcl_SetErrorCode(interp, "TK", "FONTDIALOG", "READONLY", NULL);
+	    return TCL_ERROR;
+	}
+	case FontchooserParent: {
+	    Tk_Window parent = Tk_NameToWindow(interp,
+		    Tcl_GetString(objv[i+1]), tkwin);
+
+	    if (parent == None) {
+		return TCL_ERROR;
+	    }
+	    if (fcdPtr->parent) {
+		Tk_DeleteEventHandler(fcdPtr->parent, StructureNotifyMask,
+			FontchooserParentEventHandler, fcdPtr);
+	    }
+	    fcdPtr->parent = parent;
+	    Tk_CreateEventHandler(fcdPtr->parent, StructureNotifyMask,
+		    FontchooserParentEventHandler, fcdPtr);
+	    break;
+	}
+	case FontchooserTitle:
+	    if (fcdPtr->titleObj) {
+		Tcl_DecrRefCount(fcdPtr->titleObj);
+	    }
+	    Tcl_GetStringFromObj(objv[i+1], &len);
+	    if (len) {
+		fcdPtr->titleObj = objv[i+1];
+		if (Tcl_IsShared(fcdPtr->titleObj)) {
+		    fcdPtr->titleObj = Tcl_DuplicateObj(fcdPtr->titleObj);
+		}
+		Tcl_IncrRefCount(fcdPtr->titleObj);
+	    } else {
+		fcdPtr->titleObj = NULL;
+	    }
+	    break;
+	case FontchooserFont:
+	    Tcl_GetStringFromObj(objv[i+1], &len);
+	    if (len) {
+		Tk_Font f = Tk_AllocFontFromObj(interp, tkwin, objv[i+1]);
+
+		if (!f) {
+		    return TCL_ERROR;
+		}
+		[fontPanelFont autorelease];
+		fontPanelFont = [TkMacOSXNSFontForFont(f) retain];
+		[fontPanelFontAttributes setDictionary:
+			TkMacOSXNSFontAttributesForFont(f)];
+		[fontPanelFontAttributes removeObjectsForKeys:[NSArray
+			arrayWithObjects:NSFontAttributeName,
+			NSLigatureAttributeName, NSKernAttributeName, nil]];
+		Tk_FreeFont(f);
+	    } else {
+		[fontPanelFont release];
+		fontPanelFont = nil;
+		[fontPanelFontAttributes removeAllObjects];
+	    }
+
+	    NSFontManager *fm = [NSFontManager sharedFontManager];
+	    NSFontPanel *fp = [fm fontPanel:NO];
+
+	    [fp setPanelFont:fontPanelFont isMultiple:NO];
+	    [fm setSelectedFont:fontPanelFont isMultiple:NO];
+	    [fm setSelectedAttributes:fontPanelFontAttributes
+		    isMultiple:NO];
+	    if ([fp isVisible]) {
+		TkSendVirtualEvent(fcdPtr->parent, "TkFontchooserFontChanged");
+	    }
+	    break;
+	case FontchooserCmd:
+	    if (fcdPtr->cmdObj) {
+		Tcl_DecrRefCount(fcdPtr->cmdObj);
+	    }
+	    Tcl_GetStringFromObj(objv[i+1], &len);
+	    if (len) {
+		fcdPtr->cmdObj = objv[i+1];
+		if (Tcl_IsShared(fcdPtr->cmdObj)) {
+		    fcdPtr->cmdObj = Tcl_DuplicateObj(fcdPtr->cmdObj);
+		}
+		Tcl_IncrRefCount(fcdPtr->cmdObj);
+	    } else {
+		fcdPtr->cmdObj = NULL;
+	    }
+	    break;
+	}
+    }
+    return TCL_OK;
 }
+
+/*
+ * ----------------------------------------------------------------------
+ *
+ * FontchooserShowCmd --
+ *
+ *	Implements the 'tk fontchooser show' ensemble command. The
+ *	per-interp configuration data for the dialog is held in an interp
+ *	associated structure.
+ *
+ * Results:
+ *	See the user documentation.
+ *
+ * Side effects:
+ *	Font Panel may be shown.
+ *
+ * ----------------------------------------------------------------------
+ */
 
+static int
+FontchooserShowCmd(
+    ClientData clientData,	/* Main window */
+    Tcl_Interp *interp,
+    int objc,
+    Tcl_Obj *const objv[])
+{
+    FontchooserData *fcdPtr = Tcl_GetAssocData(interp, "::tk::fontchooser",
+	    NULL);
+
+    if (fcdPtr->parent == None) {
+	fcdPtr->parent = (Tk_Window) clientData;
+	Tk_CreateEventHandler(fcdPtr->parent, StructureNotifyMask,
+		FontchooserParentEventHandler, fcdPtr);
+    }
+    NSFontManager *fm = [NSFontManager sharedFontManager];
+    NSFontPanel *fp = [fm fontPanel:YES];
+    if ([fp delegate] != NSApp) {
+	[fp setDelegate:NSApp];
+    }
+    if (![fp isVisible]) {
+	[fm orderFrontFontPanel:NSApp];
+	TkSendVirtualEvent(fcdPtr->parent, "TkFontchooserVisibility");
+    }
+    fontchooserInterp = interp;
+
+    return TCL_OK;
+}
+
+/*
+ * ----------------------------------------------------------------------
+ *
+ * FontchooserHideCmd --
+ *
+ *	Implementation of the 'tk fontchooser hide' ensemble. See the
+ *	user documentation for details.
+ *
+ * Results:
+ *	See the user documentation.
+ *
+ * Side effects:
+ *	Font Panel may be hidden.
+ *
+ * ----------------------------------------------------------------------
+ */
+
+static int
+FontchooserHideCmd(
+    ClientData clientData,	/* Main window */
+    Tcl_Interp *interp,
+    int objc,
+    Tcl_Obj *const objv[])
+{
+    NSFontPanel *fp = [[NSFontManager sharedFontManager] fontPanel:NO];
+    if ([fp isVisible]) {
+	[fp orderOut:NSApp];
+    }
+    return TCL_OK;
+}
+
+/*
+ * ----------------------------------------------------------------------
+ *
+ * FontchooserParentEventHandler --
+ *
+ *	Event handler for StructureNotify events on the font chooser's parent
+ *	window.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Font chooser parent info is cleared and font panel is hidden.
+ *
+ * ----------------------------------------------------------------------
+ */
+
+static void
+FontchooserParentEventHandler(
+    ClientData clientData,
+    XEvent *eventPtr)
+{
+    FontchooserData *fcdPtr = clientData;
+
+    if (eventPtr->type == DestroyNotify) {
+	Tk_DeleteEventHandler(fcdPtr->parent, StructureNotifyMask,
+		FontchooserParentEventHandler, fcdPtr);
+	fcdPtr->parent = None;
+	FontchooserHideCmd(NULL, NULL, 0, NULL);
+    }
+}
+
+/*
+ * ----------------------------------------------------------------------
+ *
+ * DeleteFontchooserData --
+ *
+ *	Clean up the font chooser configuration data when the interp is
+ *	destroyed.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	per-interp configuration data is destroyed.
+ *
+ * ----------------------------------------------------------------------
+ */
+
+static void
+DeleteFontchooserData(
+    ClientData clientData,
+    Tcl_Interp *interp)
+{
+    FontchooserData *fcdPtr = clientData;
+
+    if (fcdPtr->titleObj) {
+	Tcl_DecrRefCount(fcdPtr->titleObj);
+    }
+    if (fcdPtr->cmdObj) {
+	Tcl_DecrRefCount(fcdPtr->cmdObj);
+    }
+    ckfree(fcdPtr);
+
+    if (fontchooserInterp == interp) {
+	fontchooserInterp = NULL;
+    }
+}
+
+/*
+ * ----------------------------------------------------------------------
+ *
+ * TkInitFontchooser --
+ *
+ *	Associate the font chooser configuration data with the Tcl
+ *	interpreter. There is one font chooser per interp.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	per-interp configuration data is destroyed.
+ *
+ * ----------------------------------------------------------------------
+ */
+
+MODULE_SCOPE int
+TkInitFontchooser(
+    Tcl_Interp *interp,
+    ClientData clientData)
+{
+    FontchooserData *fcdPtr = ckalloc(sizeof(FontchooserData));
+
+    bzero(fcdPtr, sizeof(FontchooserData));
+    Tcl_SetAssocData(interp, "::tk::fontchooser", DeleteFontchooserData,
+	    fcdPtr);
+    if (!fontPanelFontAttributes) {
+	NSAutoreleasePool *pool = [NSAutoreleasePool new];
+	fontPanelFontAttributes = [NSMutableDictionary new];
+	[pool drain];
+    }
+    return TCL_OK;
+}
+
 /*
  * Local Variables:
  * mode: objc
