@@ -14,8 +14,6 @@
 
 #include "tkMacOSXPrivate.h"
 
-#include "tclInt.h" /* for Tcl_GetStartupScript() & Tcl_SetStartupScript() */
-
 #include <sys/stat.h>
 #include <sys/utsname.h>
 #include <dlfcn.h>
@@ -30,9 +28,10 @@ static char tkLibPath[PATH_MAX + 1] = "";
 
 static char scriptPath[PATH_MAX + 1] = "";
 
+int tkMacOSXGCEnabled = 0;
 long tkMacOSXMacOSXVersion = 0;
 
-#pragma mark TKApplication(TKInit) 
+#pragma mark TKApplication(TKInit)
 
 #if MAC_OS_X_VERSION_MIN_REQUIRED < 1060
 #define NSTextInputContextKeyboardSelectionDidChangeNotification @"NSTextInputContextKeyboardSelectionDidChangeNotification"
@@ -42,7 +41,7 @@ static void keyboardChanged(CFNotificationCenterRef center, void *observer, CFSt
 #endif
 
 @interface TKApplication(TKKeyboard)
-- (void)keyboardChanged:(NSNotification *)notification;
+- (void) keyboardChanged: (NSNotification *) notification;
 @end
 
 #if MAC_OS_X_VERSION_MAX_ALLOWED >= 1060
@@ -51,35 +50,33 @@ static void keyboardChanged(CFNotificationCenterRef center, void *observer, CFSt
 #define TKApplication_NSApplicationDelegate
 #endif
 @interface TKApplication(TKWindowEvent) TKApplication_NSApplicationDelegate
-- (void)_setupWindowNotifications;
+- (void) _setupWindowNotifications;
+@end
+
+@interface TKApplication(TKScrlbr)
+- (void) _setupScrollBarNotifications;
 @end
 
 @interface TKApplication(TKMenus)
-- (void)_setupMenus;
+- (void) _setupMenus;
 @end
 
 @implementation TKApplication
-@synthesize poolProtected = _poolProtected;
 @end
 
 @implementation TKApplication(TKInit)
-- (void) _resetAutoreleasePool
-{
-    if(![self poolProtected]) {
-	[_mainPool drain];
-	_mainPool = [NSAutoreleasePool new];
-    }
-}
-
 #ifdef TK_MAC_DEBUG_NOTIFICATIONS
-- (void)_postedNotification:(NSNotification *)notification {
+- (void) _postedNotification: (NSNotification *) notification
+{
     TKLog(@"-[%@(%p) %s] %@", [self class], self, _cmd, notification);
 }
 #endif
 
-- (void)_setupApplicationNotifications {
+- (void) _setupApplicationNotifications
+{
     NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
-#define observe(n, s) [nc addObserver:self selector:@selector(s) name:(n) object:nil]
+#define observe(n, s) \
+	[nc addObserver:self selector:@selector(s) name:(n) object:nil]
     observe(NSApplicationDidBecomeActiveNotification, applicationActivate:);
     observe(NSApplicationDidResignActiveNotification, applicationDeactivate:);
     observe(NSApplicationDidUnhideNotification, applicationShowHide:);
@@ -92,17 +89,19 @@ static void keyboardChanged(CFNotificationCenterRef center, void *observer, CFSt
 #endif
 }
 
-- (void)_setupEventLoop {
-    NSAutoreleasePool *pool = [NSAutoreleasePool new];
-    [self finishLaunching];
+- (void) _setupEventLoop
+{
+    _running = 1;
+    if (!_appFlags._hasBeenRun) {
+        _appFlags._hasBeenRun = YES;
+	[self finishLaunching];
+    }
     [self setWindowsNeedUpdate:YES];
-    [pool drain];
 }
 
-- (void)_setup:(Tcl_Interp *)interp {
+- (void) _setup: (Tcl_Interp *) interp
+{
     _eventInterp = interp;
-    _mainPool = [NSAutoreleasePool new];
-    [NSApp setPoolProtected:NO];
     _defaultMainMenu = nil;
     [self _setupMenus];
     [self setDelegate:self];
@@ -111,12 +110,14 @@ static void keyboardChanged(CFNotificationCenterRef center, void *observer, CFSt
 	    selector:@selector(_postedNotification:) name:nil object:nil];
 #endif
     [self _setupWindowNotifications];
+    [self _setupScrollBarNotifications];
     [self _setupApplicationNotifications];
 }
 
-- (NSString *)tkFrameworkImagePath:(NSString*)image {
+- (NSString *) tkFrameworkImagePath: (NSString *) image
+{
     NSString *path = nil;
-    NSAutoreleasePool *pool = [NSAutoreleasePool new];
+
     if (tkLibPath[0] != '\0') {
 	path = [[NSBundle bundleWithPath:[[NSString stringWithUTF8String:
 		tkLibPath] stringByAppendingString:@"/../.."]]
@@ -125,8 +126,10 @@ static void keyboardChanged(CFNotificationCenterRef center, void *observer, CFSt
     if (!path) {
 	const char *tk_library = Tcl_GetVar2(_eventInterp, "tk_library", NULL,
 		TCL_GLOBAL_ONLY);
+
 	if (tk_library) {
 	    NSFileManager *fm = [NSFileManager defaultManager];
+
 	    path = [[NSString stringWithUTF8String:tk_library]
 		    stringByAppendingFormat:@"/%@", image];
 	    if (![fm isReadableFileAtPath:path]) {
@@ -147,8 +150,6 @@ static void keyboardChanged(CFNotificationCenterRef center, void *observer, CFSt
 	}
     }
 #endif
-    [path retain];
-    [pool drain];
     return path;
 }
 @end
@@ -175,7 +176,6 @@ static void
 SetApplicationIcon(
     ClientData clientData)
 {
-    NSAutoreleasePool *pool = [NSAutoreleasePool new];
     NSString *path = [NSApp tkFrameworkImagePath:@"Tk.icns"];
     if (path) {
 	NSImage *image = [[NSImage alloc] initWithContentsOfFile:path];
@@ -184,7 +184,6 @@ SetApplicationIcon(
 	    [image release];
 	}
     }
-    [pool drain];
 }
 
 /*
@@ -237,16 +236,11 @@ TkpInit(
 	if (!uname(&name)) {
 	    tkMacOSXMacOSXVersion = (strtod(name.release, NULL) + 96) * 10;
 	}
-        /*Check for new versioning scheme on Yosemite (10.10) and later.*/
-	if (MAC_OS_X_VERSION_MIN_REQUIRED > 100000) {
-		tkMacOSXMacOSXVersion = MAC_OS_X_VERSION_MIN_REQUIRED/100;
-	    }
-	if (tkMacOSXMacOSXVersion && MAC_OS_X_VERSION_MIN_REQUIRED < 100000 &&
+	if (tkMacOSXMacOSXVersion &&
 		tkMacOSXMacOSXVersion/10 < MAC_OS_X_VERSION_MIN_REQUIRED/10) {
 	    Tcl_Panic("Mac OS X 10.%d or later required !",
 		    (MAC_OS_X_VERSION_MIN_REQUIRED/10)-100);
 	}
-		
 
 #ifdef TK_FRAMEWORK
 	/*
@@ -262,19 +256,20 @@ TkpInit(
 	}
 #endif
 
-	{
-	    NSAutoreleasePool *pool = [NSAutoreleasePool new];
-		[[NSUserDefaults standardUserDefaults] registerDefaults:
-		     [NSDictionary dictionaryWithObjectsAndKeys:
-		     [NSNumber numberWithBool:YES],
-		     @"_NSCanWrapButtonTitles",
-		     [NSNumber numberWithInt:-1],
-		     @"NSStringDrawingTypesetterBehavior",
-		     nil]];
-	    [TKApplication sharedApplication];
-	    [pool drain];
-	    [NSApp _setup:interp];
+	static NSAutoreleasePool *pool = nil;
+	if (!pool) {
+	    pool = [NSAutoreleasePool new];
 	}
+	tkMacOSXGCEnabled = ([NSGarbageCollector defaultCollector] != nil);
+	[[NSUserDefaults standardUserDefaults] registerDefaults:
+		[NSDictionary dictionaryWithObjectsAndKeys:
+		[NSNumber numberWithBool:YES],
+		@"_NSCanWrapButtonTitles",
+		[NSNumber numberWithInt:-1],
+		@"NSStringDrawingTypesetterBehavior",
+		nil]];
+	[TKApplication sharedApplication];
+	[NSApp _setup:interp];
 
 	/* Check whether we are a bundled executable: */
 	bundleRef = CFBundleGetMainBundle();
@@ -331,15 +326,13 @@ TkpInit(
 	    Tcl_DoWhenIdle(SetApplicationIcon, NULL);
 	}
 
-	{
-	    NSAutoreleasePool *pool = [NSAutoreleasePool new];
-	    [NSApp _setupEventLoop];
-	    TkMacOSXInitAppleEvents(interp);
-	    TkMacOSXUseAntialiasedText(interp, -1);
-	    TkMacOSXInitCGDrawing(interp, TRUE, 0);
-	    [pool drain];
-	}
-	
+	[NSApp _setupEventLoop];
+	TkMacOSXInitAppleEvents(interp);
+	TkMacOSXUseAntialiasedText(interp, -1);
+	TkMacOSXInitCGDrawing(interp, TRUE, 0);
+	[pool drain];
+	pool = [NSAutoreleasePool new];
+
 	/*
 	 * FIXME: Close stdin & stdout for remote debugging otherwise we will
 	 * fight with gdb for stdin & stdout
@@ -495,8 +488,9 @@ TkpDisplayWarning(
 MODULE_SCOPE void
 TkMacOSXDefaultStartupScript(void)
 {
-    NSAutoreleasePool *pool = [NSAutoreleasePool new];
-    CFBundleRef bundleRef = CFBundleGetMainBundle();
+    CFBundleRef bundleRef;
+
+    bundleRef = CFBundleGetMainBundle();
 
     if (bundleRef != NULL) {
 	CFURLRef appMainURL = CFBundleCopyResourceURL(bundleRef,
@@ -520,7 +514,6 @@ TkMacOSXDefaultStartupScript(void)
 	    CFRelease(appMainURL);
 	}
     }
-    [pool drain];
 }
 
 /*
