@@ -76,7 +76,7 @@ TkScale *
 TkpCreateScale(
     Tk_Window tkwin)
 {
-    MacScale *macScalePtr = (MacScale *) ckalloc(sizeof(MacScale));
+    MacScale *macScalePtr = ckalloc(sizeof(MacScale));
 
     macScalePtr->scaleHandle = NULL;
     if (scaleActionProc == NULL) {
@@ -145,7 +145,7 @@ TkpDisplayScale(
     Tk_Window tkwin = scalePtr->tkwin;
     Tcl_Interp *interp = scalePtr->interp;
     int result;
-    char string[TCL_DOUBLE_SPACE];
+    char string[PRINT_CHARS];
     MacScale *macScalePtr = (MacScale *) clientData;
     Rect r;
     WindowRef windowRef;
@@ -174,7 +174,7 @@ TkpDisplayScale(
 	result = Tcl_VarEval(interp, scalePtr->command, " ", string, NULL);
 	if (result != TCL_OK) {
 	    Tcl_AddErrorInfo(interp, "\n    (command executed by scale)");
-	    Tcl_BackgroundError(interp);
+	    Tcl_BackgroundException(interp, result);
 	}
 	Tcl_Release((ClientData) interp);
     }
