@@ -281,7 +281,7 @@ Tk_SetAppName(
      * We have found a unique name. Now add it to the registry.
      */
 
-    riPtr = (RegisteredInterp *) ckalloc(sizeof(RegisteredInterp));
+    riPtr = ckalloc(sizeof(RegisteredInterp));
     riPtr->interp = interp;
     riPtr->name = ckalloc(strlen(actualName) + 1);
     riPtr->nextPtr = interpListPtr;
@@ -325,7 +325,7 @@ Tk_SendObjCmd(
     int objc,			/* Number of arguments */
     Tcl_Obj *const objv[])	/* The arguments */
 {
-    const char *sendOptions[] = {"-async", "-displayof", "-", NULL};
+    const char *const sendOptions[] = {"-async", "-displayof", "-", NULL};
     char *stringRep, *destName;
     /*int async = 0;*/
     int i, index, firstArg;
