@@ -9,7 +9,6 @@
  * Copyright (c) 1995-1997 Sun Microsystems, Inc.
  * Copyright 2001-2009, Apple Inc.
  * Copyright (c) 2005-2009 Daniel A. Steffen <das@users.sourceforge.net>
- * Copyright 2014 Marc Culler.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -142,7 +141,7 @@ TkpOpenDisplay(
     static NSRect maxBounds = {{0, 0}, {0, 0}};
     static char vendor[25] = "";
     NSArray *cgVers;
-    NSAutoreleasePool *pool = [NSAutoreleasePool new];
+    NSAutoreleasePool *pool;
 
     if (gMacDisplay != NULL) {
 	if (strcmp(gMacDisplay->display->display_name, display_name) == 0) {
@@ -152,8 +151,8 @@ TkpOpenDisplay(
 	}
     }
 
-    display = (Display *) ckalloc(sizeof(Display));
-    screen  = (Screen *) ckalloc(sizeof(Screen));
+    display = ckalloc(sizeof(Display));
+    screen  = ckalloc(sizeof(Screen));
     bzero(display, sizeof(Display));
     bzero(screen, sizeof(Screen));
 
@@ -166,6 +165,7 @@ TkpOpenDisplay(
     display->default_screen = 0;
     display->display_name   = (char *) macScreenName;
 
+    pool = [NSAutoreleasePool new];
     cgVers = [[[NSBundle bundleWithIdentifier:@"com.apple.CoreGraphics"]
 	    objectForInfoDictionaryKey:@"CFBundleShortVersionString"]
 	    componentsSeparatedByString:@"."];
@@ -176,25 +176,12 @@ TkpOpenDisplay(
 	display->proto_minor_version = [[cgVers objectAtIndex:2] integerValue];
     }
     if (!vendor[0]) {
-	snprintf(vendor, sizeof(vendor), "Apple AppKit %g",
+	snprintf(vendor, sizeof(vendor), "Apple AppKit %s %g",
+		([NSGarbageCollector defaultCollector] ? "GC" : "RR"),
 		NSAppKitVersionNumber);
     }
     display->vendor = vendor;
-    {
-	int major, minor, patch;
-
-#if MAC_OS_X_VERSION_MIN_REQUIRED < 10100
-	Gestalt(gestaltSystemVersionMajor, (SInt32*)&major);
-	Gestalt(gestaltSystemVersionMinor, (SInt32*)&minor);
-	Gestalt(gestaltSystemVersionBugFix, (SInt32*)&patch);
-#else
-	NSOperatingSystemVersion systemVersion = [[NSProcessInfo processInfo] operatingSystemVersion];
-	major = systemVersion.majorVersion;
-	minor = systemVersion.minorVersion;
-	patch = systemVersion.patchVersion;
-#endif
-	display->release = major << 16 | minor << 8 | patch;
-    }
+    Gestalt(gestaltSystemVersion, (SInt32 *) &display->release);
 
     /*
      * These screen bits never change
@@ -205,7 +192,7 @@ TkpOpenDisplay(
     screen->white_pixel = 0x00FFFFFF | PIXEL_MAGIC << 24;
     screen->ext_data	= (XExtData *) &maxBounds;
 
-    screen->root_visual = (Visual *) ckalloc(sizeof(Visual));
+    screen->root_visual = ckalloc(sizeof(Visual));
     screen->root_visual->visualid     = 0;
     screen->root_visual->class	      = TrueColor;
     screen->root_visual->red_mask     = 0x00FF0000;
@@ -220,7 +207,7 @@ TkpOpenDisplay(
 
     TkMacOSXDisplayChanged(display);
 
-    gMacDisplay = (TkDisplay *) ckalloc(sizeof(TkDisplay));
+    gMacDisplay = ckalloc(sizeof(TkDisplay));
 
     /*
      * This is the quickest way to make sure that all the *Init flags get
@@ -262,11 +249,11 @@ TkpCloseDisplay(
     gMacDisplay = NULL;
     if (display->screens != NULL) {
 	if (display->screens->root_visual != NULL) {
-	    ckfree((char *) display->screens->root_visual);
+	    ckfree(display->screens->root_visual);
 	}
-	ckfree((char *) display->screens);
+	ckfree(display->screens);
     }
-    ckfree((char *) display);
+    ckfree(display);
 }
 
 /*
@@ -343,33 +330,6 @@ MacXIdAlloc(
     return ++cur_id;
 }
 
-/*
- *----------------------------------------------------------------------
- *
- * TkpWindowWasRecentlyDeleted --
- *
- *	Tries to determine whether the given window was recently deleted.
- *	Called from the generic code error handler to attempt to deal with
- *	async BadWindow errors under some circumstances.
- *
- * Results:
- *	Always 0, we do not keep this information on the Mac, so we do not
- *	know whether the window was destroyed.
- *
- * Side effects:
- *	None.
- *
- *----------------------------------------------------------------------
- */
-
-int
-TkpWindowWasRecentlyDeleted(
-    Window win,
-    TkDisplay *dispPtr)
-{
-    return 0;
-}
-
 /*
  *----------------------------------------------------------------------
  *
@@ -822,8 +782,9 @@ XCreateImage(
     int bytes_per_line)
 {
     XImage *ximage;
+
     display->request++;
-    ximage = (XImage *) ckalloc(sizeof(XImage));
+    ximage = ckalloc(sizeof(XImage));
 
     ximage->height = height;
     ximage->width = width;
@@ -831,7 +792,6 @@ XCreateImage(
     ximage->xoffset = offset;
     ximage->format = format;
     ximage->data = data;
-    ximage->obdata = NULL;
 
     if (format == ZPixmap) {
 	ximage->bits_per_pixel = 32;
@@ -863,6 +823,7 @@ XCreateImage(
     ximage->red_mask = 0x00FF0000;
     ximage->green_mask = 0x0000FF00;
     ximage->blue_mask = 0x000000FF;
+    ximage->obdata = NULL;
     ximage->f.create_image = NULL;
     ximage->f.destroy_image = DestroyImage;
     ximage->f.get_pixel = ImageGetPixel;
@@ -881,9 +842,8 @@ XCreateImage(
  *	This function copies data from a pixmap or window into an XImage.
  *
  * Results:
- *	Returns a newly allocated XImage containing the data from the given
- *	rectangle of the given drawable, or NULL if the XImage could not be
- *     constructed.
+ *	Returns a newly allocated image containing the data from the given
+ *	rectangle of the given drawable.
  *
  * Side effects:
  *	None.
@@ -902,84 +862,60 @@ XGetImage(
     unsigned long plane_mask,
     int format)
 {
-    NSBitmapImageRep *bitmap_rep;
-    NSUInteger         bitmap_fmt;
-    XImage *       imagePtr = NULL;
-    char *           bitmap = NULL;
-    char *           image_data=NULL;
-    int	        depth = 32;
-    int	        offset = 0;
-    int	        bitmap_pad = 0;
-    int	        bytes_per_row = 4*width;
-    int                size;
-    TkMacOSXDbgMsg("XGetImage");
+    MacDrawable *macDraw = (MacDrawable *) d;
+    XImage *   imagePtr = NULL;
+    Pixmap     pixmap = (Pixmap) NULL;
+    Tk_Window  win = (Tk_Window) macDraw->winPtr;
+    GC	       gc;
+    char *     data = NULL;
+    int	       depth = 32;
+    int	       offset = 0;
+    int	       bitmap_pad = 0;
+    int	       bytes_per_line = 0;
+
     if (format == ZPixmap) {
-	if (width == 0 || height == 0) {
-	    /* This happens all the time.
-	    TkMacOSXDbgMsg("XGetImage: empty image requested");
-	    */
-	    return NULL;
+	if (width > 0 && height > 0) {
+	    /*
+	     * Tk_GetPixmap fails for zero width or height.
+	     */
+
+	    pixmap = Tk_GetPixmap(display, d, width, height, depth);
 	}
+	if (win) {
+	    XGCValues values;
 
-	bitmap_rep =  BitmapRepFromDrawableRect(d, x, y,width, height);
-	bitmap_fmt = [bitmap_rep bitmapFormat];
-
-	if ( bitmap_rep == Nil                     ||
-	     (bitmap_fmt != 0 && bitmap_fmt != 1)  ||
-	     [bitmap_rep samplesPerPixel] != 4     ||
-	     [bitmap_rep isPlanar] != 0               ) {
-	    TkMacOSXDbgMsg("XGetImage: Failed to construct NSBitmapRep");
-	    return NULL;
+	    gc = Tk_GetGC(win, 0, &values);
+	} else {
+	    gc = XCreateGC(display, pixmap, 0, NULL);
 	}
+	if (pixmap) {
+	    CGContextRef context;
 
-	NSSize image_size = NSMakeSize(width, height);
-	NSImage* ns_image = [[NSImage alloc]initWithSize:image_size];
-	[ns_image addRepresentation:bitmap_rep];
-
-	/* Assume premultiplied nonplanar data with 4 bytes per pixel.*/
-	if ( [bitmap_rep isPlanar ] == 0 &&
-	     [bitmap_rep samplesPerPixel] == 4 ) {
-	    bytes_per_row = [bitmap_rep bytesPerRow];
-	    size = bytes_per_row*height;
-	    image_data = (char*)[bitmap_rep bitmapData];
-	    if ( image_data ) {
-		int row, n, m;
-		bitmap = ckalloc(size);
-		/*
-		  Oddly enough, the bitmap has the top row at the beginning,
-		  and the pixels are in BGRA or ABGR format.
-		*/
-		if (bitmap_fmt == 0) {
-		    /* BGRA */
-		    for (row=0, n=0; row<height; row++, n+=bytes_per_row) {
-			for (m=n; m<n+bytes_per_row; m+=4) {
-			    *(bitmap+m)   = *(image_data+m+2);
-			    *(bitmap+m+1) = *(image_data+m+1);
-			    *(bitmap+m+2) = *(image_data+m);
-			    *(bitmap+m+3) = *(image_data+m+3);
-			}
-		    }
-		} else {
-		    /* ABGR */
-		    for (row=0, n=0; row<height; row++, n+=bytes_per_row) {
-			for (m=n; m<n+bytes_per_row; m+=4) {
-			    *(bitmap+m)   = *(image_data+m+3);
-			    *(bitmap+m+1) = *(image_data+m+2);
-			    *(bitmap+m+2) = *(image_data+m+1);
-			    *(bitmap+m+3) = *(image_data+m);
-			}
-		    }
-		}
+	    XCopyArea(display, d, pixmap, gc, x, y, width, height, 0, 0);
+	    context = ((MacDrawable *) pixmap)->context;
+	    if (context) {
+		data = CGBitmapContextGetData(context);
+		bytes_per_line = CGBitmapContextGetBytesPerRow(context);
 	    }
 	}
-	if (bitmap) {
+	if (data) {
 	    imagePtr = XCreateImage(display, NULL, depth, format, offset,
-				    (char*)bitmap, width, height, bitmap_pad, bytes_per_row);
-	    [ns_image removeRepresentation:bitmap_rep]; /*releases the rep*/
-	    [ns_image release];
+		    data, width, height, bitmap_pad, bytes_per_line);
+
+	    /*
+	     * Track Pixmap underlying the XImage in the unused obdata field
+	     * so that we can treat XImages coming from XGetImage specially.
+	     */
+
+	    imagePtr->obdata = (XPointer) pixmap;
+	} else if (pixmap) {
+	    Tk_FreePixmap(display, pixmap);
+	}
+	if (!win) {
+	    XFreeGC(display, gc);
 	}
     } else {
-	TkMacOSXDbgMsg("Could not extract image from drawable.");
+	TkMacOSXDbgMsg("Invalid image format");
     }
     return imagePtr;
 }
@@ -1005,10 +941,12 @@ DestroyImage(
     XImage *image)
 {
     if (image) {
-	if (image->data) {
+	if (image->obdata) {
+	    Tk_FreePixmap((Display*) gMacDisplay, (Pixmap) image->obdata);
+	} else if (image->data) {
 	    ckfree(image->data);
 	}
-	ckfree((char*) image);
+	ckfree(image);
     }
     return 0;
 }
