@@ -35,8 +35,8 @@
 Tcl_Obj *
 TkpGetSystemDefault(
     Tk_Window tkwin,		/* A window to use. */
-    CONST char *dbName,		/* The option database name. */
-    CONST char *className)	/* The name of the option class. */
+    const char *dbName,		/* The option database name. */
+    const char *className)	/* The name of the option class. */
 {
     return NULL;
 }
