@@ -11,21 +11,7 @@
  */
 
 #include "tkInt.h"
-
-/*
-** Bug [3607830]: Before using Xkb, it must be initialized.  TkpOpenDisplay
-**                does this and sets the USE_XKB flag if xkb is supported.
-**                (should this be function ptr?)
-*/
-#ifdef HAVE_XKBKEYCODETOKEYSYM
-#  include <X11/XKBlib.h>
-#else
-#  define XkbKeycodeToKeysym(D,K,G,L) XKeycodeToKeysym(D,K,L)
-#endif
-#define TkKeycodeToKeysym(D,K,G,L)		\
-    ((D)->flags & TK_DISPLAY_USE_XKB) ?		\
-      XkbKeycodeToKeysym((D)->display,K,G,L) :	\
-      XKeycodeToKeysym((D)->display,K,L)
+#include <X11/XKBlib.h>
 
 /*
  * Prototypes for local functions defined in this file:
@@ -37,7 +23,7 @@
  * Tk_SetCaretPos --
  *
  *	This enables correct placement of the XIM caret. This is called by
- *	widgets to indicate their cursor placement.  This is currently only
+ *	widgets to indicate their cursor placement. This is currently only
  *	used for over-the-spot XIM.
  *
  *----------------------------------------------------------------------
@@ -53,11 +39,10 @@ Tk_SetCaretPos(
     TkWindow *winPtr = (TkWindow *) tkwin;
     TkDisplay *dispPtr = winPtr->dispPtr;
 
-    if (   dispPtr->caret.winPtr == winPtr
-	&& dispPtr->caret.x == x
-	&& dispPtr->caret.y == y
-	&& dispPtr->caret.height == height)
-    {
+    if ((dispPtr->caret.winPtr == winPtr)
+	    && (dispPtr->caret.x == x)
+	    && (dispPtr->caret.y == y)
+	    && (dispPtr->caret.height == height)) {
 	return;
     }
 
@@ -66,22 +51,21 @@ Tk_SetCaretPos(
     dispPtr->caret.y = y;
     dispPtr->caret.height = height;
 
-#ifdef TK_USE_INPUT_METHODS
     /*
      * Adjust the XIM caret position.
      */
-    if (   (dispPtr->flags & TK_DISPLAY_USE_IM)
-	&& (dispPtr->inputStyle & XIMPreeditPosition)
-	&& (winPtr->inputContext != NULL) )
-    {
+
+#ifdef TK_USE_INPUT_METHODS
+    if ((dispPtr->flags & TK_DISPLAY_USE_IM)
+	    && (dispPtr->inputStyle & XIMPreeditPosition)
+	    && (winPtr->inputContext != NULL)) {
 	XVaNestedList preedit_attr;
 	XPoint spot;
 
 	spot.x = dispPtr->caret.x;
 	spot.y = dispPtr->caret.y + dispPtr->caret.height;
 	preedit_attr = XVaCreateNestedList(0, XNSpotLocation, &spot, NULL);
-	XSetICValues(winPtr->inputContext,
-		XNPreeditAttributes, preedit_attr,
+	XSetICValues(winPtr->inputContext, XNPreeditAttributes, preedit_attr,
 		NULL);
 	XFree(preedit_attr);
     }
@@ -106,7 +90,7 @@ Tk_SetCaretPos(
  *----------------------------------------------------------------------
  */
 
-char *
+const char *
 TkpGetString(
     TkWindow *winPtr,		/* Window where event occurred */
     XEvent *eventPtr,		/* X keyboard event. */
@@ -130,8 +114,7 @@ TkpGetString(
 #ifdef TK_USE_INPUT_METHODS
     if ((winPtr->dispPtr->flags & TK_DISPLAY_USE_IM)
 	    && (winPtr->inputContext != NULL)
-	    && (eventPtr->type == KeyPress))
-    {
+	    && (eventPtr->type == KeyPress)) {
 	Status status;
 
 #if X_HAVE_UTF8_STRING
@@ -140,7 +123,11 @@ TkpGetString(
 		Tcl_DStringValue(dsPtr), Tcl_DStringLength(dsPtr),
 		&kePtr->keysym, &status);
 
-	if (status == XBufferOverflow) { /* Expand buffer and try again */
+	if (status == XBufferOverflow) {
+	    /*
+	     * Expand buffer and try again.
+	     */
+
 	    Tcl_DStringSetLength(dsPtr, len);
 	    len = Xutf8LookupString(winPtr->inputContext, &eventPtr->xkey,
 		    Tcl_DStringValue(dsPtr), Tcl_DStringLength(dsPtr),
@@ -157,7 +144,6 @@ TkpGetString(
 
 	Tcl_DStringInit(&buf);
 	Tcl_DStringSetLength(&buf, TCL_DSTRING_STATIC_SIZE-1);
-
 	len = XmbLookupString(winPtr->inputContext, &eventPtr->xkey,
 		Tcl_DStringValue(&buf), Tcl_DStringLength(&buf),
                 &kePtr->keysym, &status);
@@ -174,7 +160,6 @@ TkpGetString(
 	if ((status != XLookupChars) && (status != XLookupBoth)) {
 	    len = 0;
 	}
-
 	Tcl_DStringSetLength(&buf, len);
 	Tcl_ExternalToUtfDString(NULL, Tcl_DStringValue(&buf), len, dsPtr);
 	Tcl_DStringFree(&buf);
@@ -218,7 +203,7 @@ TkpGetString(
      * from having to reenter the XIM engine. [Bug 1373712]
      */
 
-    kePtr->charValuePtr = ckalloc((unsigned) len + 1);
+    kePtr->charValuePtr = ckalloc(len + 1);
     kePtr->charValueLen = len;
     memcpy(kePtr->charValuePtr, Tcl_DStringValue(dsPtr), (unsigned) len + 1);
     return Tcl_DStringValue(dsPtr);
@@ -226,7 +211,7 @@ TkpGetString(
 
 /*
  * When mapping from a keysym to a keycode, need information about the
- * modifier state to be used so that when they call TkKeycodeToKeysym taking
+ * modifier state to be used so that when they call XkbKeycodeToKeysym taking
  * into account the xkey.state, they will get back the original keysym.
  */
 
@@ -236,26 +221,27 @@ TkpSetKeycodeAndState(
     KeySym keySym,
     XEvent *eventPtr)
 {
-    TkDisplay *dispPtr = ((TkWindow *) tkwin)->dispPtr;
+    Display *display = Tk_Display(tkwin);
     int state;
     KeyCode keycode;
 
     if (keySym == NoSymbol) {
 	keycode = 0;
     } else {
-	keycode = XKeysymToKeycode(dispPtr->display, keySym);
-    }
-    eventPtr->xkey.keycode = keycode;
-    if (keycode != 0) {
-	for (state = 0; state < 4; state++) {
-	    if (XLookupKeysym(&eventPtr->xkey, state) == keySym) {
-		if (state & 1) {
-		    eventPtr->xkey.state |= ShiftMask;
+	keycode = XKeysymToKeycode(display, keySym);
+	if (keycode != 0) {
+	    for (state = 0; state < 4; state++) {
+		if (XkbKeycodeToKeysym(display, keycode, 0, state) == keySym){
+		    if (state & 1) {
+			eventPtr->xkey.state |= ShiftMask;
+		    }
+		    if (state & 2) {
+			TkDisplay *dispPtr = ((TkWindow *) tkwin)->dispPtr;
+
+			eventPtr->xkey.state |= dispPtr->modeModMask;
+		    }
+		    break;
 		}
-		if (state & 2) {
-		    eventPtr->xkey.state |= dispPtr->modeModMask;
-		}
-		break;
 	    }
 	}
     }
@@ -290,15 +276,6 @@ TkpGetKeySym(
     int index;
     TkKeyEvent* kePtr = (TkKeyEvent*) eventPtr;
 
-    /*
-     * Refresh the mapping information if it's stale. This must happen before
-     * we do any input method processing. [Bug 3599312]
-     */
-
-    if (dispPtr->bindInfoStale) {
-	TkpInitKeymapInfo(dispPtr);
-    }
-
 #ifdef TK_USE_INPUT_METHODS
     /*
      * If input methods are active, we may already have determined a keysym.
@@ -311,7 +288,6 @@ TkpGetKeySym(
 	    Tcl_DString ds;
 	    TkWindow *winPtr = (TkWindow *)
 		Tk_IdToWindow(eventPtr->xany.display, eventPtr->xany.window);
-
 	    Tcl_DStringInit(&ds);
 	    (void) TkpGetString(winPtr, eventPtr, &ds);
 	    Tcl_DStringFree(&ds);
@@ -322,6 +298,14 @@ TkpGetKeySym(
     }
 #endif
 
+    /*
+     * Refresh the mapping information if it's stale
+     */
+
+    if (dispPtr->bindInfoStale) {
+	TkpInitKeymapInfo(dispPtr);
+    }
+
     /*
      * Figure out which of the four slots in the keymap vector to use for this
      * key. Refer to Xlib documentation for more info on how this computation
@@ -337,7 +321,7 @@ TkpGetKeySym(
 	    && (eventPtr->xkey.state & LockMask))) {
 	index += 1;
     }
-    sym = TkKeycodeToKeysym(dispPtr, eventPtr->xkey.keycode, 0,
+    sym = XkbKeycodeToKeysym(dispPtr->display, eventPtr->xkey.keycode, 0,
 	    index);
 
     /*
@@ -352,7 +336,7 @@ TkpGetKeySym(
 		|| ((sym >= XK_Agrave) && (sym <= XK_Odiaeresis))
 		|| ((sym >= XK_Ooblique) && (sym <= XK_Thorn)))) {
 	    index &= ~1;
-	    sym = TkKeycodeToKeysym(dispPtr, eventPtr->xkey.keycode,
+	    sym = XkbKeycodeToKeysym(dispPtr->display, eventPtr->xkey.keycode,
 		    0, index);
 	}
     }
@@ -363,7 +347,7 @@ TkpGetKeySym(
      */
 
     if ((index & 1) && (sym == NoSymbol)) {
-	sym = TkKeycodeToKeysym(dispPtr, eventPtr->xkey.keycode,
+	sym = XkbKeycodeToKeysym(dispPtr->display, eventPtr->xkey.keycode,
 		0, index & ~1);
     }
     return sym;
@@ -413,7 +397,7 @@ TkpInitKeymapInfo(
 	if (*codePtr == 0) {
 	    continue;
 	}
-	keysym = TkKeycodeToKeysym(dispPtr, *codePtr, 0, 0);
+	keysym = XkbKeycodeToKeysym(dispPtr->display, *codePtr, 0, 0);
 	if (keysym == XK_Shift_Lock) {
 	    dispPtr->lockUsage = LU_SHIFT;
 	    break;
@@ -434,12 +418,12 @@ TkpInitKeymapInfo(
     dispPtr->metaModMask = 0;
     dispPtr->altModMask = 0;
     codePtr = modMapPtr->modifiermap;
-    max = 8*modMapPtr->max_keypermod;
+    max = 8 * modMapPtr->max_keypermod;
     for (i = 0; i < max; i++, codePtr++) {
 	if (*codePtr == 0) {
 	    continue;
 	}
-	keysym = TkKeycodeToKeysym(dispPtr, *codePtr, 0, 0);
+	keysym = XkbKeycodeToKeysym(dispPtr->display, *codePtr, 0, 0);
 	if (keysym == XK_Mode_switch) {
 	    dispPtr->modeModMask |= ShiftMask << (i/modMapPtr->max_keypermod);
 	}
@@ -456,12 +440,11 @@ TkpInitKeymapInfo(
      */
 
     if (dispPtr->modKeyCodes != NULL) {
-	ckfree((char *) dispPtr->modKeyCodes);
+	ckfree(dispPtr->modKeyCodes);
     }
     dispPtr->numModKeyCodes = 0;
     arraySize = KEYCODE_ARRAY_SIZE;
-    dispPtr->modKeyCodes = (KeyCode *)
-	    ckalloc((unsigned) (KEYCODE_ARRAY_SIZE * sizeof(KeyCode)));
+    dispPtr->modKeyCodes = ckalloc(KEYCODE_ARRAY_SIZE * sizeof(KeyCode));
     for (i = 0, codePtr = modMapPtr->modifiermap; i < max; i++, codePtr++) {
 	if (*codePtr == 0) {
 	    continue;
@@ -473,23 +456,26 @@ TkpInitKeymapInfo(
 
 	for (j = 0; j < dispPtr->numModKeyCodes; j++) {
 	    if (dispPtr->modKeyCodes[j] == *codePtr) {
+		/*
+		 * 'continue' the outer loop.
+		 */
+
 		goto nextModCode;
 	    }
 	}
 	if (dispPtr->numModKeyCodes >= arraySize) {
-	    KeyCode *new;
+	    KeyCode *newCodes;
 
 	    /*
 	     * Ran out of space in the array; grow it.
 	     */
 
 	    arraySize *= 2;
-	    new = (KeyCode *)
-		    ckalloc((unsigned) (arraySize * sizeof(KeyCode)));
-	    memcpy(new, dispPtr->modKeyCodes,
-		    (dispPtr->numModKeyCodes * sizeof(KeyCode)));
-	    ckfree((char *) dispPtr->modKeyCodes);
-	    dispPtr->modKeyCodes = new;
+	    newCodes = ckalloc(arraySize * sizeof(KeyCode));
+	    memcpy(newCodes, dispPtr->modKeyCodes,
+		    dispPtr->numModKeyCodes * sizeof(KeyCode));
+	    ckfree(dispPtr->modKeyCodes);
+	    dispPtr->modKeyCodes = newCodes;
 	}
 	dispPtr->modKeyCodes[dispPtr->numModKeyCodes] = *codePtr;
 	dispPtr->numModKeyCodes++;
