@@ -68,7 +68,7 @@ TkpGetAppName(
     Tcl_DString *namePtr)	/* A previously initialized Tcl_DString. */
 {
     int argc, namelength;
-    CONST char **argv = NULL, *name, *p;
+    const char **argv = NULL, *name, *p;
 
     name = Tcl_GetVar(interp, "argv0", TCL_GLOBAL_ONLY);
     namelength = -1;
@@ -90,7 +90,7 @@ TkpGetAppName(
     }
     Tcl_DStringAppend(namePtr, name, namelength);
     if (argv != NULL) {
-	ckfree((char *)argv);
+	ckfree(argv);
     }
 }
 
@@ -113,38 +113,35 @@ TkpGetAppName(
 
 void
 TkpDisplayWarning(
-    CONST char *msg,		/* Message to be displayed. */
-    CONST char *title)		/* Title of warning. */
+    const char *msg,		/* Message to be displayed. */
+    const char *title)		/* Title of warning. */
 {
 #define TK_MAX_WARN_LEN 1024
-    WCHAR msgString[TK_MAX_WARN_LEN + 5];
-    WCHAR titleString[TK_MAX_WARN_LEN + 1];
+    WCHAR titleString[TK_MAX_WARN_LEN];
+    WCHAR *msgString; /* points to titleString, just after title, leaving space for ": " */
+    int len; /* size of title, including terminating NULL */
 
-    /* If running on Cygwin and we have a stderr channel, use it. */
-#if !defined(STATIC_BUILD)
-	if (tclStubsPtr->reserved9) {
-	Tcl_Channel errChannel = Tcl_GetStdChannel(TCL_STDERR);
-	if (errChannel) {
-	    Tcl_WriteChars(errChannel, title, -1);
-	    Tcl_WriteChars(errChannel, ": ", 2);
-	    Tcl_WriteChars(errChannel, msg, -1);
-	    Tcl_WriteChars(errChannel, "\n", 1);
-	    return;
-	}
-    }
-#endif /* !STATIC_BUILD */
-
-    MultiByteToWideChar(CP_UTF8, 0, msg, -1, msgString, TK_MAX_WARN_LEN);
-    MultiByteToWideChar(CP_UTF8, 0, title, -1, titleString, TK_MAX_WARN_LEN);
+    len = MultiByteToWideChar(CP_UTF8, 0, title, -1, titleString, TK_MAX_WARN_LEN);
+    msgString = &titleString[len + 1];
+    titleString[TK_MAX_WARN_LEN - 1] = L'\0';
+    MultiByteToWideChar(CP_UTF8, 0, msg, -1, msgString, (TK_MAX_WARN_LEN - 1) - len);
     /*
      * Truncate MessageBox string if it is too long to not overflow the screen
      * and cause possible oversized window error.
      */
-	memcpy(msgString + TK_MAX_WARN_LEN, L" ...", 5 * sizeof(WCHAR));
-    titleString[TK_MAX_WARN_LEN] = L'\0';
-    MessageBoxW(NULL, msgString, titleString,
-	    MB_OK | MB_ICONEXCLAMATION | MB_SYSTEMMODAL
-	    | MB_SETFOREGROUND | MB_TOPMOST);
+    if (titleString[TK_MAX_WARN_LEN - 1] != L'\0') {
+	memcpy(titleString + (TK_MAX_WARN_LEN - 5), L" ...", 5 * sizeof(WCHAR));
+    }
+    if (IsDebuggerPresent()) {
+	titleString[len - 1] = L':';
+	titleString[len] = L' ';
+	OutputDebugStringW(titleString);
+    } else {
+	titleString[len - 1] = L'\0';
+	MessageBoxW(NULL, msgString, titleString,
+		MB_OK | MB_ICONEXCLAMATION | MB_SYSTEMMODAL
+		| MB_SETFOREGROUND | MB_TOPMOST);
+    }
 }
 
 /*
