@@ -14,21 +14,23 @@
 #ifndef _WINPORT
 #define _WINPORT
 
-#include <X11/Xlib.h>
-#include <X11/cursorfont.h>
-#include <X11/keysym.h>
-#include <X11/Xatom.h>
-#include <X11/Xutil.h>
+/*
+ *---------------------------------------------------------------------------
+ * The following sets of #includes and #ifdefs are required to get Tcl to
+ * compile under the windows compilers.
+ *---------------------------------------------------------------------------
+ */
 
-#include <malloc.h>
+#include <wchar.h>
+#include <io.h>
+#include <stdlib.h>
 #include <errno.h>
+#include <fcntl.h>
+#include <malloc.h>
 #include <ctype.h>
 #include <math.h>
-#include <stdlib.h>
 #include <string.h>
 #include <limits.h>
-#include <fcntl.h>
-#include <io.h>
 
 /*
  * Need to block out this include for building extensions with MetroWerks
@@ -61,10 +63,15 @@
     typedef _TCHAR TCHAR;
 #endif
 
+#include <X11/Xlib.h>
+#include <X11/cursorfont.h>
+#include <X11/keysym.h>
+#include <X11/Xatom.h>
+#include <X11/Xutil.h>
 
 #ifndef __GNUC__
-#    define strncasecmp _strnicmp
-#    define strcasecmp _stricmp
+#    define strncasecmp strnicmp
+#    define strcasecmp stricmp
 #endif
 
 #define NBBY 8
@@ -79,13 +86,6 @@
 
 #define REDO_KEYSYM_LOOKUP
 
-/*
- * See ticket [916c1095438eae56]: GetVersionExW triggers warnings
- */
-#if defined(_MSC_VER)
-#   pragma warning(disable:4996)
-#endif
-
 /*
  * The following macro checks to see whether there is buffered
  * input data available for a stdio FILE.
@@ -97,14 +97,6 @@
 #    define TK_READ_DATA_PENDING(f) ((f)->level > 0)
 #endif /* _MSC_VER */
 
-/*
- * The following stubs implement various calls that don't do anything
- * under Windows.
- */
-
-#define TkFreeWindowId(dispPtr,w)
-#define TkInitXId(dispPtr)
-
 /*
  * The following Tk functions are implemented as macros under Windows.
  */
