@@ -106,7 +106,7 @@ LoadXPThemeProcs(HINSTANCE *phlib)
 	 * We have successfully loaded the library. Proceed in storing the
 	 * addresses of the functions we want to use.
 	 */
-	XPThemeProcs *procs = (XPThemeProcs*)ckalloc(sizeof(XPThemeProcs));
+	XPThemeProcs *procs = ckalloc(sizeof(XPThemeProcs));
 #define LOADPROC(name) \
 	(0 != (procs->name = (name ## Proc *)GetProcAddress(handle, #name) ))
 
@@ -124,7 +124,7 @@ LoadXPThemeProcs(HINSTANCE *phlib)
 	    return procs;
 	}
 #undef LOADPROC
-	ckfree((char*)procs);
+	ckfree(procs);
     }
     return 0;
 }
@@ -411,7 +411,7 @@ typedef struct
 static ElementData *
 NewElementData(XPThemeProcs *procs, ElementInfo *info)
 {
-    ElementData *elementData = (ElementData*)ckalloc(sizeof(ElementData));
+    ElementData *elementData = ckalloc(sizeof(ElementData));
 
     elementData->procs = procs;
     elementData->info = info;
@@ -429,10 +429,10 @@ static void DestroyElementData(void *clientData)
 {
     ElementData *elementData = clientData;
     if (elementData->info->flags & HEAP_ELEMENT) {
-	ckfree((char *)elementData->info->statemap);
-	ckfree((char *)elementData->info->className);
-	ckfree((char *)elementData->info->elementName);
-	ckfree((char *)elementData->info);
+	ckfree(elementData->info->statemap);
+	ckfree(elementData->info->className);
+	ckfree(elementData->info->elementName);
+	ckfree(elementData->info);
     }
     ckfree(clientData);
 }
@@ -1062,7 +1062,8 @@ GetSysFlagFromObj(Tcl_Interp *interp, Tcl_Obj *objPtr, int *resultPtr)
     if (Tcl_ListObjGetElements(interp, objPtr, &objc, &objv) != TCL_OK)
 	return TCL_ERROR;
     if (objc != 2) {
-	Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
+	Tcl_SetObjResult(interp, Tcl_NewStringObj("wrong # args", -1));
+	Tcl_SetErrorCode(interp, "TCL", "WRONGARGS", NULL);
 	return TCL_ERROR;
     }
     for (i = 0; i < objc; ++i) {
@@ -1116,8 +1117,9 @@ Ttk_CreateVsapiElement(
 	   O_HALFHEIGHT, O_HALFWIDTH };
 
     if (objc < 2) {
-	Tcl_AppendResult(interp,
-	    "missing required arguments 'class' and/or 'partId'", NULL);
+	Tcl_SetObjResult(interp, Tcl_NewStringObj(
+	    "missing required arguments 'class' and/or 'partId'", -1));
+	Tcl_SetErrorCode(interp, "TTK", "VSAPI", "REQUIRED", NULL);
 	return TCL_ERROR;
     }
 
@@ -1132,8 +1134,10 @@ Ttk_CreateVsapiElement(
 	for (i = 3; i < objc; i += 2) {
 	    int tmp = 0;
 	    if (i == objc -1) {
-		Tcl_AppendResult(interp, "Missing value for \"",
-			Tcl_GetString(objv[i]), "\".", NULL);
+		Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+			"Missing value for \"%s\".",
+			Tcl_GetString(objv[i])));
+		Tcl_SetErrorCode(interp, "TTK", "VSAPI", "MISSING", NULL);
 		return TCL_ERROR;
 	    }
 	    if (Tcl_GetIndexFromObj(interp, objv[i], optionStrings,
@@ -1197,8 +1201,7 @@ Ttk_CreateVsapiElement(
 	if (Tcl_ListObjGetElements(interp, objv[2], &count, &specs) != TCL_OK)
 	    return TCL_ERROR;
 	/* we over-allocate to ensure there is a terminating entry */
-	stateTable = (Ttk_StateTable *)
-		ckalloc(sizeof(Ttk_StateTable) * (count + 1));
+	stateTable = ckalloc(sizeof(Ttk_StateTable) * (count + 1));
 	memset(stateTable, 0, sizeof(Ttk_StateTable) * (count + 1));
 	for (n = 0, j = 0; status == TCL_OK && n < count; n += 2, ++j) {
 	    Ttk_StateSpec spec = {0,0};
@@ -1211,15 +1214,15 @@ Ttk_CreateVsapiElement(
 	    }
 	}
 	if (status != TCL_OK) {
-	    ckfree((char *)stateTable);
+	    ckfree(stateTable);
 	    return status;
 	}
     } else {
-	stateTable = (Ttk_StateTable *)ckalloc(sizeof(Ttk_StateTable));
+	stateTable = ckalloc(sizeof(Ttk_StateTable));
 	memset(stateTable, 0, sizeof(Ttk_StateTable));
     }
 
-    elementPtr = (ElementInfo *)ckalloc(sizeof(ElementInfo));
+    elementPtr = ckalloc(sizeof(ElementInfo));
     elementPtr->elementSpec = elementSpec;
     elementPtr->partId = partId;
     elementPtr->statemap = stateTable;
@@ -1232,7 +1235,7 @@ Ttk_CreateVsapiElement(
     elementPtr->elementName = name;
 
     /* set the class name to an allocated copy */
-    wname = (LPWSTR) ckalloc(sizeof(WCHAR) * (length + 1));
+    wname = ckalloc(sizeof(WCHAR) * (length + 1));
     wcscpy(wname, className);
     elementPtr->className = wname;
 
@@ -1256,10 +1259,10 @@ MODULE_SCOPE int TtkXPTheme_Init(Tcl_Interp *interp, HWND hwnd)
     HINSTANCE hlibrary;
     Ttk_Theme themePtr, parentPtr, vistaPtr;
     ElementInfo *infoPtr;
-    OSVERSIONINFOW os;
+    OSVERSIONINFO os;
 
-    os.dwOSVersionInfoSize = sizeof(OSVERSIONINFOW);
-    GetVersionExW(&os);
+    os.dwOSVersionInfoSize = sizeof(os);
+    GetVersionEx(&os);
 
     procs = LoadXPThemeProcs(&hlibrary);
     if (!procs)
@@ -1279,7 +1282,7 @@ MODULE_SCOPE int TtkXPTheme_Init(Tcl_Interp *interp, HWND hwnd)
      * Set theme data and cleanup proc
      */
 
-    themeData = (XPThemeData *)ckalloc(sizeof(XPThemeData));
+    themeData = ckalloc(sizeof(XPThemeData));
     themeData->procs = procs;
     themeData->hlibrary = hlibrary;
 
